(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
    typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
    (factory((global.FramerMotion = {}),global.React));
}(this, (function (exports,React) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */
    /* global Reflect, Promise */

    var extendStatics = function(d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };

    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    function __rest(s, e) {
        var t = {};
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
            t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
                t[p[i]] = s[p[i]];
        return t;
    }

    /**
     * Uses the ref that is passed in, or creates a new one
     * @param external - External ref
     * @internal
     */
    function useExternalRef(external) {
        // We're conditionally calling `useRef` here which is sort of naughty as hooks
        // shouldn't be called conditionally. However, Framer Motion will break if this
        // condition changes anyway. It might be possible to use an invariant here to
        // make it explicit, but I expect changing `ref` is not normal behaviour.
        var ref = !external || typeof external === "function" ? React.useRef(null) : external;
        React.useEffect(function () {
            if (external && typeof external === "function") {
                external(ref.current);
                return function () { return external(null); };
            }
        }, []);
        return ref;
    }

    var warning = function () { };
    var invariant = function () { };
    {
        warning = function (check, message) {
            if (!check && typeof console !== 'undefined') {
                console.warn(message);
            }
        };
        invariant = function (check, message) {
            if (!check) {
                throw new Error(message);
            }
        };
    }

    var prevTime = 0;
    var onNextFrame = typeof window !== 'undefined' && window.requestAnimationFrame !== undefined
        ? function (callback) { return window.requestAnimationFrame(callback); }
        : function (callback) {
            var timestamp = Date.now();
            var timeToCall = Math.max(0, 16.7 - (timestamp - prevTime));
            prevTime = timestamp + timeToCall;
            setTimeout(function () { return callback(prevTime); }, timeToCall);
        };

    var createStep = (function (setRunNextFrame) {
        var processToRun = [];
        var processToRunNextFrame = [];
        var numThisFrame = 0;
        var isProcessing = false;
        var i = 0;
        var cancelled = new WeakSet();
        var toKeepAlive = new WeakSet();
        var renderStep = {
            cancel: function (process) {
                var indexOfCallback = processToRunNextFrame.indexOf(process);
                cancelled.add(process);
                if (indexOfCallback !== -1) {
                    processToRunNextFrame.splice(indexOfCallback, 1);
                }
            },
            process: function (frame) {
                var _a;
                isProcessing = true;
                _a = [
                    processToRunNextFrame,
                    processToRun
                ], processToRun = _a[0], processToRunNextFrame = _a[1];
                processToRunNextFrame.length = 0;
                numThisFrame = processToRun.length;
                if (numThisFrame) {
                    var process_1;
                    for (i = 0; i < numThisFrame; i++) {
                        process_1 = processToRun[i];
                        process_1(frame);
                        if (toKeepAlive.has(process_1) === true && !cancelled.has(process_1)) {
                            renderStep.schedule(process_1);
                            setRunNextFrame(true);
                        }
                    }
                }
                isProcessing = false;
            },
            schedule: function (process, keepAlive, immediate) {
                if (keepAlive === void 0) { keepAlive = false; }
                if (immediate === void 0) { immediate = false; }
                invariant(typeof process === 'function', 'Argument must be a function');
                var addToCurrentBuffer = immediate && isProcessing;
                var buffer = addToCurrentBuffer ? processToRun : processToRunNextFrame;
                cancelled.delete(process);
                if (keepAlive)
                    toKeepAlive.add(process);
                if (buffer.indexOf(process) === -1) {
                    buffer.push(process);
                    if (addToCurrentBuffer)
                        numThisFrame = processToRun.length;
                }
            }
        };
        return renderStep;
    });

    var StepId;
    (function (StepId) {
        StepId["Read"] = "read";
        StepId["Update"] = "update";
        StepId["Render"] = "render";
        StepId["PostRender"] = "postRender";
        StepId["FixedUpdate"] = "fixedUpdate";
    })(StepId || (StepId = {}));

    var maxElapsed = 40;
    var defaultElapsed = (1 / 60) * 1000;
    var useDefaultElapsed = true;
    var willRunNextFrame = false;
    var isProcessing = false;
    var frame = {
        delta: 0,
        timestamp: 0
    };
    var stepsOrder = [
        StepId.Read,
        StepId.Update,
        StepId.Render,
        StepId.PostRender
    ];
    var setWillRunNextFrame = function (willRun) { return (willRunNextFrame = willRun); };
    var _a = stepsOrder.reduce(function (acc, key) {
        var step = createStep(setWillRunNextFrame);
        acc.sync[key] = function (process, keepAlive, immediate) {
            if (keepAlive === void 0) { keepAlive = false; }
            if (immediate === void 0) { immediate = false; }
            if (!willRunNextFrame)
                startLoop();
            step.schedule(process, keepAlive, immediate);
            return process;
        };
        acc.cancelSync[key] = function (process) { return step.cancel(process); };
        acc.steps[key] = step;
        return acc;
    }, {
        steps: {},
        sync: {},
        cancelSync: {}
    }), steps = _a.steps, sync = _a.sync, cancelSync = _a.cancelSync;
    var processStep = function (stepId) { return steps[stepId].process(frame); };
    var processFrame = function (timestamp) {
        willRunNextFrame = false;
        frame.delta = useDefaultElapsed
            ? defaultElapsed
            : Math.max(Math.min(timestamp - frame.timestamp, maxElapsed), 1);
        if (!useDefaultElapsed)
            defaultElapsed = frame.delta;
        frame.timestamp = timestamp;
        isProcessing = true;
        stepsOrder.forEach(processStep);
        isProcessing = false;
        if (willRunNextFrame) {
            useDefaultElapsed = false;
            onNextFrame(processFrame);
        }
    };
    var startLoop = function () {
        willRunNextFrame = true;
        useDefaultElapsed = true;
        if (!isProcessing)
            onNextFrame(processFrame);
    };
    var getFrameData = function () { return frame; };

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    var __assign$1 = function() {
        __assign$1 = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign$1.apply(this, arguments);
    };

    var clamp = function (min, max) { return function (v) {
        return Math.max(Math.min(v, max), min);
    }; };
    var sanitize = function (v) { return (v % 1 ? Number(v.toFixed(5)) : v); };
    var floatRegex = /(-)?(\d[\d\.]*)/g;
    var colorRegex = /(#[0-9a-f]{6}|#[0-9a-f]{3}|#(?:[0-9a-f]{2}){2,4}|(rgb|hsl)a?\((-?\d+%?[,\s]+){2,3}\s*[\d\.]+%?\))/gi;
    var singleColorRegex = /^(#[0-9a-f]{3}|#(?:[0-9a-f]{2}){2,4}|(rgb|hsl)a?\((-?\d+%?[,\s]+){2,3}\s*[\d\.]+%?\))$/i;

    var number = {
        test: function (v) { return typeof v === 'number'; },
        parse: parseFloat,
        transform: function (v) { return v; }
    };
    var alpha = __assign$1({}, number, { transform: clamp(0, 1) });
    var scale = __assign$1({}, number, { default: 1 });

    var createUnitType = function (unit) { return ({
        test: function (v) {
            return typeof v === 'string' && v.endsWith(unit) && v.split(' ').length === 1;
        },
        parse: parseFloat,
        transform: function (v) { return "" + v + unit; }
    }); };
    var degrees = createUnitType('deg');
    var percent = createUnitType('%');
    var px = createUnitType('px');
    var vh = createUnitType('vh');
    var vw = createUnitType('vw');
    var progressPercentage = __assign$1({}, percent, { parse: function (v) { return percent.parse(v) / 100; }, transform: function (v) { return percent.transform(v * 100); } });

    var getValueFromFunctionString = function (value) {
        return value.substring(value.indexOf('(') + 1, value.lastIndexOf(')'));
    };
    var clampRgbUnit = clamp(0, 255);
    var isRgba = function (v) { return v.red !== undefined; };
    var isHsla = function (v) { return v.hue !== undefined; };
    var splitColorValues = function (terms) {
        return function (v) {
            if (typeof v !== 'string')
                return v;
            var values = {};
            var valuesArray = getValueFromFunctionString(v).split(/,\s*/);
            for (var i = 0; i < 4; i++) {
                values[terms[i]] =
                    valuesArray[i] !== undefined ? parseFloat(valuesArray[i]) : 1;
            }
            return values;
        };
    };
    var rgbaTemplate = function (_a) {
        var red = _a.red, green = _a.green, blue = _a.blue, _b = _a.alpha, alpha$$1 = _b === void 0 ? 1 : _b;
        return "rgba(" + red + ", " + green + ", " + blue + ", " + alpha$$1 + ")";
    };
    var hslaTemplate = function (_a) {
        var hue = _a.hue, saturation = _a.saturation, lightness = _a.lightness, _b = _a.alpha, alpha$$1 = _b === void 0 ? 1 : _b;
        return "hsla(" + hue + ", " + saturation + ", " + lightness + ", " + alpha$$1 + ")";
    };
    var rgbUnit = __assign$1({}, number, { transform: function (v) { return Math.round(clampRgbUnit(v)); } });
    function isColorString(color, colorType) {
        return color.startsWith(colorType) && singleColorRegex.test(color);
    }
    var rgba = {
        test: function (v) { return (typeof v === 'string' ? isColorString(v, 'rgb') : isRgba(v)); },
        parse: splitColorValues(['red', 'green', 'blue', 'alpha']),
        transform: function (_a) {
            var red = _a.red, green = _a.green, blue = _a.blue, alpha$$1 = _a.alpha;
            return rgbaTemplate({
                red: rgbUnit.transform(red),
                green: rgbUnit.transform(green),
                blue: rgbUnit.transform(blue),
                alpha: sanitize(alpha$$1)
            });
        }
    };
    var hsla = {
        test: function (v) { return (typeof v === 'string' ? isColorString(v, 'hsl') : isHsla(v)); },
        parse: splitColorValues(['hue', 'saturation', 'lightness', 'alpha']),
        transform: function (_a) {
            var hue = _a.hue, saturation = _a.saturation, lightness = _a.lightness, alpha$$1 = _a.alpha;
            return hslaTemplate({
                hue: Math.round(hue),
                saturation: percent.transform(sanitize(saturation)),
                lightness: percent.transform(sanitize(lightness)),
                alpha: sanitize(alpha$$1)
            });
        }
    };
    var hex = __assign$1({}, rgba, { test: function (v) { return typeof v === 'string' && isColorString(v, '#'); }, parse: function (v) {
            var r = '';
            var g = '';
            var b = '';
            if (v.length > 4) {
                r = v.substr(1, 2);
                g = v.substr(3, 2);
                b = v.substr(5, 2);
            }
            else {
                r = v.substr(1, 1);
                g = v.substr(2, 1);
                b = v.substr(3, 1);
                r += r;
                g += g;
                b += b;
            }
            return {
                red: parseInt(r, 16),
                green: parseInt(g, 16),
                blue: parseInt(b, 16),
                alpha: 1
            };
        } });
    var color = {
        test: function (v) {
            return (typeof v === 'string' && singleColorRegex.test(v)) ||
                isRgba(v) ||
                isHsla(v);
        },
        parse: function (v) {
            if (rgba.test(v)) {
                return rgba.parse(v);
            }
            else if (hsla.test(v)) {
                return hsla.parse(v);
            }
            else if (hex.test(v)) {
                return hex.parse(v);
            }
            return v;
        },
        transform: function (v) {
            if (isRgba(v)) {
                return rgba.transform(v);
            }
            else if (isHsla(v)) {
                return hsla.transform(v);
            }
            return v;
        }
    };

    var COLOR_TOKEN = '${c}';
    var NUMBER_TOKEN = '${n}';
    var convertNumbersToZero = function (v) {
        return typeof v === 'number' ? 0 : v;
    };
    var complex = {
        test: function (v) {
            if (typeof v !== 'string' || !isNaN(v))
                return false;
            var numValues = 0;
            var foundNumbers = v.match(floatRegex);
            var foundColors = v.match(colorRegex);
            if (foundNumbers)
                numValues += foundNumbers.length;
            if (foundColors)
                numValues += foundColors.length;
            return numValues > 0;
        },
        parse: function (v) {
            var input = v;
            var parsed = [];
            var foundColors = input.match(colorRegex);
            if (foundColors) {
                input = input.replace(colorRegex, COLOR_TOKEN);
                parsed.push.apply(parsed, foundColors.map(color.parse));
            }
            var foundNumbers = input.match(floatRegex);
            if (foundNumbers) {
                parsed.push.apply(parsed, foundNumbers.map(number.parse));
            }
            return parsed;
        },
        createTransformer: function (prop) {
            var template = prop;
            var token = 0;
            var foundColors = prop.match(colorRegex);
            var numColors = foundColors ? foundColors.length : 0;
            if (foundColors) {
                for (var i = 0; i < numColors; i++) {
                    template = template.replace(foundColors[i], COLOR_TOKEN);
                    token++;
                }
            }
            var foundNumbers = template.match(floatRegex);
            var numNumbers = foundNumbers ? foundNumbers.length : 0;
            if (foundNumbers) {
                for (var i = 0; i < numNumbers; i++) {
                    template = template.replace(foundNumbers[i], NUMBER_TOKEN);
                    token++;
                }
            }
            return function (v) {
                var output = template;
                for (var i = 0; i < token; i++) {
                    output = output.replace(i < numColors ? COLOR_TOKEN : NUMBER_TOKEN, i < numColors ? color.transform(v[i]) : sanitize(v[i]));
                }
                return output;
            };
        },
        getAnimatableNone: function (target) {
            var parsedTarget = complex.parse(target);
            var targetTransformer = complex.createTransformer(target);
            return targetTransformer(parsedTarget.map(convertNumbersToZero));
        }
    };

    var DEFAULT_OVERSHOOT_STRENGTH = 1.525;
    var reversed = function (easing) {
        return function (p) {
            return 1 - easing(1 - p);
        };
    };
    var mirrored = function (easing) {
        return function (p) {
            return p <= 0.5 ? easing(2 * p) / 2 : (2 - easing(2 * (1 - p))) / 2;
        };
    };
    var createReversedEasing = reversed;
    var createMirroredEasing = mirrored;
    var createExpoIn = function (power) {
        return function (p) {
            return Math.pow(p, power);
        };
    };
    var createBackIn = function (power) {
        return function (p) {
            return p * p * ((power + 1) * p - power);
        };
    };
    var createAnticipateEasing = function (power) {
        var backEasing = createBackIn(power);
        return function (p) {
            return (p *= 2) < 1 ? 0.5 * backEasing(p) : 0.5 * (2 - Math.pow(2, -10 * (p - 1)));
        };
    };
    var linear = function (p) {
        return p;
    };
    var easeIn = /*#__PURE__*/createExpoIn(2);
    var easeOut = /*#__PURE__*/reversed(easeIn);
    var easeInOut = /*#__PURE__*/mirrored(easeIn);
    var circIn = function (p) {
        return 1 - Math.sin(Math.acos(p));
    };
    var circOut = /*#__PURE__*/reversed(circIn);
    var circInOut = /*#__PURE__*/mirrored(circOut);
    var backIn = /*#__PURE__*/createBackIn(DEFAULT_OVERSHOOT_STRENGTH);
    var backOut = /*#__PURE__*/reversed(backIn);
    var backInOut = /*#__PURE__*/mirrored(backIn);
    var anticipate = /*#__PURE__*/createAnticipateEasing(DEFAULT_OVERSHOOT_STRENGTH);
    var NEWTON_ITERATIONS = 8;
    var NEWTON_MIN_SLOPE = 0.001;
    var SUBDIVISION_PRECISION = 0.0000001;
    var SUBDIVISION_MAX_ITERATIONS = 10;
    var K_SPLINE_TABLE_SIZE = 11;
    var K_SAMPLE_STEP_SIZE = 1.0 / (K_SPLINE_TABLE_SIZE - 1.0);
    var FLOAT_32_SUPPORTED = typeof Float32Array !== 'undefined';
    var a = function (a1, a2) {
        return 1.0 - 3.0 * a2 + 3.0 * a1;
    };
    var b = function (a1, a2) {
        return 3.0 * a2 - 6.0 * a1;
    };
    var c = function (a1) {
        return 3.0 * a1;
    };
    var getSlope = function (t, a1, a2) {
        return 3.0 * a(a1, a2) * t * t + 2.0 * b(a1, a2) * t + c(a1);
    };
    var calcBezier = function (t, a1, a2) {
        return ((a(a1, a2) * t + b(a1, a2)) * t + c(a1)) * t;
    };
    function cubicBezier(mX1, mY1, mX2, mY2) {
        var sampleValues = FLOAT_32_SUPPORTED ? new Float32Array(K_SPLINE_TABLE_SIZE) : new Array(K_SPLINE_TABLE_SIZE);
        var binarySubdivide = function (aX, aA, aB) {
            var i = 0;
            var currentX;
            var currentT;
            do {
                currentT = aA + (aB - aA) / 2.0;
                currentX = calcBezier(currentT, mX1, mX2) - aX;
                if (currentX > 0.0) {
                    aB = currentT;
                } else {
                    aA = currentT;
                }
            } while (Math.abs(currentX) > SUBDIVISION_PRECISION && ++i < SUBDIVISION_MAX_ITERATIONS);
            return currentT;
        };
        var newtonRaphsonIterate = function (aX, aGuessT) {
            var i = 0;
            var currentSlope = 0;
            var currentX;
            for (; i < NEWTON_ITERATIONS; ++i) {
                currentSlope = getSlope(aGuessT, mX1, mX2);
                if (currentSlope === 0.0) {
                    return aGuessT;
                }
                currentX = calcBezier(aGuessT, mX1, mX2) - aX;
                aGuessT -= currentX / currentSlope;
            }
            return aGuessT;
        };
        var calcSampleValues = function () {
            for (var i = 0; i < K_SPLINE_TABLE_SIZE; ++i) {
                sampleValues[i] = calcBezier(i * K_SAMPLE_STEP_SIZE, mX1, mX2);
            }
        };
        var getTForX = function (aX) {
            var intervalStart = 0.0;
            var currentSample = 1;
            var lastSample = K_SPLINE_TABLE_SIZE - 1;
            var dist = 0.0;
            var guessForT = 0.0;
            var initialSlope = 0.0;
            for (; currentSample !== lastSample && sampleValues[currentSample] <= aX; ++currentSample) {
                intervalStart += K_SAMPLE_STEP_SIZE;
            }
            --currentSample;
            dist = (aX - sampleValues[currentSample]) / (sampleValues[currentSample + 1] - sampleValues[currentSample]);
            guessForT = intervalStart + dist * K_SAMPLE_STEP_SIZE;
            initialSlope = getSlope(guessForT, mX1, mX2);
            if (initialSlope >= NEWTON_MIN_SLOPE) {
                return newtonRaphsonIterate(aX, guessForT);
            } else if (initialSlope === 0.0) {
                return guessForT;
            } else {
                return binarySubdivide(aX, intervalStart, intervalStart + K_SAMPLE_STEP_SIZE);
            }
        };
        calcSampleValues();
        var resolver = function (aX) {
            var returnValue;
            if (mX1 === mY1 && mX2 === mY2) {
                returnValue = aX;
            } else if (aX === 0) {
                returnValue = 0;
            } else if (aX === 1) {
                returnValue = 1;
            } else {
                returnValue = calcBezier(getTForX(aX), mY1, mY2);
            }
            return returnValue;
        };
        return resolver;
    }

    var easingLookup = /*#__PURE__*/Object.freeze({
        reversed: reversed,
        mirrored: mirrored,
        createReversedEasing: createReversedEasing,
        createMirroredEasing: createMirroredEasing,
        createExpoIn: createExpoIn,
        createBackIn: createBackIn,
        createAnticipateEasing: createAnticipateEasing,
        linear: linear,
        easeIn: easeIn,
        easeOut: easeOut,
        easeInOut: easeInOut,
        circIn: circIn,
        circOut: circOut,
        circInOut: circInOut,
        backIn: backIn,
        backOut: backOut,
        backInOut: backInOut,
        anticipate: anticipate,
        cubicBezier: cubicBezier
    });

    var zeroPoint = {
        x: 0,
        y: 0,
        z: 0
    };
    var isNum = function (v) { return typeof v === 'number'; };

    var radiansToDegrees = (function (radians) { return (radians * 180) / Math.PI; });

    var angle = (function (a, b) {
        if (b === void 0) { b = zeroPoint; }
        return radiansToDegrees(Math.atan2(b.y - a.y, b.x - a.x));
    });

    var applyOffset = (function (from, to) {
        var hasReceivedFrom = true;
        if (to === undefined) {
            to = from;
            hasReceivedFrom = false;
        }
        return function (v) {
            if (hasReceivedFrom) {
                return v - from + to;
            }
            else {
                from = v;
                hasReceivedFrom = true;
                return to;
            }
        };
    });

    var curryRange = (function (func) { return function (min, max, v) { return (v !== undefined ? func(min, max, v) : function (cv) { return func(min, max, cv); }); }; });

    var clamp$1 = function (min, max, v) {
        return Math.min(Math.max(v, min), max);
    };
    var clamp$1$1 = curryRange(clamp$1);

    var conditional = (function (check, apply) { return function (v) {
        return check(v) ? apply(v) : v;
    }; });

    var degreesToRadians = (function (degrees$$1) { return (degrees$$1 * Math.PI) / 180; });

    var isPoint = (function (point) {
        return point.hasOwnProperty('x') && point.hasOwnProperty('y');
    });

    var isPoint3D = (function (point) {
        return isPoint(point) && point.hasOwnProperty('z');
    });

    var distance1D = function (a, b) { return Math.abs(a - b); };
    var distance = (function (a, b) {
        if (b === void 0) { b = zeroPoint; }
        if (isNum(a) && isNum(b)) {
            return distance1D(a, b);
        }
        else if (isPoint(a) && isPoint(b)) {
            var xDelta = distance1D(a.x, b.x);
            var yDelta = distance1D(a.y, b.y);
            var zDelta = isPoint3D(a) && isPoint3D(b) ? distance1D(a.z, b.z) : 0;
            return Math.sqrt(Math.pow(xDelta, 2) + Math.pow(yDelta, 2) + Math.pow(zDelta, 2));
        }
        return 0;
    });

    var progress = (function (from, to, value) {
        var toFromDifference = to - from;
        return toFromDifference === 0 ? 1 : (value - from) / toFromDifference;
    });

    var mix = (function (from, to, progress) {
        return -progress * from + progress * to + from;
    });

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    var __assign$2 = function() {
        __assign$2 = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign$2.apply(this, arguments);
    };

    var mixLinearColor = function (from, to, v) {
        var fromExpo = from * from;
        var toExpo = to * to;
        return Math.sqrt(Math.max(0, v * (toExpo - fromExpo) + fromExpo));
    };
    var colorTypes = [hex, rgba, hsla];
    var getColorType = function (v) {
        return colorTypes.find(function (type) { return type.test(v); });
    };
    var notAnimatable = function (color$$1) {
        return "'" + color$$1 + "' is not an animatable color. Use the equivalent color code instead.";
    };
    var mixColor = (function (from, to) {
        var fromColorType = getColorType(from);
        var toColorType = getColorType(to);
        invariant(!!fromColorType, notAnimatable(from));
        invariant(!!toColorType, notAnimatable(to));
        invariant(fromColorType.transform === toColorType.transform, 'Both colors must be hex/RGBA, OR both must be HSLA.');
        var fromColor = fromColorType.parse(from);
        var toColor = toColorType.parse(to);
        var blended = __assign$2({}, fromColor);
        var mixFunc = fromColorType === hsla ? mix : mixLinearColor;
        return function (v) {
            for (var key in blended) {
                if (key !== 'alpha') {
                    blended[key] = mixFunc(fromColor[key], toColor[key], v);
                }
            }
            blended.alpha = mix(fromColor.alpha, toColor.alpha, v);
            return fromColorType.transform(blended);
        };
    });

    var combineFunctions = function (a, b) { return function (v) { return b(a(v)); }; };
    var pipe = (function () {
        var transformers = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            transformers[_i] = arguments[_i];
        }
        return transformers.reduce(combineFunctions);
    });

    function getMixer(origin, target) {
        if (isNum(origin)) {
            return function (v) { return mix(origin, target, v); };
        }
        else if (color.test(origin)) {
            return mixColor(origin, target);
        }
        else {
            return mixComplex(origin, target);
        }
    }
    var mixArray = function (from, to) {
        var output = from.slice();
        var numValues = output.length;
        var blendValue = from.map(function (fromThis, i) { return getMixer(fromThis, to[i]); });
        return function (v) {
            for (var i = 0; i < numValues; i++) {
                output[i] = blendValue[i](v);
            }
            return output;
        };
    };
    var mixObject = function (origin, target) {
        var output = __assign$2({}, origin, target);
        var blendValue = {};
        for (var key in output) {
            if (origin[key] !== undefined && target[key] !== undefined) {
                blendValue[key] = getMixer(origin[key], target[key]);
            }
        }
        return function (v) {
            for (var key in blendValue) {
                output[key] = blendValue[key](v);
            }
            return output;
        };
    };
    function analyse(value) {
        var parsed = complex.parse(value);
        var numValues = parsed.length;
        var numNumbers = 0;
        var numRGB = 0;
        var numHSL = 0;
        for (var i = 0; i < numValues; i++) {
            if (numNumbers || typeof parsed[i] === 'number') {
                numNumbers++;
            }
            else {
                if (parsed[i].hue !== undefined) {
                    numHSL++;
                }
                else {
                    numRGB++;
                }
            }
        }
        return { parsed: parsed, numNumbers: numNumbers, numRGB: numRGB, numHSL: numHSL };
    }
    var mixComplex = function (origin, target) {
        var template = complex.createTransformer(target);
        var originStats = analyse(origin);
        var targetStats = analyse(target);
        invariant(originStats.numHSL === targetStats.numHSL &&
            originStats.numRGB === targetStats.numRGB &&
            originStats.numNumbers >= targetStats.numNumbers, "Complex values '" + origin + "' and '" + target + "' too different to mix. Ensure all colors are of the same type.");
        return pipe(mixArray(originStats.parsed, targetStats.parsed), template);
    };

    var mixNumber = function (from, to) { return function (p) { return mix(from, to, p); }; };
    function detectMixerFactory(v) {
        if (typeof v === 'number') {
            return mixNumber;
        }
        else if (typeof v === 'string') {
            if (color.test(v)) {
                return mixColor;
            }
            else {
                return mixComplex;
            }
        }
        else if (Array.isArray(v)) {
            return mixArray;
        }
        else if (typeof v === 'object') {
            return mixObject;
        }
    }
    function createMixers(output, ease, customMixer) {
        var mixers = [];
        var mixerFactory = customMixer || detectMixerFactory(output[0]);
        var numMixers = output.length - 1;
        for (var i = 0; i < numMixers; i++) {
            var mixer = mixerFactory(output[i], output[i + 1]);
            if (ease) {
                var easingFunction = Array.isArray(ease) ? ease[i] : ease;
                mixer = pipe(easingFunction, mixer);
            }
            mixers.push(mixer);
        }
        return mixers;
    }
    function fastInterpolate(_a, _b) {
        var from = _a[0], to = _a[1];
        var mixer = _b[0];
        return function (v) { return mixer(progress(from, to, v)); };
    }
    function slowInterpolate(input, mixers) {
        var inputLength = input.length;
        var lastInputIndex = inputLength - 1;
        return function (v) {
            var mixerIndex = 0;
            var foundMixerIndex = false;
            if (v <= input[0]) {
                foundMixerIndex = true;
            }
            else if (v >= input[lastInputIndex]) {
                mixerIndex = lastInputIndex - 1;
                foundMixerIndex = true;
            }
            if (!foundMixerIndex) {
                var i = 1;
                for (; i < inputLength; i++) {
                    if (input[i] > v || i === lastInputIndex) {
                        break;
                    }
                }
                mixerIndex = i - 1;
            }
            var progressInRange = progress(input[mixerIndex], input[mixerIndex + 1], v);
            return mixers[mixerIndex](progressInRange);
        };
    }
    function interpolate$$1(input, output, _a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.clamp, clamp = _c === void 0 ? true : _c, ease = _b.ease, mixer = _b.mixer;
        var inputLength = input.length;
        invariant(inputLength === output.length, 'Both input and output ranges must be the same length');
        invariant(!ease || !Array.isArray(ease) || ease.length === inputLength - 1, 'Array of easing functions must be of length `input.length - 1`, as it applies to the transitions **between** the defined values.');
        if (input[0] > input[inputLength - 1]) {
            input = [].concat(input);
            output = [].concat(output);
            input.reverse();
            output.reverse();
        }
        var mixers = createMixers(output, ease, mixer);
        var interpolator = inputLength === 2
            ? fastInterpolate(input, mixers)
            : slowInterpolate(input, mixers);
        return clamp
            ? pipe(clamp$1$1(input[0], input[inputLength - 1]), interpolator)
            : interpolator;
    }

    var pointFromVector = (function (origin, angle, distance) {
        angle = degreesToRadians(angle);
        return {
            x: distance * Math.cos(angle) + origin.x,
            y: distance * Math.sin(angle) + origin.y
        };
    });

    var toDecimal = (function (num, precision) {
        if (precision === void 0) { precision = 2; }
        precision = Math.pow(10, precision);
        return Math.round(num * precision) / precision;
    });

    var smoothFrame = (function (prevValue, nextValue, duration, smoothing) {
        if (smoothing === void 0) { smoothing = 0; }
        return toDecimal(prevValue +
            (duration * (nextValue - prevValue)) / Math.max(smoothing, duration));
    });

    var smooth = (function (strength) {
        if (strength === void 0) { strength = 50; }
        var previousValue = 0;
        var lastUpdated = 0;
        return function (v) {
            var currentFramestamp = getFrameData().timestamp;
            var timeDelta = currentFramestamp !== lastUpdated ? currentFramestamp - lastUpdated : 0;
            var newValue = timeDelta
                ? smoothFrame(previousValue, v, timeDelta, strength)
                : previousValue;
            lastUpdated = currentFramestamp;
            previousValue = newValue;
            return newValue;
        };
    });

    var snap = (function (points) {
        if (typeof points === 'number') {
            return function (v) { return Math.round(v / points) * points; };
        }
        else {
            var i_1 = 0;
            var numPoints_1 = points.length;
            return function (v) {
                var lastDistance = Math.abs(points[0] - v);
                for (i_1 = 1; i_1 < numPoints_1; i_1++) {
                    var point = points[i_1];
                    var distance = Math.abs(point - v);
                    if (distance === 0)
                        return point;
                    if (distance > lastDistance)
                        return points[i_1 - 1];
                    if (i_1 === numPoints_1 - 1)
                        return point;
                    lastDistance = distance;
                }
            };
        }
    });

    var identity = function (v) { return v; };
    var springForce = function (alterDisplacement) {
        if (alterDisplacement === void 0) { alterDisplacement = identity; }
        return curryRange(function (constant, origin, v) {
            var displacement = origin - v;
            var springModifiedDisplacement = -(0 - constant + 1) * (0 - alterDisplacement(Math.abs(displacement)));
            return displacement <= 0
                ? origin + springModifiedDisplacement
                : origin - springModifiedDisplacement;
        });
    };
    var springForceLinear = springForce();
    var springForceExpo = springForce(Math.sqrt);

    var velocityPerFrame = (function (xps, frameDuration) {
        return isNum(xps) ? xps / (1000 / frameDuration) : 0;
    });

    var velocityPerSecond = (function (velocity, frameDuration) {
        return frameDuration ? velocity * (1000 / frameDuration) : 0;
    });

    var wrap = function (min, max, v) {
        var rangeSize = max - min;
        return ((((v - min) % rangeSize) + rangeSize) % rangeSize) + min;
    };
    var wrap$1 = curryRange(wrap);

    var clampProgress = clamp$1$1(0, 1);

    var isFloat = function (value) {
        return !isNaN(parseFloat(value));
    };
    /**
     * `MotionValue` is used to track the state and velocity of motion values.
     *
     * @public
     */
    var MotionValue = /** @class */ (function () {
        /**
         * @param init - The initiating value
         * @param config - Optional configuration options
         *
         * -  `transformer`: A function to transform incoming values with.
         *
         * @internal
         */
        function MotionValue(init, _a) {
            var _b = _a === void 0 ? {} : _a, transformer = _b.transformer, parent = _b.parent;
            var _this = this;
            /**
             * Duration, in milliseconds, since last updating frame.
             *
             * @internal
             */
            this.timeDelta = 0;
            /**
             * Timestamp of the last time this `MotionValue` was updated.
             *
             * @internal
             */
            this.lastUpdated = 0;
            /**
             * Tracks whether this value can output a velocity. Currently this is only true
             * if the value is numerical, but we might be able to widen the scope here and support
             * other value types.
             *
             * @internal
             */
            this.canTrackVelocity = false;
            /**
             * Notify a subscriber with the latest value.
             *
             * This is an instanced and bound function to prevent generating a new
             * function once per frame.
             *
             * @param subscriber - The subscriber to notify.
             *
             * @internal
             */
            this.notifySubscriber = function (subscriber) {
                subscriber(_this.current);
            };
            /**
             * Schedule a velocity check for the next frame.
             *
             * This is an instanced and bound function to prevent generating a new
             * function once per frame.
             *
             * @internal
             */
            this.scheduleVelocityCheck = function () { return sync.postRender(_this.velocityCheck); };
            /**
             * Updates `prev` with `current` if the value hasn't been updated this frame.
             * This ensures velocity calculations return `0`.
             *
             * This is an instanced and bound function to prevent generating a new
             * function once per frame.
             *
             * @internal
             */
            this.velocityCheck = function (_a) {
                var timestamp = _a.timestamp;
                if (timestamp !== _this.lastUpdated) {
                    _this.prev = _this.current;
                }
            };
            /**
             * Updates child `MotionValue`.
             *
             * @param child - Child `MotionValue`.
             *
             * @internal
             */
            this.setChild = function (child) { return child.set(_this.current); };
            this.parent = parent;
            this.transformer = transformer;
            this.set(init, false);
            this.canTrackVelocity = isFloat(this.current);
        }
        /**
         * Creates a new `MotionValue` that's subscribed to the output of this one.
         *
         * @param config - Optional configuration options
         *
         * -  `transformer`: A function to transform incoming values with.
         *
         * @internal
         */
        MotionValue.prototype.addChild = function (config) {
            var child = new MotionValue(this.current, __assign({ parent: this }, config));
            if (!this.children)
                this.children = new Set();
            this.children.add(child);
            return child;
        };
        /**
         * Stops a `MotionValue` from being subscribed to this one.
         *
         * @param child - The subscribed `MotionValue`
         *
         * @internal
         */
        MotionValue.prototype.removeChild = function (child) {
            if (!this.children) {
                return;
            }
            this.children.delete(child);
        };
        /**
         * Subscribes a subscriber function to a subscription list.
         *
         * @param subscriptions - A `Set` of subscribers.
         * @param subscription - A subscriber function.
         */
        MotionValue.prototype.subscribeTo = function (subscriptions, subscription) {
            var _this = this;
            var updateSubscriber = function () { return subscription(_this.current); };
            subscriptions.add(updateSubscriber);
            return function () { return subscriptions.delete(updateSubscriber); };
        };
        /**
         * Adds a function that will be notified when the `MotionValue` is updated.
         *
         * It returns a function that, when called, will cancel the subscription.
         *
         * When calling `onChange` inside a React component, it should be wrapped with the
         * `useEffect` hook. As it returns an unsubscribe function, this should be returned
         * from the `useEffect` function to ensure only one active subscription persists
         * for the lifecycle of the component.
         *
         * ```jsx
         * const MyComponent = () => {
         *   const x = useMotionValue(0)
         *
         *   useEffect(() => {
         *     const unsubscribe = x.onChange((latestX) => {
         *       // Do stuff with latest x value
         *     })
         *
         *     return unsubscribe
         *   })
         *
         *   return <motion.div style={{ x }} />
         * }
         * ```
         *
         * @internalremarks
         *
         * We could look into a `useOnChange` hook if the above lifecycle management proves confusing.
         *
         * ```jsx
         * useOnChange(x, () => {})
         * ```
         *
         * @param subscriber - A function that receives the latest value.
         * @returns A function that, when called, will cancel this subscription.
         *
         * @public
         */
        MotionValue.prototype.onChange = function (subscription) {
            if (!this.updateSubscribers)
                this.updateSubscribers = new Set();
            return this.subscribeTo(this.updateSubscribers, subscription);
        };
        /**
         * Adds a function that will be notified when the `MotionValue` requests a render.
         *
         * @param subscriber - A function that's provided the latest value.
         * @returns A function that, when called, will cancel this subscription.
         *
         * @internal
         */
        MotionValue.prototype.onRenderRequest = function (subscription) {
            if (!this.renderSubscribers)
                this.renderSubscribers = new Set();
            // Render immediately
            this.notifySubscriber(subscription);
            return this.subscribeTo(this.renderSubscribers, subscription);
        };
        /**
         * Sets the state of the `MotionValue`.
         *
         * @remarks
         *
         * ```jsx
         * const x = useMotionValue(0)
         * x.set(10)
         * ```
         *
         * @param latest - Latest value to set.
         * @param render - Whether to notify render subscribers. Defaults to `true`
         *
         * @public
         */
        MotionValue.prototype.set = function (v, render) {
            if (render === void 0) { render = true; }
            this.prev = this.current;
            this.current = this.transformer ? this.transformer(v) : v;
            if (this.updateSubscribers && this.prev !== this.current) {
                this.updateSubscribers.forEach(this.notifySubscriber);
            }
            if (this.children) {
                this.children.forEach(this.setChild);
            }
            if (render && this.renderSubscribers) {
                this.renderSubscribers.forEach(this.notifySubscriber);
            }
            // Update timestamp
            var _a = getFrameData(), delta = _a.delta, timestamp = _a.timestamp;
            if (this.lastUpdated !== timestamp) {
                this.timeDelta = delta;
                this.lastUpdated = timestamp;
                sync.postRender(this.scheduleVelocityCheck);
            }
        };
        /**
         * Returns the latest state of `MotionValue`
         *
         * @returns - The latest state of `MotionValue`
         *
         * @public
         */
        MotionValue.prototype.get = function () {
            return this.current;
        };
        /**
         * Returns the latest velocity of `MotionValue`
         *
         * @returns - The latest velocity of `MotionValue`. Returns `0` if the state is non-numerical.
         *
         * @public
         */
        MotionValue.prototype.getVelocity = function () {
            // This could be isFloat(this.prev) && isFloat(this.current), but that would be wasteful
            return this.canTrackVelocity
                ? // These casts could be avoided if parseFloat would be typed better
                    velocityPerSecond(parseFloat(this.current) -
                        parseFloat(this.prev), this.timeDelta)
                : 0;
        };
        /**
         * Registers a new animation to control this `MotionValue`. Only one
         * animation can drive a `MotionValue` at one time.
         *
         * ```jsx
         * value.start()
         * ```
         *
         * @param animation - A function that starts the provided animation
         *
         * @internal
         */
        MotionValue.prototype.start = function (animation) {
            var _this = this;
            this.stop();
            return new Promise(function (resolve) {
                _this.stopAnimation = animation(resolve);
            }).then(function () { return _this.clearAnimation(); });
        };
        /**
         * Stop the currently controlling animation.
         *
         * @public
         */
        MotionValue.prototype.stop = function () {
            if (this.stopAnimation)
                this.stopAnimation();
            this.clearAnimation();
        };
        /**
         * Returns `true` if this value is currently animating.
         *
         * @public
         */
        MotionValue.prototype.isAnimating = function () {
            return !!this.stopAnimation;
        };
        MotionValue.prototype.clearAnimation = function () {
            this.stopAnimation = null;
        };
        /**
         * Destroy and clean up this `MotionValue`.
         *
         * @public
         */
        MotionValue.prototype.destroy = function () {
            this.updateSubscribers && this.updateSubscribers.clear();
            this.renderSubscribers && this.renderSubscribers.clear();
            this.parent && this.parent.removeChild(this);
            this.stop();
        };
        return MotionValue;
    }());
    /**
     * @internal
     */
    function motionValue(init, opts) {
        return new MotionValue(init, opts);
    }

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    var __assign$3 = function () {
        __assign$3 = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign$3.apply(this, arguments);
    };

    function __rest$1(s, e) {
        var t = {};
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0) t[p[i]] = s[p[i]];
        return t;
    }

    var createStyler = function (_a) {
        var onRead = _a.onRead,
            onRender = _a.onRender,
            _b = _a.uncachedValues,
            uncachedValues = _b === void 0 ? new Set() : _b,
            _c = _a.useCache,
            useCache = _c === void 0 ? true : _c;
        return function (_a) {
            if (_a === void 0) {
                _a = {};
            }
            var props = __rest$1(_a, []);
            var state = {};
            var changedValues = [];
            var hasChanged = false;
            function setValue(key, value) {
                if (key.startsWith('--')) {
                    props.hasCSSVariable = true;
                }
                var currentValue = state[key];
                state[key] = value;
                if (state[key] === currentValue) return;
                if (changedValues.indexOf(key) === -1) {
                    changedValues.push(key);
                }
                if (!hasChanged) {
                    hasChanged = true;
                    sync.render(styler.render);
                }
            }
            var styler = {
                get: function (key, forceRead) {
                    if (forceRead === void 0) {
                        forceRead = false;
                    }
                    var useCached = !forceRead && useCache && !uncachedValues.has(key) && state[key] !== undefined;
                    return useCached ? state[key] : onRead(key, props);
                },
                set: function (values, value) {
                    if (typeof values === 'string') {
                        setValue(values, value);
                    } else {
                        for (var key in values) {
                            setValue(key, values[key]);
                        }
                    }
                    return this;
                },
                render: function (forceRender) {
                    if (forceRender === void 0) {
                        forceRender = false;
                    }
                    if (hasChanged || forceRender === true) {
                        onRender(state, props, changedValues);
                        hasChanged = false;
                        changedValues.length = 0;
                    }
                    return this;
                }
            };
            return styler;
        };
    };

    var CAMEL_CASE_PATTERN = /([a-z])([A-Z])/g;
    var REPLACE_TEMPLATE = '$1-$2';
    var camelToDash = function (str) {
        return str.replace(CAMEL_CASE_PATTERN, REPLACE_TEMPLATE).toLowerCase();
    };
    var setDomAttrs = function (element, attrs) {
        for (var key in attrs) {
            if (attrs.hasOwnProperty(key)) {
                element.setAttribute(key, attrs[key]);
            }
        }
    };

    var camelCache = /*#__PURE__*/new Map();
    var dashCache = /*#__PURE__*/new Map();
    var prefixes = ['Webkit', 'Moz', 'O', 'ms', ''];
    var numPrefixes = prefixes.length;
    var isBrowser = typeof document !== 'undefined';
    var testElement;
    var setDashPrefix = function (key, prefixed) {
        return dashCache.set(key, camelToDash(prefixed));
    };
    var testPrefix = function (key) {
        testElement = testElement || document.createElement('div');
        for (var i = 0; i < numPrefixes; i++) {
            var prefix = prefixes[i];
            var noPrefix = prefix === '';
            var prefixedPropertyName = noPrefix ? key : prefix + key.charAt(0).toUpperCase() + key.slice(1);
            if (prefixedPropertyName in testElement.style || noPrefix) {
                camelCache.set(key, prefixedPropertyName);
                setDashPrefix(key, "" + (noPrefix ? '' : '-') + camelToDash(prefixedPropertyName));
            }
        }
    };
    var setServerProperty = function (key) {
        return setDashPrefix(key, key);
    };
    var prefixer = function (key, asDashCase) {
        if (asDashCase === void 0) {
            asDashCase = false;
        }
        var cache = asDashCase ? dashCache : camelCache;
        if (!cache.has(key)) isBrowser ? testPrefix(key) : setServerProperty(key);
        return cache.get(key) || key;
    };

    var axes = ['', 'X', 'Y', 'Z'];
    var order = ['scale', 'rotate', 'skew', 'transformPerspective'];
    var transformProps = /*#__PURE__*/order.reduce(function (acc, key) {
        return axes.reduce(function (axesAcc, axesKey) {
            axesAcc.push(key + axesKey);
            return axesAcc;
        }, acc);
    }, ['x', 'y', 'z']);
    var transformPropDictionary = /*#__PURE__*/transformProps.reduce(function (dict, key) {
        dict[key] = true;
        return dict;
    }, {});
    function isTransformProp(key) {
        return transformPropDictionary[key] === true;
    }
    function sortTransformProps(a, b) {
        return transformProps.indexOf(a) - transformProps.indexOf(b);
    }
    var transformOriginProps = /*#__PURE__*/new Set(['originX', 'originY', 'originZ']);
    function isTransformOriginProp(key) {
        return transformOriginProps.has(key);
    }

    var rounded = /*#__PURE__*/__assign$3({}, number, { transform: Math.round });
    var valueTypes = {
        color: color,
        backgroundColor: color,
        outlineColor: color,
        fill: color,
        stroke: color,
        borderColor: color,
        borderTopColor: color,
        borderRightColor: color,
        borderBottomColor: color,
        borderLeftColor: color,
        borderWidth: px,
        borderTopWidth: px,
        borderRightWidth: px,
        borderBottomWidth: px,
        borderLeftWidth: px,
        borderRadius: px,
        radius: px,
        borderTopLeftRadius: px,
        borderTopRightRadius: px,
        borderBottomRightRadius: px,
        borderBottomLeftRadius: px,
        width: px,
        maxWidth: px,
        height: px,
        maxHeight: px,
        size: px,
        top: px,
        right: px,
        bottom: px,
        left: px,
        padding: px,
        paddingTop: px,
        paddingRight: px,
        paddingBottom: px,
        paddingLeft: px,
        margin: px,
        marginTop: px,
        marginRight: px,
        marginBottom: px,
        marginLeft: px,
        rotate: degrees,
        rotateX: degrees,
        rotateY: degrees,
        rotateZ: degrees,
        scale: scale,
        scaleX: scale,
        scaleY: scale,
        scaleZ: scale,
        skew: degrees,
        skewX: degrees,
        skewY: degrees,
        distance: px,
        x: px,
        y: px,
        z: px,
        perspective: px,
        opacity: alpha,
        originX: progressPercentage,
        originY: progressPercentage,
        originZ: px,
        zIndex: rounded
    };
    var getValueType = function (key) {
        return valueTypes[key];
    };
    var getValueAsType = function (value, type) {
        return type && typeof value === 'number' ? type.transform(value) : value;
    };

    var SCROLL_LEFT = 'scrollLeft';
    var SCROLL_TOP = 'scrollTop';
    var scrollKeys = /*#__PURE__*/new Set([SCROLL_LEFT, SCROLL_TOP]);

    var blacklist = /*#__PURE__*/new Set([SCROLL_LEFT, SCROLL_TOP, 'transform']);
    var translateAlias = {
        x: 'translateX',
        y: 'translateY',
        z: 'translateZ'
    };
    function isCustomTemplate(v) {
        return typeof v === 'function';
    }
    function buildTransform(state, transform, transformKeys, transformIsDefault, enableHardwareAcceleration) {
        var transformString = '';
        var transformHasZ = false;
        transformKeys.sort(sortTransformProps);
        var numTransformKeys = transformKeys.length;
        for (var i = 0; i < numTransformKeys; i++) {
            var key = transformKeys[i];
            transformString += (translateAlias[key] || key) + "(" + transform[key] + ") ";
            transformHasZ = key === 'z' ? true : transformHasZ;
        }
        if (!transformHasZ && enableHardwareAcceleration) {
            transformString += 'translateZ(0)';
        } else {
            transformString = transformString.trim();
        }
        if (isCustomTemplate(state.transform)) {
            transformString = state.transform(transform, transformString);
        } else if (transformIsDefault) {
            transformString = 'none';
        }
        return transformString;
    }
    function buildStyleProperty(state, enableHardwareAcceleration, styles, transform, transformOrigin, transformKeys, isDashCase) {
        if (enableHardwareAcceleration === void 0) {
            enableHardwareAcceleration = true;
        }
        if (styles === void 0) {
            styles = {};
        }
        if (transform === void 0) {
            transform = {};
        }
        if (transformOrigin === void 0) {
            transformOrigin = {};
        }
        if (transformKeys === void 0) {
            transformKeys = [];
        }
        if (isDashCase === void 0) {
            isDashCase = false;
        }
        var transformIsDefault = true;
        var hasTransform = false;
        var hasTransformOrigin = false;
        for (var key in state) {
            var value = state[key];
            var valueType = getValueType(key);
            var valueAsType = getValueAsType(value, valueType);
            if (isTransformProp(key)) {
                hasTransform = true;
                transform[key] = valueAsType;
                transformKeys.push(key);
                if (transformIsDefault) {
                    if (valueType.default && value !== valueType.default || !valueType.default && value !== 0) {
                        transformIsDefault = false;
                    }
                }
            } else if (isTransformOriginProp(key)) {
                transformOrigin[key] = valueAsType;
                hasTransformOrigin = true;
            } else if (!blacklist.has(key) || !isCustomTemplate(valueAsType)) {
                styles[prefixer(key, isDashCase)] = valueAsType;
            }
        }
        if (hasTransform || typeof state.transform === 'function') {
            styles.transform = buildTransform(state, transform, transformKeys, transformIsDefault, enableHardwareAcceleration);
        }
        if (hasTransformOrigin) {
            styles.transformOrigin = (transformOrigin.originX || 0) + " " + (transformOrigin.originY || 0) + " " + (transformOrigin.originZ || 0);
        }
        return styles;
    }
    function createStyleBuilder(enableHardwareAcceleration) {
        if (enableHardwareAcceleration === void 0) {
            enableHardwareAcceleration = true;
        }
        var styles = {};
        var transform = {};
        var transformOrigin = {};
        var transformKeys = [];
        return function (state) {
            transformKeys.length = 0;
            buildStyleProperty(state, enableHardwareAcceleration, styles, transform, transformOrigin, transformKeys, true);
            return styles;
        };
    }

    function onRead(key, options) {
        var element = options.element,
            preparseOutput = options.preparseOutput;
        var defaultValueType = getValueType(key);
        if (isTransformProp(key)) {
            return defaultValueType ? defaultValueType.default || 0 : 0;
        } else if (scrollKeys.has(key)) {
            return element[key];
        } else {
            var domValue = window.getComputedStyle(element, null).getPropertyValue(prefixer(key, true)) || 0;
            return preparseOutput && defaultValueType && defaultValueType.test(domValue) && defaultValueType.parse ? defaultValueType.parse(domValue) : domValue;
        }
    }
    function onRender(state, _a, changedValues) {
        var element = _a.element,
            buildStyles = _a.buildStyles,
            hasCSSVariable = _a.hasCSSVariable;
        Object.assign(element.style, buildStyles(state));
        if (hasCSSVariable) {
            var numChangedValues = changedValues.length;
            for (var i = 0; i < numChangedValues; i++) {
                var key = changedValues[i];
                if (key.startsWith('--')) {
                    element.style.setProperty(key, state[key]);
                }
            }
        }
        if (changedValues.indexOf(SCROLL_LEFT) !== -1) {
            element[SCROLL_LEFT] = state[SCROLL_LEFT];
        }
        if (changedValues.indexOf(SCROLL_TOP) !== -1) {
            element[SCROLL_TOP] = state[SCROLL_TOP];
        }
    }
    var cssStyler = /*#__PURE__*/createStyler({
        onRead: onRead,
        onRender: onRender,
        uncachedValues: scrollKeys
    });
    function createCssStyler(element, _a) {
        if (_a === void 0) {
            _a = {};
        }
        var enableHardwareAcceleration = _a.enableHardwareAcceleration,
            props = __rest$1(_a, ["enableHardwareAcceleration"]);
        return cssStyler(__assign$3({ element: element, buildStyles: createStyleBuilder(enableHardwareAcceleration), preparseOutput: true }, props));
    }

    var camelCaseAttributes = /*#__PURE__*/new Set(['baseFrequency', 'diffuseConstant', 'kernelMatrix', 'kernelUnitLength', 'keySplines', 'keyTimes', 'limitingConeAngle', 'markerHeight', 'markerWidth', 'numOctaves', 'targetX', 'targetY', 'surfaceScale', 'specularConstant', 'specularExponent', 'stdDeviation', 'tableValues']);

    var ZERO_NOT_ZERO = 0.0000001;
    var progressToPixels = function (progress, length) {
        return progress * length + 'px';
    };
    var build = function (state, dimensions, isPath, pathLength) {
        var hasTransform = false;
        var hasDashArray = false;
        var props = {};
        var dashArrayStyles = isPath ? {
            pathLength: '0',
            pathSpacing: "" + pathLength
        } : undefined;
        var scale$$1 = state.scale !== undefined ? state.scale || ZERO_NOT_ZERO : state.scaleX || 1;
        var scaleY = state.scaleY !== undefined ? state.scaleY || ZERO_NOT_ZERO : scale$$1 || 1;
        var transformOriginX = dimensions.width * (state.originX || 50) + dimensions.x;
        var transformOriginY = dimensions.height * (state.originY || 50) + dimensions.y;
        var scaleTransformX = -transformOriginX * (scale$$1 * 1);
        var scaleTransformY = -transformOriginY * (scaleY * 1);
        var scaleReplaceX = transformOriginX / scale$$1;
        var scaleReplaceY = transformOriginY / scaleY;
        var transform = {
            translate: "translate(" + state.x + ", " + state.y + ") ",
            scale: "translate(" + scaleTransformX + ", " + scaleTransformY + ") scale(" + scale$$1 + ", " + scaleY + ") translate(" + scaleReplaceX + ", " + scaleReplaceY + ") ",
            rotate: "rotate(" + state.rotate + ", " + transformOriginX + ", " + transformOriginY + ") ",
            skewX: "skewX(" + state.skewX + ") ",
            skewY: "skewY(" + state.skewY + ") "
        };
        for (var key in state) {
            if (state.hasOwnProperty(key)) {
                var value = state[key];
                if (isTransformProp(key)) {
                    hasTransform = true;
                } else if (isPath && (key === 'pathLength' || key === 'pathSpacing') && typeof value === 'number') {
                    hasDashArray = true;
                    dashArrayStyles[key] = progressToPixels(value, pathLength);
                } else if (isPath && key === 'pathOffset') {
                    props['stroke-dashoffset'] = progressToPixels(-value, pathLength);
                } else {
                    var attrKey = !camelCaseAttributes.has(key) ? camelToDash(key) : key;
                    props[attrKey] = value;
                }
            }
        }
        if (hasDashArray) {
            props['stroke-dasharray'] = dashArrayStyles.pathLength + ' ' + dashArrayStyles.pathSpacing;
        }
        if (hasTransform) {
            props.transform = '';
            for (var key in transform) {
                if (transform.hasOwnProperty(key)) {
                    var defaultValue = key === 'scale' ? '1' : '0';
                    props.transform += transform[key].replace(/undefined/g, defaultValue);
                }
            }
        }
        return props;
    };

    var int = /*#__PURE__*/__assign$3({}, number, { transform: Math.round });
    var valueTypes$1 = {
        fill: color,
        stroke: color,
        scale: scale,
        scaleX: scale,
        scaleY: scale,
        opacity: alpha,
        fillOpacity: alpha,
        strokeOpacity: alpha,
        numOctaves: int
    };
    var getValueType$1 = function (key) {
        return valueTypes$1[key];
    };

    var getDimensions = function (element) {
        return typeof element.getBBox === 'function' ? element.getBBox() : element.getBoundingClientRect();
    };
    var getSVGElementDimensions = function (element) {
        try {
            return getDimensions(element);
        } catch (e) {
            return { x: 0, y: 0, width: 0, height: 0 };
        }
    };

    var svgStyler = /*#__PURE__*/createStyler({
        onRead: function (key, _a) {
            var element = _a.element;
            if (!isTransformProp(key)) {
                return element.getAttribute(key);
            } else {
                var valueType = getValueType$1(key);
                return valueType ? valueType.default : 0;
            }
        },
        onRender: function (state, _a) {
            var dimensions = _a.dimensions,
                element = _a.element,
                isPath = _a.isPath,
                pathLength = _a.pathLength;
            setDomAttrs(element, build(state, dimensions, isPath, pathLength));
        }
    });
    var svg = function (element) {
        var dimensions = getSVGElementDimensions(element);
        var props = {
            element: element,
            dimensions: dimensions,
            isPath: false
        };
        if (element.tagName === 'path') {
            props.isPath = true;
            props.pathLength = element.getTotalLength();
        }
        return svgStyler(props);
    };

    var viewport = /*#__PURE__*/createStyler({
        useCache: false,
        onRead: function (key) {
            return key === 'scrollTop' ? window.pageYOffset : window.pageXOffset;
        },
        onRender: function (_a) {
            var _b = _a.scrollTop,
                scrollTop = _b === void 0 ? 0 : _b,
                _c = _a.scrollLeft,
                scrollLeft = _c === void 0 ? 0 : _c;
            return window.scrollTo(scrollLeft, scrollTop);
        }
    });

    var cache = /*#__PURE__*/new WeakMap();
    var createDOMStyler = function (node, props) {
        var styler;
        if (node instanceof HTMLElement) {
            styler = createCssStyler(node, props);
        } else if (node instanceof SVGElement) {
            styler = svg(node);
        } else if (node === window) {
            styler = viewport(node);
        }
        invariant(styler !== undefined, 'No valid node provided. Node must be HTMLElement, SVGElement or window.');
        cache.set(node, styler);
        return styler;
    };
    var getStyler = function (node, props) {
        return cache.has(node) ? cache.get(node) : createDOMStyler(node, props);
    };
    function index(nodeOrSelector, props) {
        var node = typeof nodeOrSelector === 'string' ? document.querySelector(nodeOrSelector) : nodeOrSelector;
        return getStyler(node, props);
    }

    /**
     * Creates a constant value over the lifecycle of a component.
     *
     * Even if `useMemo` is provided an empty array as its final argument, it doesn't offer
     * a guarantee that it won't re-run for performance reasons later on. By using `useConstant`
     * you can ensure that initialisers don't execute twice or more.
     */
    function useConstant(init) {
        var ref = React.useRef(null);
        if (ref.current === null) {
            ref.current = init();
        }
        return ref.current;
    }

    // Creating a styler factory for the `onUpdate` prop allows all values
    // to fire and the `onUpdate` prop will only fire once per frame
    var updateStyler = createStyler({
        onRead: function () { return null; },
        onRender: function (state, _a) {
            var onUpdate = _a.onUpdate;
            return onUpdate(state);
        },
    });
    var MotionValuesMap = /** @class */ (function () {
        function MotionValuesMap() {
            this.hasMounted = false;
            this.values = new Map();
            this.unsubscribers = new Map();
        }
        MotionValuesMap.prototype.has = function (key) {
            return this.values.has(key);
        };
        MotionValuesMap.prototype.set = function (key, value) {
            this.values.set(key, value);
            if (this.hasMounted) {
                this.bindValueToOutput(key, value);
            }
        };
        MotionValuesMap.prototype.get = function (key, defaultValue) {
            var value = this.values.get(key);
            if (value === undefined && defaultValue !== undefined) {
                value = new MotionValue(defaultValue);
                this.set(key, value);
            }
            return value;
        };
        MotionValuesMap.prototype.forEach = function (callback) {
            return this.values.forEach(callback);
        };
        MotionValuesMap.prototype.bindValueToOutput = function (key, value) {
            var _this = this;
            var onRender = function (v) { return _this.output && _this.output(key, v); };
            var unsubscribeOnRender = value.onRenderRequest(onRender);
            var onChange = function (v) {
                _this.onUpdate && _this.onUpdate.set(key, v);
            };
            var unsubscribeOnChange = value.onChange(onChange);
            this.unsubscribers.set(key, function () {
                unsubscribeOnRender();
                unsubscribeOnChange();
            });
        };
        MotionValuesMap.prototype.setOnUpdate = function (onUpdate) {
            this.onUpdate = undefined;
            if (onUpdate) {
                this.onUpdate = updateStyler({ onUpdate: onUpdate });
            }
        };
        MotionValuesMap.prototype.setTransformTemplate = function (transformTemplate) {
            if (this.transformTemplate !== transformTemplate) {
                this.transformTemplate = transformTemplate;
                this.updateTransformTemplate();
            }
        };
        MotionValuesMap.prototype.getTransformTemplate = function () {
            return this.transformTemplate;
        };
        MotionValuesMap.prototype.updateTransformTemplate = function () {
            if (this.output) {
                this.output("transform", this.transformTemplate);
            }
        };
        MotionValuesMap.prototype.mount = function (output) {
            var _this = this;
            this.hasMounted = true;
            if (output)
                this.output = output;
            this.values.forEach(function (value, key) { return _this.bindValueToOutput(key, value); });
            this.updateTransformTemplate();
        };
        MotionValuesMap.prototype.unmount = function () {
            var _this = this;
            this.values.forEach(function (_value, key) {
                var unsubscribe = _this.unsubscribers.get(key);
                unsubscribe && unsubscribe();
            });
        };
        return MotionValuesMap;
    }());
    var useMotionValues = function (_a) {
        var onUpdate = _a.onUpdate, transformTemplate = _a.transformTemplate;
        var motionValues = useConstant(function () { return new MotionValuesMap(); });
        motionValues.setOnUpdate(onUpdate);
        motionValues.setTransformTemplate(transformTemplate);
        return motionValues;
    };
    /**
     * `useEffect` gets resolved bottom-up. We defer some optional functionality to child
     * components, so to ensure everything runs correctly we export the ref-binding logic
     * to a new component rather than in `useMotionValues`.
     */
    var MountMotionValuesComponent = function (_a, ref) {
        var values = _a.values, isStatic = _a.isStatic;
        React.useEffect(function () {
            invariant(ref.current instanceof Element, "No `ref` found. Ensure components created with `motion.custom` forward refs using `React.forwardRef`");
            var domStyler = index(ref.current, {
                preparseOutput: false,
                enableHardwareAcceleration: !isStatic,
            });
            values.mount(function (key, value) { return domStyler.set(key, value); });
            return function () { return values.unmount(); };
        }, []);
        return null;
    };
    var MountMotionValues = React.memo(React.forwardRef(MountMotionValuesComponent));

    var createValueResolver = function (resolver) { return function (values) {
        var resolvedValues = {};
        values.forEach(function (value, key) { return (resolvedValues[key] = resolver(value)); });
        return resolvedValues;
    }; };
    var resolveCurrent = createValueResolver(function (value) { return value.get(); });

    var transformOriginProps$1 = new Set(["originX", "originY"]);
    var isTransformOriginProp$1 = function (key) { return transformOriginProps$1.has(key); };
    var isMotionValue = function (value) {
        return value instanceof MotionValue;
    };
    var buildStyleAttr = function (values, styleProp, isStatic) {
        var motionValueStyles = resolveCurrent(values);
        var transformTemplate = values.getTransformTemplate();
        if (transformTemplate) {
            // If `transform` has been manually set as a string, pass that through the template
            // otherwise pass it forward to Stylefire's style property builder
            motionValueStyles.transform = styleProp.transform
                ? transformTemplate({}, styleProp.transform)
                : transformTemplate;
        }
        return __assign({}, styleProp, buildStyleProperty(motionValueStyles, !isStatic));
    };
    var useMotionStyles = function (values, styleProp, transformValues) {
        if (styleProp === void 0) { styleProp = {}; }
        var style = React.useRef({}).current;
        var prevMotionStyles = React.useRef({}).current;
        var currentStyleKeys = new Set(Object.keys(style));
        for (var key in styleProp) {
            currentStyleKeys.delete(key);
            var thisStyle = styleProp[key];
            if (isMotionValue(thisStyle)) {
                // If this is a motion value, add it to our MotionValuesMap
                values.set(key, thisStyle);
            }
            else if (isTransformProp(key) || isTransformOriginProp$1(key)) {
                // Or if it's a transform prop, create a motion value (or update an existing one)
                // to ensure Stylefire can reconcile all the transform values together.
                if (!values.has(key)) {
                    // If it doesn't exist as a motion value, create it
                    values.set(key, motionValue(thisStyle));
                }
                else {
                    // Otherwise only update it if it's changed from a previous render
                    if (thisStyle !== prevMotionStyles[key]) {
                        var value = values.get(key);
                        value.set(thisStyle);
                    }
                }
                prevMotionStyles[key] = thisStyle;
            }
            else {
                style[key] = thisStyle;
            }
        }
        currentStyleKeys.forEach(function (key) { return delete style[key]; });
        return transformValues ? transformValues(style) : style;
    };

    var isKeyframesTarget = function (v) {
        return Array.isArray(v);
    };

    var isCustomValue = function (v) {
        return Boolean(v && typeof v === "object" && v.mix && v.toValue);
    };
    var resolveFinalValueInKeyframes = function (v) {
        // TODO maybe throw if v.length - 1 is placeholder token?
        return isKeyframesTarget(v) ? v[v.length - 1] || 0 : v;
    };

    var auto = {
        test: function (v) { return v === "auto"; },
        parse: function (v) { return v; },
    };
    var dimensionTypes = [number, px, percent, degrees, vw, vh, auto];
    var valueTypes$2 = dimensionTypes.concat([color, complex]);
    var testValueType = function (v) { return function (type) { return type.test(v); }; };
    var getDimensionValueType = function (v) {
        return dimensionTypes.find(testValueType(v));
    };
    var getValueType$2 = function (v) { return valueTypes$2.find(testValueType(v)); };

    var Chainable = /*#__PURE__*/function () {
        function Chainable(props) {
            if (props === void 0) {
                props = {};
            }
            this.props = props;
        }
        Chainable.prototype.applyMiddleware = function (middleware) {
            return this.create(__assign({}, this.props, { middleware: this.props.middleware ? [middleware].concat(this.props.middleware) : [middleware] }));
        };
        Chainable.prototype.pipe = function () {
            var funcs = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                funcs[_i] = arguments[_i];
            }
            var pipedUpdate = funcs.length === 1 ? funcs[0] : pipe.apply(void 0, funcs);
            return this.applyMiddleware(function (update) {
                return function (v) {
                    return update(pipedUpdate(v));
                };
            });
        };
        Chainable.prototype.while = function (predicate) {
            return this.applyMiddleware(function (update, complete) {
                return function (v) {
                    return predicate(v) ? update(v) : complete();
                };
            });
        };
        Chainable.prototype.filter = function (predicate) {
            return this.applyMiddleware(function (update) {
                return function (v) {
                    return predicate(v) && update(v);
                };
            });
        };
        return Chainable;
    }();

    var Observer = /*#__PURE__*/function () {
        function Observer(_a, observer) {
            var middleware = _a.middleware,
                onComplete = _a.onComplete;
            var _this = this;
            this.isActive = true;
            this.update = function (v) {
                if (_this.observer.update) _this.updateObserver(v);
            };
            this.complete = function () {
                if (_this.observer.complete && _this.isActive) _this.observer.complete();
                if (_this.onComplete) _this.onComplete();
                _this.isActive = false;
            };
            this.error = function (err) {
                if (_this.observer.error && _this.isActive) _this.observer.error(err);
                _this.isActive = false;
            };
            this.observer = observer;
            this.updateObserver = function (v) {
                return observer.update(v);
            };
            this.onComplete = onComplete;
            if (observer.update && middleware && middleware.length) {
                middleware.forEach(function (m) {
                    return _this.updateObserver = m(_this.updateObserver, _this.complete);
                });
            }
        }
        return Observer;
    }();
    var createObserver = function (observerCandidate, _a, onComplete) {
        var middleware = _a.middleware;
        if (typeof observerCandidate === 'function') {
            return new Observer({ middleware: middleware, onComplete: onComplete }, { update: observerCandidate });
        } else {
            return new Observer({ middleware: middleware, onComplete: onComplete }, observerCandidate);
        }
    };

    var Action = /*#__PURE__*/function (_super) {
        __extends(Action, _super);
        function Action() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Action.prototype.create = function (props) {
            return new Action(props);
        };
        Action.prototype.start = function (observerCandidate) {
            if (observerCandidate === void 0) {
                observerCandidate = {};
            }
            var isComplete = false;
            var subscription = {
                stop: function () {
                    return undefined;
                }
            };
            var _a = this.props,
                init = _a.init,
                observerProps = __rest(_a, ["init"]);
            var observer = createObserver(observerCandidate, observerProps, function () {
                isComplete = true;
                subscription.stop();
            });
            var api = init(observer);
            subscription = api ? __assign({}, subscription, api) : subscription;
            if (observerCandidate.registerParent) {
                observerCandidate.registerParent(subscription);
            }
            if (isComplete) subscription.stop();
            return subscription;
        };
        return Action;
    }(Chainable);
    var action = function (init) {
        return new Action({ init: init });
    };

    var BaseMulticast = /*#__PURE__*/function (_super) {
        __extends(BaseMulticast, _super);
        function BaseMulticast() {
            var _this = _super !== null && _super.apply(this, arguments) || this;
            _this.subscribers = [];
            return _this;
        }
        BaseMulticast.prototype.complete = function () {
            this.subscribers.forEach(function (subscriber) {
                return subscriber.complete();
            });
        };
        BaseMulticast.prototype.error = function (err) {
            this.subscribers.forEach(function (subscriber) {
                return subscriber.error(err);
            });
        };
        BaseMulticast.prototype.update = function (v) {
            for (var i = 0; i < this.subscribers.length; i++) {
                this.subscribers[i].update(v);
            }
        };
        BaseMulticast.prototype.subscribe = function (observerCandidate) {
            var _this = this;
            var observer = createObserver(observerCandidate, this.props);
            this.subscribers.push(observer);
            var subscription = {
                unsubscribe: function () {
                    var index$$1 = _this.subscribers.indexOf(observer);
                    if (index$$1 !== -1) _this.subscribers.splice(index$$1, 1);
                }
            };
            return subscription;
        };
        BaseMulticast.prototype.stop = function () {
            if (this.parent) this.parent.stop();
        };
        BaseMulticast.prototype.registerParent = function (subscription) {
            this.stop();
            this.parent = subscription;
        };
        return BaseMulticast;
    }(Chainable);

    var Multicast = /*#__PURE__*/function (_super) {
        __extends(Multicast, _super);
        function Multicast() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        Multicast.prototype.create = function (props) {
            return new Multicast(props);
        };
        return Multicast;
    }(BaseMulticast);

    var stepProgress = function (steps, progress$$1) {
        var segment = 1 / (steps - 1);
        var subsegment = 1 / (2 * (steps - 1));
        var percentProgressOfTarget = Math.min(progress$$1, 1);
        var subsegmentProgressOfTarget = percentProgressOfTarget / subsegment;
        var segmentProgressOfTarget = Math.floor((subsegmentProgressOfTarget + 1) / 2);
        return segmentProgressOfTarget * segment;
    };

    var calc = /*#__PURE__*/Object.freeze({
        angle: angle,
        degreesToRadians: degreesToRadians,
        distance: distance,
        isPoint3D: isPoint3D,
        isPoint: isPoint,
        dilate: mix,
        getValueFromProgress: mix,
        pointFromAngleAndDistance: pointFromVector,
        getProgressFromValue: progress,
        radiansToDegrees: radiansToDegrees,
        smooth: smoothFrame,
        speedPerFrame: velocityPerFrame,
        speedPerSecond: velocityPerSecond,
        stepProgress: stepProgress
    });

    var isValueList = function (v) {
        return Array.isArray(v);
    };
    var isSingleValue = function (v) {
        var typeOfV = typeof v;
        return typeOfV === 'string' || typeOfV === 'number';
    };
    var ValueReaction = /*#__PURE__*/function (_super) {
        __extends(ValueReaction, _super);
        function ValueReaction(props) {
            var _this = _super.call(this, props) || this;
            _this.scheduleVelocityCheck = function () {
                return sync.postRender(_this.velocityCheck);
            };
            _this.velocityCheck = function (_a) {
                var timestamp = _a.timestamp;
                if (timestamp !== _this.lastUpdated) {
                    _this.prev = _this.current;
                }
            };
            _this.prev = _this.current = props.value || 0;
            if (isSingleValue(_this.current)) {
                _this.updateCurrent = function (v) {
                    return _this.current = v;
                };
                _this.getVelocityOfCurrent = function () {
                    return _this.getSingleVelocity(_this.current, _this.prev);
                };
            } else if (isValueList(_this.current)) {
                _this.updateCurrent = function (v) {
                    return _this.current = v.slice();
                };
                _this.getVelocityOfCurrent = function () {
                    return _this.getListVelocity();
                };
            } else {
                _this.updateCurrent = function (v) {
                    _this.current = {};
                    for (var key in v) {
                        if (v.hasOwnProperty(key)) {
                            _this.current[key] = v[key];
                        }
                    }
                };
                _this.getVelocityOfCurrent = function () {
                    return _this.getMapVelocity();
                };
            }
            if (props.initialSubscription) _this.subscribe(props.initialSubscription);
            return _this;
        }
        ValueReaction.prototype.create = function (props) {
            return new ValueReaction(props);
        };
        ValueReaction.prototype.get = function () {
            return this.current;
        };
        ValueReaction.prototype.getVelocity = function () {
            return this.getVelocityOfCurrent();
        };
        ValueReaction.prototype.update = function (v) {
            _super.prototype.update.call(this, v);
            this.prev = this.current;
            this.updateCurrent(v);
            var _a = getFrameData(),
                delta = _a.delta,
                timestamp = _a.timestamp;
            this.timeDelta = delta;
            this.lastUpdated = timestamp;
            sync.postRender(this.scheduleVelocityCheck);
        };
        ValueReaction.prototype.subscribe = function (observerCandidate) {
            var sub = _super.prototype.subscribe.call(this, observerCandidate);
            this.subscribers[this.subscribers.length - 1].update(this.current);
            return sub;
        };
        ValueReaction.prototype.getSingleVelocity = function (current, prev) {
            return typeof current === 'number' && typeof prev === 'number' ? velocityPerSecond(current - prev, this.timeDelta) : velocityPerSecond(parseFloat(current) - parseFloat(prev), this.timeDelta) || 0;
        };
        ValueReaction.prototype.getListVelocity = function () {
            var _this = this;
            return this.current.map(function (c, i) {
                return _this.getSingleVelocity(c, _this.prev[i]);
            });
        };
        ValueReaction.prototype.getMapVelocity = function () {
            var velocity = {};
            for (var key in this.current) {
                if (this.current.hasOwnProperty(key)) {
                    velocity[key] = this.getSingleVelocity(this.current[key], this.prev[key]);
                }
            }
            return velocity;
        };
        return ValueReaction;
    }(BaseMulticast);
    var value = function (value, initialSubscription) {
        return new ValueReaction({ value: value, initialSubscription: initialSubscription });
    };

    var multi = function (_a) {
        var getCount = _a.getCount,
            getFirst = _a.getFirst,
            getOutput = _a.getOutput,
            mapApi = _a.mapApi,
            setProp = _a.setProp,
            startActions = _a.startActions;
        return function (actions) {
            return action(function (_a) {
                var update = _a.update,
                    complete = _a.complete,
                    error = _a.error;
                var numActions = getCount(actions);
                var output = getOutput();
                var updateOutput = function () {
                    return update(output);
                };
                var numCompletedActions = 0;
                var subs = startActions(actions, function (a, name) {
                    var hasCompleted = false;
                    return a.start({
                        complete: function () {
                            if (!hasCompleted) {
                                hasCompleted = true;
                                numCompletedActions++;
                                if (numCompletedActions === numActions) sync.update(complete);
                            }
                        },
                        error: error,
                        update: function (v) {
                            setProp(output, name, v);
                            sync.update(updateOutput, false, true);
                        }
                    });
                });
                return Object.keys(getFirst(subs)).reduce(function (api, methodName) {
                    api[methodName] = mapApi(subs, methodName);
                    return api;
                }, {});
            });
        };
    };

    var composite = /*#__PURE__*/multi({
        getOutput: function () {
            return {};
        },
        getCount: function (subs) {
            return Object.keys(subs).length;
        },
        getFirst: function (subs) {
            return subs[Object.keys(subs)[0]];
        },
        mapApi: function (subs, methodName) {
            return function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return Object.keys(subs).reduce(function (output, propKey) {
                    var _a;
                    if (subs[propKey][methodName]) {
                        args[0] && args[0][propKey] !== undefined ? output[propKey] = subs[propKey][methodName](args[0][propKey]) : output[propKey] = (_a = subs[propKey])[methodName].apply(_a, args);
                    }
                    return output;
                }, {});
            };
        },
        setProp: function (output, name, v) {
            return output[name] = v;
        },
        startActions: function (actions, starter) {
            return Object.keys(actions).reduce(function (subs, key) {
                subs[key] = starter(actions[key], key);
                return subs;
            }, {});
        }
    });

    var parallel = /*#__PURE__*/multi({
        getOutput: function () {
            return [];
        },
        getCount: function (subs) {
            return subs.length;
        },
        getFirst: function (subs) {
            return subs[0];
        },
        mapApi: function (subs, methodName) {
            return function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                return subs.map(function (sub, i) {
                    if (sub[methodName]) {
                        return Array.isArray(args[0]) ? sub[methodName](args[0][i]) : sub[methodName].apply(sub, args);
                    }
                });
            };
        },
        setProp: function (output, name, v) {
            return output[name] = v;
        },
        startActions: function (actions, starter) {
            return actions.map(function (action, i) {
                return starter(action, i);
            });
        }
    });
    var parallel$1 = function () {
        var actions = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            actions[_i] = arguments[_i];
        }
        return parallel(actions);
    };

    var createVectorTests = function (typeTests) {
        var testNames = Object.keys(typeTests);
        var isVectorProp = function (prop, key) {
            return prop !== undefined && !typeTests[key](prop);
        };
        var getVectorKeys = function (props) {
            return testNames.reduce(function (vectorKeys, key) {
                if (isVectorProp(props[key], key)) vectorKeys.push(key);
                return vectorKeys;
            }, []);
        };
        var testVectorProps = function (props) {
            return props && testNames.some(function (key) {
                return isVectorProp(props[key], key);
            });
        };
        return { getVectorKeys: getVectorKeys, testVectorProps: testVectorProps };
    };
    var unitTypes = [px, percent, degrees, vh, vw];
    var findUnitType = function (prop) {
        return unitTypes.find(function (type) {
            return type.test(prop);
        });
    };
    var isUnitProp = function (prop) {
        return Boolean(findUnitType(prop));
    };
    var createAction = function (action, props) {
        return action(props);
    };
    var reduceArrayValue = function (i) {
        return function (props, key) {
            props[key] = props[key][i];
            return props;
        };
    };
    var createArrayAction = function (action, props, vectorKeys) {
        var firstVectorKey = vectorKeys[0];
        var actionList = props[firstVectorKey].map(function (v, i) {
            var childActionProps = vectorKeys.reduce(reduceArrayValue(i), __assign({}, props));
            return getActionCreator(v)(action, childActionProps);
        });
        return parallel$1.apply(void 0, actionList);
    };
    var reduceObjectValue = function (key) {
        return function (props, propKey) {
            props[propKey] = props[propKey][key];
            return props;
        };
    };
    var createObjectAction = function (action, props, vectorKeys) {
        var firstVectorKey = vectorKeys[0];
        var actionMap = Object.keys(props[firstVectorKey]).reduce(function (map, key) {
            var childActionProps = vectorKeys.reduce(reduceObjectValue(key), __assign({}, props));
            map[key] = getActionCreator(props[firstVectorKey][key])(action, childActionProps);
            return map;
        }, {});
        return composite(actionMap);
    };
    var createUnitAction = function (action, _a) {
        var from = _a.from,
            to = _a.to,
            props = __rest(_a, ["from", "to"]);
        var unitType = findUnitType(from) || findUnitType(to);
        var transform = unitType.transform,
            parse = unitType.parse;
        return action(__assign({}, props, { from: typeof from === 'string' ? parse(from) : from, to: typeof to === 'string' ? parse(to) : to })).pipe(transform);
    };
    var createMixerAction = function (mixer) {
        return function (action, _a) {
            var from = _a.from,
                to = _a.to,
                props = __rest(_a, ["from", "to"]);
            return action(__assign({}, props, { from: 0, to: 1 })).pipe(mixer(from, to));
        };
    };
    var createColorAction = /*#__PURE__*/createMixerAction(mixColor);
    var createComplexAction = /*#__PURE__*/createMixerAction(mixComplex);
    var createVectorAction = function (action, typeTests) {
        var _a = createVectorTests(typeTests),
            testVectorProps = _a.testVectorProps,
            getVectorKeys = _a.getVectorKeys;
        var vectorAction = function (props) {
            var isVector = testVectorProps(props);
            if (!isVector) return action(props);
            var vectorKeys = getVectorKeys(props);
            var testKey = vectorKeys[0];
            var testProp = props[testKey];
            return getActionCreator(testProp)(action, props, vectorKeys);
        };
        return vectorAction;
    };
    var getActionCreator = function (prop) {
        if (typeof prop === 'number') {
            return createAction;
        } else if (Array.isArray(prop)) {
            return createArrayAction;
        } else if (isUnitProp(prop)) {
            return createUnitAction;
        } else if (color.test(prop)) {
            return createColorAction;
        } else if (complex.test(prop)) {
            return createComplexAction;
        } else if (typeof prop === 'object') {
            return createObjectAction;
        }
    };

    var decay = function (props) {
        if (props === void 0) {
            props = {};
        }
        return action(function (_a) {
            var complete = _a.complete,
                update = _a.update;
            var _b = props.velocity,
                velocity = _b === void 0 ? 0 : _b,
                _c = props.from,
                from = _c === void 0 ? 0 : _c,
                _d = props.power,
                power = _d === void 0 ? 0.8 : _d,
                _e = props.timeConstant,
                timeConstant = _e === void 0 ? 350 : _e,
                _f = props.restDelta,
                restDelta = _f === void 0 ? 0.5 : _f,
                modifyTarget = props.modifyTarget;
            var elapsed = 0;
            var amplitude = power * velocity;
            var idealTarget = Math.round(from + amplitude);
            var target = typeof modifyTarget === 'undefined' ? idealTarget : modifyTarget(idealTarget);
            var process = sync.update(function (_a) {
                var frameDelta = _a.delta;
                elapsed += frameDelta;
                var delta = -amplitude * Math.exp(-elapsed / timeConstant);
                var isMoving = delta > restDelta || delta < -restDelta;
                var current = isMoving ? target + delta : target;
                update(current);
                if (!isMoving) {
                    cancelSync.update(process);
                    complete();
                }
            }, true);
            return {
                stop: function () {
                    return cancelSync.update(process);
                }
            };
        });
    };
    var vectorDecay = /*#__PURE__*/createVectorAction(decay, {
        from: number.test,
        modifyTarget: function (func) {
            return typeof func === 'function';
        },
        velocity: number.test
    });

    var spring = function (props) {
        if (props === void 0) {
            props = {};
        }
        return action(function (_a) {
            var update = _a.update,
                complete = _a.complete;
            var _b = props.velocity,
                velocity = _b === void 0 ? 0.0 : _b;
            var _c = props.from,
                from = _c === void 0 ? 0.0 : _c,
                _d = props.to,
                to = _d === void 0 ? 0.0 : _d,
                _e = props.stiffness,
                stiffness = _e === void 0 ? 100 : _e,
                _f = props.damping,
                damping = _f === void 0 ? 10 : _f,
                _g = props.mass,
                mass = _g === void 0 ? 1.0 : _g,
                _h = props.restSpeed,
                restSpeed = _h === void 0 ? 0.01 : _h,
                _j = props.restDelta,
                restDelta = _j === void 0 ? 0.01 : _j;
            var initialVelocity = velocity ? -(velocity / 1000) : 0.0;
            var t = 0;
            var delta = to - from;
            var position = from;
            var prevPosition = position;
            var process = sync.update(function (_a) {
                var timeDelta = _a.delta;
                t += timeDelta;
                var dampingRatio = damping / (2 * Math.sqrt(stiffness * mass));
                var angularFreq = Math.sqrt(stiffness / mass) / 1000;
                prevPosition = position;
                if (dampingRatio < 1) {
                    var envelope = Math.exp(-dampingRatio * angularFreq * t);
                    var expoDecay = angularFreq * Math.sqrt(1.0 - dampingRatio * dampingRatio);
                    position = to - envelope * ((initialVelocity + dampingRatio * angularFreq * delta) / expoDecay * Math.sin(expoDecay * t) + delta * Math.cos(expoDecay * t));
                } else {
                    var envelope = Math.exp(-angularFreq * t);
                    position = to - envelope * (delta + (initialVelocity + angularFreq * delta) * t);
                }
                velocity = velocityPerSecond(position - prevPosition, timeDelta);
                var isBelowVelocityThreshold = Math.abs(velocity) <= restSpeed;
                var isBelowDisplacementThreshold = Math.abs(to - position) <= restDelta;
                if (isBelowVelocityThreshold && isBelowDisplacementThreshold) {
                    position = to;
                    update(position);
                    cancelSync.update(process);
                    complete();
                } else {
                    update(position);
                }
            }, true);
            return {
                stop: function () {
                    return cancelSync.update(process);
                }
            };
        });
    };
    var vectorSpring = /*#__PURE__*/createVectorAction(spring, {
        from: number.test,
        to: number.test,
        stiffness: number.test,
        damping: number.test,
        mass: number.test,
        velocity: number.test
    });

    var inertia = function (_a) {
        var _b = _a.from,
            from = _b === void 0 ? 0 : _b,
            _c = _a.velocity,
            velocity = _c === void 0 ? 0 : _c,
            min = _a.min,
            max = _a.max,
            _d = _a.power,
            power = _d === void 0 ? 0.8 : _d,
            _e = _a.timeConstant,
            timeConstant = _e === void 0 ? 700 : _e,
            _f = _a.bounceStiffness,
            bounceStiffness = _f === void 0 ? 500 : _f,
            _g = _a.bounceDamping,
            bounceDamping = _g === void 0 ? 10 : _g,
            _h = _a.restDelta,
            restDelta = _h === void 0 ? 1 : _h,
            modifyTarget = _a.modifyTarget;
        return action(function (_a) {
            var update = _a.update,
                complete = _a.complete;
            var current = value(from);
            var activeAnimation;
            var isSpring = false;
            var isLessThanMin = function (v) {
                return min !== undefined && v <= min;
            };
            var isMoreThanMax = function (v) {
                return max !== undefined && v >= max;
            };
            var isOutOfBounds = function (v) {
                return isLessThanMin(v) || isMoreThanMax(v);
            };
            var isTravellingAwayFromBounds = function (v, currentVelocity) {
                return isLessThanMin(v) && currentVelocity < 0 || isMoreThanMax(v) && currentVelocity > 0;
            };
            var startAnimation = function (animation, next) {
                activeAnimation && activeAnimation.stop();
                activeAnimation = animation.start({
                    update: function (v) {
                        return current.update(v);
                    },
                    complete: function () {
                        if (next) {
                            next();
                            return;
                        }
                        complete();
                    }
                });
            };
            var startSpring = function (props) {
                isSpring = true;
                startAnimation(vectorSpring(__assign({}, props, { to: isLessThanMin(props.from) ? min : max, stiffness: bounceStiffness, damping: bounceDamping, restDelta: restDelta })));
            };
            current.subscribe(function (v) {
                update(v);
                var currentVelocity = current.getVelocity();
                if (activeAnimation && !isSpring && isTravellingAwayFromBounds(v, currentVelocity)) {
                    startSpring({ from: v, velocity: currentVelocity });
                }
            });
            if (isOutOfBounds(from)) {
                startSpring({ from: from, velocity: velocity });
            } else if (velocity !== 0) {
                var animation = vectorDecay({
                    from: from,
                    velocity: velocity,
                    timeConstant: timeConstant,
                    power: power,
                    restDelta: isOutOfBounds(from) ? 20 : restDelta,
                    modifyTarget: modifyTarget
                });
                startAnimation(animation, function () {
                    var v = current.get();
                    if (isOutOfBounds(v)) {
                        startSpring({ from: v, velocity: current.getVelocity() });
                    } else {
                        complete();
                    }
                });
            } else {
                complete();
            }
            return {
                stop: function () {
                    return activeAnimation && activeAnimation.stop();
                }
            };
        });
    };
    var index$1 = /*#__PURE__*/createVectorAction(inertia, {
        from: number.test,
        velocity: number.test,
        min: number.test,
        max: number.test,
        damping: number.test,
        stiffness: number.test,
        modifyTarget: function (func) {
            return typeof func === 'function';
        }
    });

    var scrubber = function (_a) {
        var _b = _a.from,
            from = _b === void 0 ? 0 : _b,
            _c = _a.to,
            to = _c === void 0 ? 1 : _c,
            _d = _a.ease,
            ease = _d === void 0 ? linear : _d;
        return action(function (_a) {
            var update = _a.update;
            return {
                seek: function (progress$$1) {
                    return update(progress$$1);
                }
            };
        }).pipe(ease, function (v) {
            return mix(from, to, v);
        });
    };
    var vectorScrubber = /*#__PURE__*/createVectorAction(scrubber, {
        ease: function (func) {
            return typeof func === 'function';
        },
        from: number.test,
        to: number.test
    });

    var clampProgress$1 = /*#__PURE__*/clamp$1$1(0, 1);
    var tween = function (props) {
        if (props === void 0) {
            props = {};
        }
        return action(function (_a) {
            var update = _a.update,
                complete = _a.complete;
            var _b = props.duration,
                duration = _b === void 0 ? 300 : _b,
                _c = props.ease,
                ease = _c === void 0 ? easeOut : _c,
                _d = props.flip,
                flip = _d === void 0 ? 0 : _d,
                _e = props.loop,
                loop = _e === void 0 ? 0 : _e,
                _f = props.yoyo,
                yoyo = _f === void 0 ? 0 : _f;
            var _g = props.from,
                from = _g === void 0 ? 0 : _g,
                _h = props.to,
                to = _h === void 0 ? 1 : _h,
                _j = props.elapsed,
                elapsed = _j === void 0 ? 0 : _j,
                _k = props.playDirection,
                playDirection = _k === void 0 ? 1 : _k,
                _l = props.flipCount,
                flipCount = _l === void 0 ? 0 : _l,
                _m = props.yoyoCount,
                yoyoCount = _m === void 0 ? 0 : _m,
                _o = props.loopCount,
                loopCount = _o === void 0 ? 0 : _o;
            var playhead = vectorScrubber({ from: from, to: to, ease: ease }).start(update);
            var currentProgress = 0;
            var process;
            var isActive = false;
            var reverseTween = function () {
                if (elapsed > duration) {
                    var remainder = elapsed - duration;
                    elapsed = elapsed - remainder * 2;
                } else if (elapsed < 0) {
                    var remainder = -1 * elapsed;
                    elapsed = elapsed + remainder * 2;
                }
                playDirection *= -1;
            };
            var isTweenComplete = function () {
                var _a;
                var isComplete = playDirection === 1 ? isActive && elapsed >= duration : isActive && elapsed <= 0;
                if (!isComplete) return false;
                if (isComplete && !loop && !flip && !yoyo) return true;
                var isStepTaken = false;
                if (loop && loopCount < loop) {
                    elapsed = duration - elapsed;
                    loopCount++;
                    isStepTaken = true;
                } else if (flip && flipCount < flip) {
                    elapsed = duration - elapsed;
                    _a = [to, from], from = _a[0], to = _a[1];
                    playhead = vectorScrubber({ from: from, to: to, ease: ease }).start(update);
                    flipCount++;
                    isStepTaken = true;
                } else if (yoyo && yoyoCount < yoyo) {
                    reverseTween();
                    yoyoCount++;
                    isStepTaken = true;
                }
                return !isStepTaken;
            };
            var updateTween = function () {
                currentProgress = clampProgress$1(progress(0, duration, elapsed));
                playhead.seek(currentProgress);
            };
            var startTimer = function () {
                isActive = true;
                process = sync.update(function (_a) {
                    var delta = _a.delta;
                    elapsed += delta * playDirection;
                    updateTween();
                    if (isTweenComplete() && complete) {
                        cancelSync.update(process);
                        sync.update(complete, false, true);
                    }
                }, true);
            };
            var stopTimer = function () {
                isActive = false;
                if (process) cancelSync.update(process);
            };
            startTimer();
            return {
                isActive: function () {
                    return isActive;
                },
                getElapsed: function () {
                    return clamp$1$1(0, duration, elapsed);
                },
                getProgress: function () {
                    return currentProgress;
                },
                stop: function () {
                    stopTimer();
                },
                pause: function () {
                    stopTimer();
                    return this;
                },
                resume: function () {
                    if (!isActive) startTimer();
                    return this;
                },
                seek: function (newProgress) {
                    elapsed = mix(0, duration, newProgress);
                    sync.update(updateTween, false, true);
                    return this;
                },
                reverse: function () {
                    reverseTween();
                    return this;
                }
            };
        });
    };

    var clampProgress$1$1 = /*#__PURE__*/clamp$1$1(0, 1);
    var defaultEasings = function (values, easing$$1) {
        return values.map(function () {
            return easing$$1 || easeOut;
        }).splice(0, values.length - 1);
    };
    var defaultTimings = function (values) {
        var numValues = values.length;
        return values.map(function (value, i) {
            return i !== 0 ? i / (numValues - 1) : 0;
        });
    };
    var interpolateScrubbers = function (input, scrubbers, update) {
        var rangeLength = input.length;
        var finalInputIndex = rangeLength - 1;
        var finalScrubberIndex = finalInputIndex - 1;
        var subs = scrubbers.map(function (scrub) {
            return scrub.start(update);
        });
        return function (v) {
            if (v <= input[0]) {
                subs[0].seek(0);
            }
            if (v >= input[finalInputIndex]) {
                subs[finalScrubberIndex].seek(1);
            }
            var i = 1;
            for (; i < rangeLength; i++) {
                if (input[i] > v || i === finalInputIndex) break;
            }
            var progressInRange = progress(input[i - 1], input[i], v);
            subs[i - 1].seek(clampProgress$1$1(progressInRange));
        };
    };
    var keyframes = function (_a) {
        var easings = _a.easings,
            _b = _a.ease,
            ease = _b === void 0 ? linear : _b,
            times = _a.times,
            values = _a.values,
            tweenProps = __rest(_a, ["easings", "ease", "times", "values"]);
        easings = Array.isArray(easings) ? easings : defaultEasings(values, easings);
        times = times || defaultTimings(values);
        var scrubbers = easings.map(function (easing$$1, i) {
            return vectorScrubber({
                from: values[i],
                to: values[i + 1],
                ease: easing$$1
            });
        });
        return tween(__assign({}, tweenProps, { ease: ease })).applyMiddleware(function (update) {
            return interpolateScrubbers(times, scrubbers, update);
        });
    };

    var listen = function (element, events, options) {
        return action(function (_a) {
            var update = _a.update;
            var eventNames = events.split(' ').map(function (eventName) {
                element.addEventListener(eventName, update, options);
                return eventName;
            });
            return {
                stop: function () {
                    return eventNames.forEach(function (eventName) {
                        return element.removeEventListener(eventName, update, options);
                    });
                }
            };
        });
    };

    var defaultPointerPos = function () {
        return {
            clientX: 0,
            clientY: 0,
            pageX: 0,
            pageY: 0,
            x: 0,
            y: 0
        };
    };
    var eventToPoint = function (e, point) {
        if (point === void 0) {
            point = defaultPointerPos();
        }
        point.clientX = point.x = e.clientX;
        point.clientY = point.y = e.clientY;
        point.pageX = e.pageX;
        point.pageY = e.pageY;
        return point;
    };

    var points = [/*#__PURE__*/defaultPointerPos()];
    if (typeof document !== 'undefined') {
        var updatePointsLocation = function (_a) {
            var touches = _a.touches;
            var numTouches = touches.length;
            points.length = 0;
            for (var i = 0; i < numTouches; i++) {
                var thisTouch = touches[i];
                points.push(eventToPoint(thisTouch));
            }
        };
        listen(document, 'touchstart touchmove', {
            passive: true,
            capture: true
        }).start(updatePointsLocation);
    }

    var point = /*#__PURE__*/defaultPointerPos();
    if (typeof document !== 'undefined') {
        var updatePointLocation = function (e) {
            eventToPoint(e, point);
        };
        listen(document, 'mousedown mousemove', true).start(updatePointLocation);
    }

    var chain = function () {
        var actions = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            actions[_i] = arguments[_i];
        }
        return action(function (_a) {
            var update = _a.update,
                complete = _a.complete;
            var i = 0;
            var current;
            var playCurrent = function () {
                current = actions[i].start({
                    complete: function () {
                        i++;
                        i >= actions.length ? complete() : playCurrent();
                    },
                    update: update
                });
            };
            playCurrent();
            return {
                stop: function () {
                    return current && current.stop();
                }
            };
        });
    };

    var delay = function (timeToDelay) {
        return action(function (_a) {
            var complete = _a.complete;
            var timeout = setTimeout(complete, timeToDelay);
            return {
                stop: function () {
                    return clearTimeout(timeout);
                }
            };
        });
    };

    var appendUnit = function (unit) {
        return function (v) {
            return "" + v + unit;
        };
    };
    var steps$2 = function (st, min, max) {
        if (min === void 0) {
            min = 0;
        }
        if (max === void 0) {
            max = 1;
        }
        return function (v) {
            var current = progress(min, max, v);
            return mix(min, max, stepProgress(st, current));
        };
    };
    var transformMap = function (childTransformers) {
        return function (v) {
            var output = __assign({}, v);
            for (var key in childTransformers) {
                if (childTransformers.hasOwnProperty(key)) {
                    var childTransformer = childTransformers[key];
                    output[key] = childTransformer(v[key]);
                }
            }
            return output;
        };
    };

    var transformers = /*#__PURE__*/Object.freeze({
        applyOffset: applyOffset,
        clamp: clamp$1$1,
        conditional: conditional,
        interpolate: interpolate$$1,
        blendArray: mixArray,
        blendColor: mixColor,
        pipe: pipe,
        smooth: smooth,
        snap: snap,
        generateStaticSpring: springForce,
        nonlinearSpring: springForceExpo,
        linearSpring: springForceLinear,
        wrap: wrap$1,
        appendUnit: appendUnit,
        steps: steps$2,
        transformMap: transformMap
    });

    var underDampedSpring = function () { return ({
        type: "spring",
        stiffness: 500,
        damping: 25,
        restDelta: 0.5,
        restSpeed: 10,
    }); };
    var overDampedSpring = function (to) { return ({
        type: "spring",
        stiffness: 700,
        damping: to === 0 ? 100 : 35,
    }); };
    var linearTween = function () { return ({
        ease: "linear",
        duration: 0.3,
    }); };
    var keyframes$1 = function (values) { return ({
        type: "keyframes",
        duration: 0.8,
        values: values,
    }); };
    var defaultTransitions = {
        x: underDampedSpring,
        y: underDampedSpring,
        z: underDampedSpring,
        rotate: underDampedSpring,
        rotateX: underDampedSpring,
        rotateY: underDampedSpring,
        rotateZ: underDampedSpring,
        scaleX: overDampedSpring,
        scaleY: overDampedSpring,
        scale: overDampedSpring,
        opacity: linearTween,
        backgroundColor: linearTween,
        color: linearTween,
        default: overDampedSpring,
    };
    var getDefaultTransition = function (valueKey, to) {
        var transitionFactory;
        if (isKeyframesTarget(to)) {
            transitionFactory = keyframes$1;
        }
        else {
            transitionFactory =
                defaultTransitions[valueKey] || defaultTransitions.default;
        }
        return __assign({ to: to }, transitionFactory(to));
    };

    /**
     * A Popmotion action that accepts a single `to` prop. When it starts, it immediately
     * updates with `to` and then completes. By using this we can compose instant transitions
     * in with the same logic that applies `delay` or returns a `Promise` etc.
     */
    var just = function (_a) {
        var to = _a.to;
        return action(function (_a) {
            var update = _a.update, complete = _a.complete;
            update(to);
            complete();
        });
    };

    var easingDefinitionToFunction = function (definition) {
        if (Array.isArray(definition)) {
            // If cubic bezier definition, create bezier curve
            invariant(definition.length === 4, "Cubic bezier arrays must contain four numerical values.");
            var x1 = definition[0], y1 = definition[1], x2 = definition[2], y2 = definition[3];
            return cubicBezier(x1, y1, x2, y2);
        }
        else if (typeof definition === "string") {
            // Else lookup from table
            invariant(easingLookup[definition] !== undefined, "Invalid easing type '" + definition + "'");
            return easingLookup[definition];
        }
        return definition;
    };
    var isEasingArray = function (ease) {
        return Array.isArray(ease) && typeof ease[0] !== "number";
    };

    var isDurationAnimation = function (v) { return v.hasOwnProperty("duration"); };

    /**
     * Check if a value is animatable. Examples:
     *
     * ✅: 100, "100px", "#fff"
     * ❌: "block", "url(2.jpg)"
     * @param value
     *
     * @internal
     */
    var isAnimatable = function (key, value) {
        // If the list of keys tat might be non-animatable grows, replace with Set
        if (key === "zIndex")
            return false;
        // If it's a number or a keyframes array, we can animate it. We might at some point
        // need to do a deep isAnimatable check of keyframes, or let Popmotion handle this,
        // but for now lets leave it like this for performance reasons
        if (typeof value === "number" || Array.isArray(value))
            return true;
        if (typeof value === "string" && // It's animatable if we have a string
            complex.test(value) && // And it contains numbers and/or colors
            !value.startsWith("url(") // Unless it starts with "url("
        ) {
            return true;
        }
        return false;
    };

    var transitions = { tween: tween, spring: vectorSpring, keyframes: keyframes, inertia: index$1, just: just };
    var transitionOptionParser = {
        tween: function (opts) {
            if (opts.ease) {
                var ease = isEasingArray(opts.ease) ? opts.ease[0] : opts.ease;
                opts.ease = easingDefinitionToFunction(ease);
            }
            return opts;
        },
        keyframes: function (_a) {
            var from = _a.from, to = _a.to, velocity = _a.velocity, opts = __rest(_a, ["from", "to", "velocity"]);
            if (opts.values && opts.values[0] === null) {
                var values = opts.values.slice();
                values[0] = from;
                opts.values = values;
            }
            if (opts.ease) {
                opts.easings = isEasingArray(opts.ease)
                    ? opts.ease.map(easingDefinitionToFunction)
                    : easingDefinitionToFunction(opts.ease);
            }
            opts.ease = linear;
            return opts;
        },
    };
    var isTransitionDefined = function (_a) {
        var when = _a.when, delay$$1 = _a.delay, delayChildren = _a.delayChildren, staggerChildren = _a.staggerChildren, staggerDirection = _a.staggerDirection, transition = __rest(_a, ["when", "delay", "delayChildren", "staggerChildren", "staggerDirection"]);
        return Object.keys(transition).length;
    };
    var getTransitionDefinition = function (key, to, transitionDefinition) {
        var delay$$1 = transitionDefinition ? transitionDefinition.delay : 0;
        // If no object, return default transition
        // A better way to handle this would be to deconstruct out all the shared Orchestration props
        // and see if there's any props remaining
        if (transitionDefinition === undefined ||
            !isTransitionDefined(transitionDefinition)) {
            return __assign({ delay: delay$$1 }, getDefaultTransition(key, to));
        }
        var valueTransitionDefinition = transitionDefinition[key] ||
            transitionDefinition.default ||
            transitionDefinition;
        if (valueTransitionDefinition.type === false) {
            return {
                type: "just",
                delay: delay$$1,
                to: isKeyframesTarget(to)
                    ? to[to.length - 1]
                    : to,
            };
        }
        else if (isKeyframesTarget(to)) {
            return __assign({ values: to, duration: 0.8, delay: delay$$1, ease: "linear" }, valueTransitionDefinition, { 
                // This animation must be keyframes if we're animating through an array
                type: "keyframes" });
        }
        else {
            return __assign({ type: "tween", to: to,
                delay: delay$$1 }, valueTransitionDefinition);
        }
    };
    var preprocessOptions = function (type, opts) {
        return transitionOptionParser[type]
            ? transitionOptionParser[type](opts)
            : opts;
    };
    var getAnimation = function (key, value$$1, target, transition) {
        var origin = value$$1.get();
        var isOriginAnimatable = isAnimatable(key, origin);
        var isTargetAnimatable = isAnimatable(key, target);
        // TODO we could probably improve this check to ensure both values are of the same type -
        // for instance 100 to #fff. This might live better in Popmotion.
        warning(isOriginAnimatable === isTargetAnimatable, "You are trying to animate " + key + " from \"" + origin + "\" to " + target + ". \"" + origin + "\" is not an animatable value - to enable this animation set " + origin + " to a value animatable to " + target + " via the `style` property.");
        // Parse the `transition` prop and return options for the Popmotion animation
        var _a = getTransitionDefinition(key, target, transition), _b = _a.type, type = _b === void 0 ? "tween" : _b, transitionDefinition = __rest(_a, ["type"]);
        // If this is an animatable pair of values, return an animation, otherwise use `just`
        var actionFactory = isOriginAnimatable && isTargetAnimatable ? transitions[type] : just;
        var opts = preprocessOptions(type, __assign({ from: origin, velocity: value$$1.getVelocity() }, transitionDefinition));
        return [actionFactory, opts];
    };
    /**
     * Start animation on a value. This function completely encapsulates Popmotion-specific logic.
     *
     * @internal
     */
    function startAnimation(key, value$$1, target, _a) {
        var _b = _a.delay, delay$$1 = _b === void 0 ? 0 : _b, transition = __rest(_a, ["delay"]);
        var _c = getAnimation(key, value$$1, target, transition), animationFactory = _c[0], opts = _c[1];
        // Convert durations from Framer Motion seconds into Popmotion milliseconds
        if (isDurationAnimation(opts) && opts.duration) {
            opts.duration *= 1000;
        }
        delay$$1 *= 1000;
        // Bind animation opts to animation
        var animation = animationFactory(opts);
        // Compose delay
        if (delay$$1) {
            animation = chain(delay(delay$$1), animation);
        }
        return value$$1.start(function (complete) {
            var activeAnimation = animation.start({
                update: function (v) { return value$$1.set(v); },
                complete: complete,
            });
            return function () { return activeAnimation.stop(); };
        });
    }

    /**
     * Get the current value of every `MotionValue`
     * @param values -
     */
    var getCurrent = function (values) {
        var current = {};
        values.forEach(function (value, key) { return (current[key] = value.get()); });
        return current;
    };
    /**
     * Get the current velocity of every `MotionValue`
     * @param values -
     */
    var getVelocity = function (values) {
        var velocity = {};
        values.forEach(function (value, key) { return (velocity[key] = value.getVelocity()); });
        return velocity;
    };
    /**
     * Check if value is a function that returns a `Target`. A generic typeof === 'function'
     * check, just helps with typing.
     * @param p -
     */
    var isTargetResolver = function (p) {
        return typeof p === "function";
    };
    /**
     * Check if value is a list of variant labels
     * @param v -
     */
    var isVariantLabels = function (v) { return Array.isArray(v); };
    /**
     * Check if value is a numerical string, ie "100" or "100px"
     */
    var isNumericalString = function (v) { return /^\d*\.?\d+$/.test(v); };
    /**
     * Control animations for a single component
     * @internal
     */
    var ValueAnimationControls = /** @class */ (function () {
        function ValueAnimationControls(_a) {
            var values = _a.values, readValueFromSource = _a.readValueFromSource, makeTargetAnimatable = _a.makeTargetAnimatable;
            var _this = this;
            /**
             * The component's variants, as provided by `variants`
             */
            this.variants = {};
            /**
             * A set of values that we animate back to when a value is cleared of all overrides.
             */
            this.baseTarget = {};
            /**
             * A series of target overrides that we can animate to/from when overrides are set/cleared.
             */
            this.overrides = [];
            /**
             * A series of target overrides as they were originally resolved.
             */
            this.resolvedOverrides = [];
            /**
             * A Set of currently active override indexes
             */
            this.activeOverrides = new Set();
            /**
             * A Set of value keys that are currently animating.
             */
            this.isAnimating = new Set();
            this.values = values;
            this.readValueFromSource = readValueFromSource;
            this.makeTargetAnimatable = makeTargetAnimatable;
            this.values.forEach(function (value, key) { return (_this.baseTarget[key] = value.get()); });
        }
        /**
         * Set the reference to the component's props.
         * @param props -
         */
        ValueAnimationControls.prototype.setProps = function (props) {
            this.props = props;
        };
        /**
         * Set the reference to the component's variants
         * @param variants -
         */
        ValueAnimationControls.prototype.setVariants = function (variants) {
            if (variants)
                this.variants = variants;
        };
        /**
         * Set the component's default transition
         * @param transition -
         */
        ValueAnimationControls.prototype.setDefaultTransition = function (transition) {
            if (transition)
                this.defaultTransition = transition;
        };
        /**
         * Set motion values without animation.
         *
         * @param target -
         * @param isActive -
         */
        ValueAnimationControls.prototype.setValues = function (target, isActive) {
            var _this = this;
            if (isActive === void 0) { isActive = new Set(); }
            target = this.transformValues(target);
            return Object.keys(target).forEach(function (key) {
                if (isActive.has(key))
                    return;
                isActive.add(key);
                var targetValue = resolveFinalValueInKeyframes(target[key]);
                if (_this.values.has(key)) {
                    var value = _this.values.get(key);
                    value && value.set(targetValue);
                }
                else {
                    _this.values.set(key, motionValue(targetValue));
                }
                _this.baseTarget[key] = targetValue;
            });
        };
        /**
         * Allows `transformValues` to be set by a component that allows us to
         * transform the values in a given `Target`. This allows Framer Library
         * to extend Framer Motion to animate `Color` variables etc. Currently we have
         * to manually support these extended types here in Framer Motion.
         *
         * @param values -
         */
        ValueAnimationControls.prototype.transformValues = function (values) {
            var transformValues = this.props.transformValues;
            return transformValues ? transformValues(values) : values;
        };
        /**
         * Check a `Target` for new values we haven't animated yet, and add them
         * to the `MotionValueMap`.
         *
         * Currently there's functionality here that is DOM-specific, we should allow
         * this functionality to be injected by the factory that creates DOM-specific
         * components.
         *
         * @param target -
         */
        ValueAnimationControls.prototype.checkForNewValues = function (target) {
            var _this = this;
            var newValueKeys = Object.keys(target).filter(function (key) { return !_this.values.has(key); });
            if (!newValueKeys.length)
                return;
            newValueKeys.forEach(function (key) {
                var value = _this.readValueFromSource(key);
                if (typeof value === "string" && isNumericalString(value)) {
                    // If this is a number read as a string, ie "0" or "200", convert it to a number
                    value = parseFloat(value);
                }
                else if (!getValueType$2(value) && complex.test(target[key])) {
                    // If value is not recognised as animatable, ie "none", create an animatable version origin based on the target
                    value = complex.getAnimatableNone(target[key]);
                }
                _this.values.set(key, motionValue(value));
                _this.baseTarget[key] = value;
            });
        };
        /**
         * Resolve a variant from its label or resolver into an actual `Target` we can animate to.
         * @param variant -
         */
        ValueAnimationControls.prototype.resolveVariant = function (variant) {
            if (!variant) {
                return {
                    target: undefined,
                    transition: undefined,
                    transitionEnd: undefined,
                };
            }
            if (isTargetResolver(variant)) {
                // resolve current and velocity
                variant = variant(this.props.custom, getCurrent(this.values), getVelocity(this.values));
            }
            var _a = variant.transition, transition = _a === void 0 ? this.defaultTransition : _a, transitionEnd = variant.transitionEnd, target = __rest(variant, ["transition", "transitionEnd"]);
            return { transition: transition, transitionEnd: transitionEnd, target: target };
        };
        /**
         * Get the highest active override priority index
         */
        ValueAnimationControls.prototype.getHighestPriority = function () {
            if (!this.activeOverrides.size)
                return 0;
            return Math.max.apply(Math, Array.from(this.activeOverrides));
        };
        /**
         * Set an override. We add this layer of indirection so if, for instance, a tap gesture
         * starts and overrides a hover gesture, when we clear the tap gesture and fallback to the
         * hover gesture, if that hover gesture has changed in the meantime we can go to that rather
         * than the one that was resolved when the hover gesture animation started.
         *
         * @param definition -
         * @param overrideIndex -
         */
        ValueAnimationControls.prototype.setOverride = function (definition, overrideIndex) {
            this.overrides[overrideIndex] = definition;
            if (this.children) {
                this.children.forEach(function (child) {
                    return child.setOverride(definition, overrideIndex);
                });
            }
        };
        /**
         * Start an override animation.
         * @param overrideIndex -
         */
        ValueAnimationControls.prototype.startOverride = function (overrideIndex) {
            var override = this.overrides[overrideIndex];
            if (override) {
                return this.start(override, { priority: overrideIndex });
            }
        };
        /**
         * Clear an override. We check every value we animated to in this override to see if
         * its present on any lower-priority overrides. If not, we animate it back to its base target.
         * @param overrideIndex -
         */
        ValueAnimationControls.prototype.clearOverride = function (overrideIndex) {
            if (this.children) {
                this.children.forEach(function (child) { return child.clearOverride(overrideIndex); });
            }
            var override = this.overrides[overrideIndex];
            if (!override)
                return;
            this.activeOverrides.delete(overrideIndex);
            var highest = this.getHighestPriority();
            this.resetIsAnimating();
            if (highest) {
                var highestOverride = this.overrides[highest];
                highestOverride && this.startOverride(highest);
            }
            // Figure out which remaining values were affected by the override and animate those
            var overrideTarget = this.resolvedOverrides[overrideIndex];
            if (!overrideTarget)
                return;
            var remainingValues = {};
            for (var key in this.baseTarget) {
                if (overrideTarget[key] !== undefined) {
                    remainingValues[key] = this.baseTarget[key];
                }
            }
            this.animate(remainingValues);
        };
        /**
         * Apply a target/variant without any animation
         */
        ValueAnimationControls.prototype.apply = function (definition) {
            if (Array.isArray(definition)) {
                return this.applyVariantLabels(definition);
            }
            else if (typeof definition === "string") {
                return this.applyVariantLabels([definition]);
            }
            else {
                this.setValues(definition);
            }
        };
        /**
         * Apply variant labels without animation
         */
        ValueAnimationControls.prototype.applyVariantLabels = function (variantLabelList) {
            var _this = this;
            var isSetting = new Set();
            var reversedList = variantLabelList.slice().reverse();
            reversedList.forEach(function (key) {
                var _a = _this.resolveVariant(_this.variants[key]), target = _a.target, transitionEnd = _a.transitionEnd;
                if (transitionEnd) {
                    _this.setValues(transitionEnd, isSetting);
                }
                if (target) {
                    _this.setValues(target, isSetting);
                }
            });
        };
        ValueAnimationControls.prototype.start = function (definition, opts) {
            var _this = this;
            if (opts === void 0) { opts = {}; }
            if (opts.priority) {
                this.activeOverrides.add(opts.priority);
            }
            this.resetIsAnimating(opts.priority);
            var animation;
            if (isVariantLabels(definition)) {
                animation = this.animateVariantLabels(definition, opts);
            }
            else if (typeof definition === "string") {
                animation = this.animateVariant(definition, opts);
            }
            else {
                animation = this.animate(definition, opts);
            }
            return animation.then(function () {
                var onAnimationComplete = _this.props.onAnimationComplete;
                onAnimationComplete && onAnimationComplete();
            });
        };
        ValueAnimationControls.prototype.animate = function (animationDefinition, _a) {
            var _this = this;
            var _b = _a === void 0 ? {} : _a, _c = _b.delay, delay = _c === void 0 ? 0 : _c, _d = _b.priority, priority = _d === void 0 ? 0 : _d, transitionOverride = _b.transitionOverride;
            var _e = this.resolveVariant(animationDefinition), target = _e.target, transition = _e.transition, transitionEnd = _e.transitionEnd;
            if (transitionOverride) {
                transition = transitionOverride;
            }
            if (!target)
                return Promise.resolve();
            target = this.transformValues(target);
            if (transitionEnd) {
                transitionEnd = this.transformValues(transitionEnd);
            }
            if (this.makeTargetAnimatable) {
                var animatable = this.makeTargetAnimatable(target, transitionEnd);
                target = animatable.target;
                transitionEnd = animatable.transitionEnd;
            }
            if (priority) {
                this.resolvedOverrides[priority] = target;
            }
            this.checkForNewValues(target);
            var animations = Object.keys(target).reduce(function (acc, key) {
                var value = _this.values.get(key);
                if (!value || !target || target[key] === undefined)
                    return acc;
                var valueTarget = target[key];
                if (!priority) {
                    _this.baseTarget[key] = resolveFinalValueInKeyframes(valueTarget);
                }
                if (_this.isAnimating.has(key))
                    return acc;
                acc.push(startAnimation(key, value, valueTarget, __assign({ delay: delay }, transition)));
                _this.isAnimating.add(key);
                return acc;
            }, []);
            return Promise.all(animations).then(function () {
                if (!transitionEnd)
                    return;
                _this.setValues(transitionEnd);
            });
        };
        ValueAnimationControls.prototype.animateVariantLabels = function (variantLabels, opts) {
            var _this = this;
            var animations = variantLabels.slice().reverse()
                .map(function (label) { return _this.animateVariant(label, opts); });
            return Promise.all(animations);
        };
        ValueAnimationControls.prototype.animateVariant = function (variantLabel, opts) {
            var _this = this;
            var when = false;
            var delayChildren = 0;
            var staggerChildren = 0;
            var staggerDirection = 1;
            var priority = (opts && opts.priority) || 0;
            var variant = this.variants[variantLabel];
            var getAnimations = variant
                ? function () { return _this.animate(variant, opts); }
                : function () { return Promise.resolve(); };
            var getChildrenAnimations = this.children
                ? function () {
                    return _this.animateChildren(variantLabel, delayChildren, staggerChildren, staggerDirection, priority);
                }
                : function () { return Promise.resolve(); };
            if (variant && this.children) {
                var transition = this.resolveVariant(variant).transition;
                if (transition) {
                    when = transition.when || when;
                    delayChildren = transition.delayChildren || delayChildren;
                    staggerChildren = transition.staggerChildren || staggerChildren;
                    staggerDirection =
                        transition.staggerDirection || staggerDirection;
                }
            }
            if (when) {
                var _a = when === "beforeChildren"
                    ? [getAnimations, getChildrenAnimations]
                    : [getChildrenAnimations, getAnimations], first = _a[0], last = _a[1];
                return first().then(last);
            }
            else {
                return Promise.all([getAnimations(), getChildrenAnimations()]);
            }
        };
        ValueAnimationControls.prototype.animateChildren = function (variantLabel, delayChildren, staggerChildren, staggerDirection, priority) {
            if (delayChildren === void 0) { delayChildren = 0; }
            if (staggerChildren === void 0) { staggerChildren = 0; }
            if (staggerDirection === void 0) { staggerDirection = 1; }
            if (priority === void 0) { priority = 0; }
            if (!this.children) {
                return Promise.resolve();
            }
            var animations = [];
            var maxStaggerDuration = (this.children.size - 1) * staggerChildren;
            var generateStaggerDuration = staggerDirection === 1
                ? function (i) { return i * staggerChildren; }
                : function (i) { return maxStaggerDuration - i * staggerChildren; };
            Array.from(this.children).forEach(function (childControls, i) {
                var animation = childControls.animateVariant(variantLabel, {
                    priority: priority,
                    delay: delayChildren + generateStaggerDuration(i),
                });
                animations.push(animation);
            });
            return Promise.all(animations);
        };
        ValueAnimationControls.prototype.checkOverrideIsAnimating = function (priority) {
            var numOverrides = this.overrides.length;
            for (var i = priority + 1; i < numOverrides; i++) {
                var resolvedOverride = this.resolvedOverrides[i];
                if (resolvedOverride) {
                    for (var key in resolvedOverride) {
                        this.isAnimating.add(key);
                    }
                }
            }
        };
        ValueAnimationControls.prototype.resetIsAnimating = function (priority) {
            if (priority === void 0) { priority = 0; }
            this.isAnimating.clear();
            // If this isn't the highest priority gesture, block the animation
            // of anything that's currently being animated
            if (priority < this.getHighestPriority()) {
                this.checkOverrideIsAnimating(priority);
            }
            if (this.children) {
                this.children.forEach(function (child) { return child.resetIsAnimating(priority); });
            }
        };
        ValueAnimationControls.prototype.stop = function () {
            this.values.forEach(function (value) { return value.stop(); });
        };
        /**
         * Add the controls of a child component.
         * @param controls -
         */
        ValueAnimationControls.prototype.addChild = function (controls) {
            if (!this.children) {
                this.children = new Set();
            }
            this.children.add(controls);
            // We set child overrides when `setOverride` is called, but also have to do it here
            // as the first time `setOverride` is called all the children might not have been added yet.
            this.overrides.forEach(function (override, i) {
                override && controls.setOverride(override, i);
            });
        };
        ValueAnimationControls.prototype.removeChild = function (controls) {
            if (!this.children) {
                return;
            }
            this.children.delete(controls);
        };
        ValueAnimationControls.prototype.resetChildren = function () {
            if (this.children)
                this.children.clear();
        };
        return ValueAnimationControls;
    }());

    /**
     * Use a callback a maximum number of times
     * @param callback - Callback to run
     * @param times - Maximum number of times to run the callback. Defaults to `1`
     *
     * @public
     */
    function useMaxTimes(callback, times) {
        if (times === void 0) { times = 1; }
        var count = React.useRef(0);
        if (count.current++ < times)
            callback();
    }

    /**
     * @internal
     */
    var MotionContext = React.createContext({
        static: false,
    });
    var isVariantLabel = function (v) {
        return typeof v === "string" || Array.isArray(v);
    };
    /**
     * Set up the context for children motion components.
     *
     * We also use this opportunity to apply `initial` values
     */
    var useMotionContext = function (parentContext, controls, isStatic, _a) {
        if (isStatic === void 0) { isStatic = false; }
        var initial = _a.initial, animate = _a.animate, variants = _a.variants, whileTap = _a.whileTap, whileHover = _a.whileHover;
        // We pass on this component's ValueAnimationControls *if* we're being provided variants,
        // or if we're being used to control variants. Otherwise this component should be "invisible" to
        // variant propagation.
        var shouldPropagateControls = variants ||
            isVariantLabel(animate) ||
            isVariantLabel(whileTap) ||
            isVariantLabel(whileHover);
        // If this component's `initial` prop is a variant label, propagate it. Otherwise pass the parent's.
        var targetInitial = isVariantLabel(initial)
            ? initial
            : parentContext.initial;
        // Only allow `initial` to trigger context re-renders if this is a `static` component (ie we're on the Framer canvas)
        // or in another non-animation/interaction environment.
        var initialDependency = isStatic ? targetInitial : null;
        // Only allow `animate` to trigger context re-renders if it's a variant label. If this is an array of
        // variant labels there's probably an optimisation to deep-compare but it might be an over-optimisation.
        // We want to do this as we rely on React's component rendering order each render cycle to determine
        // the new order of any child components for the `staggerChildren` functionality.
        var animateDependency = isVariantLabel(animate) ? animate : null;
        // The context to provide to the child. We `useMemo` because although `controls` and `initial` are
        // unlikely to change, by making the context an object it'll be considered a new value every render.
        // So all child motion components will re-render as a result.
        var context = React.useMemo(function () { return ({
            controls: shouldPropagateControls
                ? controls
                : parentContext.controls,
            initial: targetInitial,
        }); }, [initialDependency, animateDependency]);
        // Update the `static` property every render. This is unlikely to change but also essentially free.
        context.static = isStatic;
        // Set initial state. If this is a static component (ie in Framer canvas), respond to updates
        // in `initial`.
        useMaxTimes(function () {
            var initialToApply = initial || parentContext.initial;
            initialToApply && controls.apply(initialToApply);
        }, isStatic ? Infinity : 1);
        return context;
    };

    /**
     * Creates an imperative set of controls to trigger animations.
     *
     * This allows a consolidated, uniform API for animations, to be triggered by other APIs like the `animate` prop, or the gesture handlers.
     *
     * @param values
     * @param props
     * @param ref
     * @param inheritVariantChanges
     *
     * @internal
     */
    function useValueAnimationControls(config, props, inheritVariantChanges) {
        var variants = props.variants, transition = props.transition;
        var parentControls = React.useContext(MotionContext).controls;
        var controls = useConstant(function () { return new ValueAnimationControls(config); });
        // Reset and resubscribe children every render to ensure stagger order is correct
        controls.resetChildren();
        if (inheritVariantChanges && parentControls) {
            parentControls.addChild(controls);
        }
        React.useEffect(function () { return function () { return parentControls && parentControls.removeChild(controls); }; }, []);
        controls.setProps(props);
        controls.setVariants(variants);
        controls.setDefaultTransition(transition);
        return controls;
    }

    /**
     * Control animations on one or more components.
     *
     * @public
     */
    var AnimationControls = /** @class */ (function () {
        function AnimationControls() {
            /**
             * Track whether the host component has mounted.
             *
             * @internal
             */
            this.hasMounted = false;
            /**
             * Pending animations that are started before a component is mounted.
             *
             * @internal
             */
            this.pendingAnimations = [];
            /**
             * A collection of linked component animation controls.
             *
             * @internal
             */
            this.componentControls = new Set();
        }
        /**
         * Set variants on this and all child components.
         *
         * @param variants - The variants to set
         *
         * @internal
         */
        AnimationControls.prototype.setVariants = function (variants) {
            this.variants = variants;
            this.componentControls.forEach(function (controls) {
                return controls.setVariants(variants);
            });
        };
        /**
         * Set a default transition on this and all child components
         *
         * @param transition - The default transition to set
         *
         * @internal
         */
        AnimationControls.prototype.setDefaultTransition = function (transition) {
            this.defaultTransition = transition;
            this.componentControls.forEach(function (controls) {
                return controls.setDefaultTransition(transition);
            });
        };
        /**
         * Subscribes a component's animation controls to this.
         *
         * @param controls - The controls to subscribe
         * @returns An unsubscribe function.
         *
         * @internal
         */
        AnimationControls.prototype.subscribe = function (controls) {
            var _this = this;
            this.componentControls.add(controls);
            if (this.variants)
                controls.setVariants(this.variants);
            if (this.defaultTransition)
                controls.setDefaultTransition(this.defaultTransition);
            return function () { return _this.componentControls.delete(controls); };
        };
        /**
         * Starts an animation on all linked components.
         *
         * @remarks
         *
         * ```jsx
         * animation.start("variantLabel")
         * animation.start({
         *   x: 0,
         *   transition: { duration: 1 }
         * })
         * ```
         *
         * @param definition - Properties or variant label to animate to
         * @param transition - Optional `transtion` to apply to a variant
         * @returns - A `Promise` that resolves when all animations have completed.
         *
         * @public
         */
        AnimationControls.prototype.start = function (definition, transitionOverride) {
            var _this = this;
            if (this.hasMounted) {
                var animations_1 = [];
                this.componentControls.forEach(function (controls) {
                    var animation = controls.start(definition, {
                        transitionOverride: transitionOverride,
                    });
                    animations_1.push(animation);
                });
                return Promise.all(animations_1);
            }
            else {
                return new Promise(function (resolve) {
                    _this.pendingAnimations.push({
                        animation: [definition, transitionOverride],
                        resolve: resolve,
                    });
                });
            }
        };
        /**
         * Stops animations on all linked components.
         *
         * @public
         */
        AnimationControls.prototype.stop = function () {
            this.componentControls.forEach(function (controls) { return controls.stop(); });
        };
        /**
         * Initialises the animation controls.
         *
         * @internal
         */
        AnimationControls.prototype.mount = function () {
            var _this = this;
            this.hasMounted = true;
            this.pendingAnimations.forEach(function (_a) {
                var animation = _a.animation, resolve = _a.resolve;
                return _this.start.apply(_this, animation).then(resolve);
            });
        };
        /**
         * Stops all child animations when the host component unmounts.
         *
         * @internal
         */
        AnimationControls.prototype.unmount = function () {
            this.hasMounted = false;
            this.stop();
        };
        return AnimationControls;
    }());
    /**
     * @internal
     */
    var animationControls = function () { return new AnimationControls(); };

    var checkShouldInheritVariant = function (_a) {
        var animate = _a.animate, variants = _a.variants, _b = _a.inherit, inherit = _b === void 0 ? true : _b;
        return (inherit &&
            !!variants &&
            (!animate || animate instanceof AnimationControls));
    };

    var AnimatePropType;
    (function (AnimatePropType) {
        AnimatePropType["Target"] = "Target";
        AnimatePropType["VariantLabel"] = "VariantLabel";
        AnimatePropType["AnimationSubscription"] = "AnimationSubscription";
    })(AnimatePropType || (AnimatePropType = {}));

    var makeHookComponent = function (hook) { return function (props) {
        hook(props);
        return null;
    }; };

    function shallowCompare(next, prev) {
        if (prev === null)
            return false;
        var prevLength = prev.length;
        if (prevLength !== next.length)
            return false;
        for (var i = 0; i < prevLength; i++) {
            if (prev[i] !== next[i])
                return false;
        }
        return true;
    }

    var hasUpdated = function (prev, next) {
        return (next !== undefined &&
            (Array.isArray(prev) && Array.isArray(next)
                ? !shallowCompare(next, prev)
                : prev !== next));
    };
    function justTarget(_a) {
        var transition = _a.transition, transitionEnd = _a.transitionEnd, target = __rest(_a, ["transition", "transitionEnd"]);
        return target;
    }
    /**
     * Handle the `animate` prop when its an object of values, ie:
     *
     * ```jsx
     * <motion.div animate={{ opacity: 1 }} />
     * ```
     *
     * @internalremarks
     * It might be worth consolidating this with `use-variants`
     *
     * ```jsx
     * <motion.div animate="visible" />
     * ```
     *
     * @param target
     * @param controls
     * @param values
     * @param transition
     *
     * @internal
     */
    function useAnimateProp(targetAndTransition, controls, values, defaultTransition) {
        var isInitialRender = React.useRef(true);
        var prevValues = React.useRef(null);
        if (!prevValues.current) {
            prevValues.current = justTarget(targetAndTransition);
        }
        React.useEffect(function () {
            var targetToAnimate = {};
            var target = justTarget(targetAndTransition);
            // Detect which values have changed between renders
            for (var key in prevValues.current) {
                // This value should animate on mount if this value doesn't already exist (wasn't
                // defined in `style` or `initial`) or if it does exist and it's already changed.
                var shouldAnimateOnMount = isInitialRender.current &&
                    (!values.has(key) || values.get(key) !== target[key]);
                // If this value has updated between renders or it's we're animating this value on mount,
                // add it to the animate target.
                if (hasUpdated(prevValues.current[key], target[key]) ||
                    shouldAnimateOnMount) {
                    targetToAnimate[key] = target[key];
                }
            }
            isInitialRender.current = false;
            prevValues.current = __assign({}, prevValues.current, target);
            if (Object.keys(targetToAnimate).length) {
                controls.start(__assign({}, targetToAnimate, { transition: targetAndTransition.transition || defaultTransition, transitionEnd: targetAndTransition.transitionEnd }));
            }
        }, [targetAndTransition]);
    }

    var labelsToArray = function (label) {
        if (!label) {
            return [];
        }
        if (Array.isArray(label)) {
            return label;
        }
        return [label];
    };
    var resolveVariantLabels = function (variant) {
        var unresolvedVariant = variant instanceof MotionValue ? variant.get() : variant;
        return Array.from(new Set(labelsToArray(unresolvedVariant)));
    };
    /**
     * Hooks in React sometimes accept a dependency array as their final argument. (ie useEffect/useMemo)
     * When values in this array change, React re-runs the dependency. However if the array
     * contains a variable number of items, React throws an error.
     */
    var asDependencyList = function (list) { return [
        list.join(","),
    ]; };

    var hasVariantChanged = function (oldVariant, newVariant) {
        return oldVariant.join(",") !== newVariant.join(",");
    };
    /**
     * Handle variants and the `animate` prop when its set as variant labels.
     *
     * @param targetVariant
     * @param inherit
     * @param controls
     * @param initialVariant
     *
     * @internal
     */
    function useVariants(targetVariant, inherit, controls, initialVariant) {
        var variantList = resolveVariantLabels(targetVariant);
        var hasMounted = React.useRef(false);
        // Fire animations when poses change
        React.useEffect(function () {
            // TODO: This logic might mean we don't need to load this hook at all
            if (inherit)
                return;
            if (hasMounted.current ||
                hasVariantChanged(resolveVariantLabels(initialVariant), variantList)) {
                controls.start(variantList);
            }
            hasMounted.current = true;
        }, asDependencyList(variantList));
    }

    /**
     * `useAnimationGroupSubscription` allows a component to subscribe to an
     * externally-created `AnimationControls`, created by the `useAnimation` hook.
     *
     * @param animation
     * @param controls
     *
     * @internal
     */
    function useAnimationGroupSubscription(animation, controls) {
        var unsubscribe = React.useMemo(function () { return animation.subscribe(controls); }, [
            animation,
        ]);
        React.useEffect(function () { return function () { return unsubscribe && unsubscribe(); }; }, [unsubscribe]);
    }

    var _a$1, _b;
    var AnimatePropComponents = (_a$1 = {},
        _a$1[AnimatePropType.Target] = makeHookComponent(function (_a) {
            var animate = _a.animate, controls = _a.controls, values = _a.values, transition = _a.transition;
            return useAnimateProp(animate, controls, values, transition);
        }),
        _a$1[AnimatePropType.VariantLabel] = makeHookComponent(function (_a) {
            var animate = _a.animate, _b = _a.inherit, inherit = _b === void 0 ? true : _b, controls = _a.controls, initial = _a.initial;
            return useVariants(animate, inherit, controls, initial);
        }),
        _a$1[AnimatePropType.AnimationSubscription] = makeHookComponent(function (_a) {
            var animate = _a.animate, controls = _a.controls;
            return useAnimationGroupSubscription(animate, controls);
        }),
        _a$1);
    var isVariantLabel$1 = function (prop) {
        return Array.isArray(prop) || typeof prop === "string";
    };
    var isAnimationSubscription = function (_a) {
        var animate = _a.animate;
        return animate instanceof AnimationControls;
    };
    var animationProps = ["initial", "animate", "whileTap", "whileHover"];
    var animatePropTypeTests = (_b = {},
        _b[AnimatePropType.Target] = function (props) {
            return (props.animate !== undefined &&
                !isVariantLabel$1(props.animate) &&
                !isAnimationSubscription(props));
        },
        _b[AnimatePropType.VariantLabel] = function (props) {
            return (props.variants !== undefined ||
                animationProps.some(function (key) { return typeof props[key] === "string"; }));
        },
        _b[AnimatePropType.AnimationSubscription] = isAnimationSubscription,
        _b);
    var getAnimateComponent = function (props, isStatic) {
        if (isStatic === void 0) { isStatic = false; }
        var animatePropType = undefined;
        for (var key in AnimatePropType) {
            if (animatePropTypeTests[key](props)) {
                animatePropType = key;
            }
        }
        return !isStatic && animatePropType
            ? AnimatePropComponents[animatePropType]
            : undefined;
    };

    /**
     * @internal
     */
    var createMotionComponent = function (_a) {
        var useFunctionalityComponents = _a.useFunctionalityComponents, getValueControlsConfig = _a.getValueControlsConfig;
        function MotionComponent(props, externalRef) {
            var ref = useExternalRef(externalRef);
            var parentContext = React.useContext(MotionContext);
            var isStatic = parentContext.static || props.static || false;
            var values = useMotionValues(props);
            var style = useMotionStyles(values, props.style, props.transformValues);
            var shouldInheritVariant = checkShouldInheritVariant(props);
            var controlsConfig = useConstant(function () {
                return getValueControlsConfig(ref, values);
            });
            var controls = useValueAnimationControls(controlsConfig, props, shouldInheritVariant);
            var context = useMotionContext(parentContext, controls, isStatic, props);
            // Add functionality
            var Animate = getAnimateComponent(props, context.static);
            var handleAnimate = Animate && (React.createElement(Animate, __assign({}, props, { inherit: shouldInheritVariant, innerRef: ref, values: values, controls: controls })));
            var handleActiveFunctionality = useFunctionalityComponents(props, values, controls, ref, style, context.static);
            return (React.createElement(MotionContext.Provider, { value: context },
                React.createElement(MountMotionValues, { ref: ref, values: values, isStatic: isStatic }),
                handleAnimate,
                handleActiveFunctionality));
        }
        return React.forwardRef(MotionComponent);
    };

    function memoize(fn) {
      var cache = {};
      return function (arg) {
        if (cache[arg] === undefined) cache[arg] = fn(arg);
        return cache[arg];
      };
    }

    var reactPropsRegex = /^((children|dangerouslySetInnerHTML|key|ref|autoFocus|defaultValue|defaultChecked|innerHTML|suppressContentEditableWarning|suppressHydrationWarning|valueLink|accept|acceptCharset|accessKey|action|allow|allowUserMedia|allowPaymentRequest|allowFullScreen|allowTransparency|alt|async|autoComplete|autoPlay|capture|cellPadding|cellSpacing|challenge|charSet|checked|cite|classID|className|cols|colSpan|content|contentEditable|contextMenu|controls|controlsList|coords|crossOrigin|data|dateTime|default|defer|dir|disabled|download|draggable|encType|form|formAction|formEncType|formMethod|formNoValidate|formTarget|frameBorder|headers|height|hidden|high|href|hrefLang|htmlFor|httpEquiv|id|inputMode|integrity|is|keyParams|keyType|kind|label|lang|list|loop|low|marginHeight|marginWidth|max|maxLength|media|mediaGroup|method|min|minLength|multiple|muted|name|nonce|noValidate|open|optimum|pattern|placeholder|playsInline|poster|preload|profile|radioGroup|readOnly|referrerPolicy|rel|required|reversed|role|rows|rowSpan|sandbox|scope|scoped|scrolling|seamless|selected|shape|size|sizes|slot|span|spellCheck|src|srcDoc|srcLang|srcSet|start|step|style|summary|tabIndex|target|title|type|useMap|value|width|wmode|wrap|about|datatype|inlist|prefix|property|resource|typeof|vocab|autoCapitalize|autoCorrect|autoSave|color|itemProp|itemScope|itemType|itemID|itemRef|results|security|unselectable|accentHeight|accumulate|additive|alignmentBaseline|allowReorder|alphabetic|amplitude|arabicForm|ascent|attributeName|attributeType|autoReverse|azimuth|baseFrequency|baselineShift|baseProfile|bbox|begin|bias|by|calcMode|capHeight|clip|clipPathUnits|clipPath|clipRule|colorInterpolation|colorInterpolationFilters|colorProfile|colorRendering|contentScriptType|contentStyleType|cursor|cx|cy|d|decelerate|descent|diffuseConstant|direction|display|divisor|dominantBaseline|dur|dx|dy|edgeMode|elevation|enableBackground|end|exponent|externalResourcesRequired|fill|fillOpacity|fillRule|filter|filterRes|filterUnits|floodColor|floodOpacity|focusable|fontFamily|fontSize|fontSizeAdjust|fontStretch|fontStyle|fontVariant|fontWeight|format|from|fr|fx|fy|g1|g2|glyphName|glyphOrientationHorizontal|glyphOrientationVertical|glyphRef|gradientTransform|gradientUnits|hanging|horizAdvX|horizOriginX|ideographic|imageRendering|in|in2|intercept|k|k1|k2|k3|k4|kernelMatrix|kernelUnitLength|kerning|keyPoints|keySplines|keyTimes|lengthAdjust|letterSpacing|lightingColor|limitingConeAngle|local|markerEnd|markerMid|markerStart|markerHeight|markerUnits|markerWidth|mask|maskContentUnits|maskUnits|mathematical|mode|numOctaves|offset|opacity|operator|order|orient|orientation|origin|overflow|overlinePosition|overlineThickness|panose1|paintOrder|pathLength|patternContentUnits|patternTransform|patternUnits|pointerEvents|points|pointsAtX|pointsAtY|pointsAtZ|preserveAlpha|preserveAspectRatio|primitiveUnits|r|radius|refX|refY|renderingIntent|repeatCount|repeatDur|requiredExtensions|requiredFeatures|restart|result|rotate|rx|ry|scale|seed|shapeRendering|slope|spacing|specularConstant|specularExponent|speed|spreadMethod|startOffset|stdDeviation|stemh|stemv|stitchTiles|stopColor|stopOpacity|strikethroughPosition|strikethroughThickness|string|stroke|strokeDasharray|strokeDashoffset|strokeLinecap|strokeLinejoin|strokeMiterlimit|strokeOpacity|strokeWidth|surfaceScale|systemLanguage|tableValues|targetX|targetY|textAnchor|textDecoration|textRendering|textLength|to|transform|u1|u2|underlinePosition|underlineThickness|unicode|unicodeBidi|unicodeRange|unitsPerEm|vAlphabetic|vHanging|vIdeographic|vMathematical|values|vectorEffect|version|vertAdvY|vertOriginX|vertOriginY|viewBox|viewTarget|visibility|widths|wordSpacing|writingMode|x|xHeight|x1|x2|xChannelSelector|xlinkActuate|xlinkArcrole|xlinkHref|xlinkRole|xlinkShow|xlinkTitle|xlinkType|xmlBase|xmlns|xmlnsXlink|xmlLang|xmlSpace|y|y1|y2|yChannelSelector|z|zoomAndPan|for|class|autofocus)|(([Dd][Aa][Tt][Aa]|[Aa][Rr][Ii][Aa]|x)-.*))$/; // https://esbench.com/bench/5bfee68a4cd7e6009ef61d23

    var index$2 = memoize(function (prop) {
      return reactPropsRegex.test(prop) || prop.charCodeAt(0) === 111
      /* o */
      && prop.charCodeAt(1) === 110
      /* n */
      && prop.charCodeAt(2) < 91;
    }
    /* Z+1 */
    );

    // These are identity functions that help TypeScript understand the type of the provided array
    var htmlTuple = function (args) { return args; };
    var svgTuple = function (args) { return args; };
    /**
     * @internal
     */
    var htmlElements = htmlTuple([
        "a",
        "abbr",
        "address",
        "area",
        "article",
        "aside",
        "audio",
        "b",
        "base",
        "bdi",
        "bdo",
        "big",
        "blockquote",
        "body",
        "br",
        "button",
        "canvas",
        "caption",
        "cite",
        "code",
        "col",
        "colgroup",
        "data",
        "datalist",
        "dd",
        "del",
        "details",
        "dfn",
        "dialog",
        "div",
        "dl",
        "dt",
        "em",
        "embed",
        "fieldset",
        "figcaption",
        "figure",
        "footer",
        "form",
        "h1",
        "h2",
        "h3",
        "h4",
        "h5",
        "h6",
        "head",
        "header",
        "hgroup",
        "hr",
        "html",
        "i",
        "iframe",
        "img",
        "input",
        "ins",
        "kbd",
        "keygen",
        "label",
        "legend",
        "li",
        "link",
        "main",
        "map",
        "mark",
        "menu",
        "menuitem",
        "meta",
        "meter",
        "nav",
        "noscript",
        "object",
        "ol",
        "optgroup",
        "option",
        "output",
        "p",
        "param",
        "picture",
        "pre",
        "progress",
        "q",
        "rp",
        "rt",
        "ruby",
        "s",
        "samp",
        "script",
        "section",
        "select",
        "small",
        "source",
        "span",
        "strong",
        "style",
        "sub",
        "summary",
        "sup",
        "table",
        "tbody",
        "td",
        "textarea",
        "tfoot",
        "th",
        "thead",
        "time",
        "title",
        "tr",
        "track",
        "u",
        "ul",
        "var",
        "video",
        "wbr",
        "webview",
    ]);
    /**
     * @internal
     */
    var svgElements = svgTuple([
        "animate",
        "circle",
        "clipPath",
        "defs",
        "desc",
        "ellipse",
        "feBlend",
        "feColorMatrix",
        "feComponentTransfer",
        "feComposite",
        "feConvolveMatrix",
        "feDiffuseLighting",
        "feDisplacementMap",
        "feDistantLight",
        "feDropShadow",
        "feFlood",
        "feFuncA",
        "feFuncB",
        "feFuncG",
        "feFuncR",
        "feGaussianBlur",
        "feImage",
        "feMerge",
        "feMergeNode",
        "feMorphology",
        "feOffset",
        "fePointLight",
        "feSpecularLighting",
        "feSpotLight",
        "feTile",
        "feTurbulence",
        "filter",
        "foreignObject",
        "g",
        "image",
        "line",
        "linearGradient",
        "marker",
        "mask",
        "metadata",
        "path",
        "pattern",
        "polygon",
        "polyline",
        "radialGradient",
        "rect",
        "stop",
        "svg",
        "switch",
        "symbol",
        "text",
        "textPath",
        "tspan",
        "use",
        "view",
    ]);

    function isEventTarget(target) {
        return (typeof target.addEventListener === "function" &&
            typeof target.removeEventListener === "function");
    }
    var eventListener = function (target, name, handler, options) {
        var startListening = function () {
            if (!target) {
                return;
            }
            // TODO: Pointer events are stacking after every pan start
            // https://github.com/framer/company/issues/12814
            target.addEventListener(name, handler, options);
        };
        var stopListening = function () {
            if (!target) {
                return;
            }
            target.removeEventListener(name, handler, options);
        };
        return [startListening, stopListening];
    };
    var useEvent = function (type, target, handler, options) {
        var result = undefined;
        if ((!target || isEventTarget(target)) && handler) {
            result = eventListener(target, type, handler, options);
        }
        React.useEffect(function () {
            var ref = target;
            if (!handler || !target || isEventTarget(target) || !ref.current) {
                return;
            }
            var _a = eventListener(ref.current, type, handler, options), start = _a[0], stop = _a[1];
            start();
            return stop;
        }, [type, target, handler, options]);
        return result;
    };

    var extractEventInfo = function (_a) {
        var _b = _a.pageX, pageX = _b === void 0 ? 0 : _b, _c = _a.pageY, pageY = _c === void 0 ? 0 : _c;
        return {
            point: { x: pageX, y: pageY },
        };
    };
    var wrapHandler = function (handler) {
        if (!handler) {
            return undefined;
        }
        var listener = function (event, info) {
            if (!info) {
                info = extractEventInfo(event);
            }
            handler(event, info);
        };
        return listener;
    };

    var mockWindow = {
        addEventListener: function () { },
        removeEventListener: function () { },
        dispatchEvent: function () { return false; },
        onpointerdown: false,
        onpointermove: false,
        onpointerup: false,
        ontouchstart: false,
        ontouchmove: false,
        ontouchend: false,
        onmousedown: false,
        onmousemove: false,
        onmouseup: false,
        devicePixelRatio: 1,
        scrollX: 0,
        scrollY: 0,
        location: {
            href: "",
        },
        setTimeout: function () { return 0; },
        clearTimeout: function () { },
        setInterval: function () { return 0; },
        clearInterval: function () { },
        webkitConvertPointFromPageToNode: function (_, pt) { return pt; },
    };
    /**
     * Creates a server-safe reference to `window`, returning a mock if none is available.
     *
     * @internal
     */
    var safeWindow = typeof window === "undefined" ? mockWindow : window;

    var noop = [function () { }, function () { }];
    var mergeUseEventResults = function () {
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        if (values.every(function (v) { return v === undefined; })) {
            return noop;
        }
        var start = function () {
            for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
                var value = values_1[_i];
                if (value) {
                    value[0]();
                }
            }
        };
        var stop = function () {
            for (var _i = 0, values_2 = values; _i < values_2.length; _i++) {
                var value = values_2[_i];
                if (value) {
                    value[1]();
                }
            }
        };
        return [start, stop];
    };
    /**
     * Filters out events not attached to the primary pointer (currently left mouse button)
     * @param eventHandler
     */
    var filterPrimaryPointer = function (eventHandler) {
        if (!eventHandler)
            return undefined;
        return function (event, info) {
            var isMouseEvent = event instanceof MouseEvent;
            var isPrimaryPointer = !isMouseEvent ||
                (isMouseEvent && event.button === 0);
            if (isPrimaryPointer) {
                eventHandler(event, info);
            }
        };
    };
    var useMouseEvents = function (_a, ref, options) {
        var onMouseDown = _a.onMouseDown, onMouseMove = _a.onMouseMove, onMouseUp = _a.onMouseUp, onMouseOver = _a.onMouseOver, onMouseOut = _a.onMouseOut, onMouseEnter = _a.onMouseEnter, onMouseLeave = _a.onMouseLeave;
        var down = useEvent("mousedown", ref, wrapHandler(filterPrimaryPointer(onMouseDown)), options);
        var move = useEvent("mousemove", ref, wrapHandler(onMouseMove), options);
        var up = useEvent("mouseup", ref, wrapHandler(onMouseUp), options);
        var over = useEvent("mouseover", ref, wrapHandler(onMouseOver), options);
        var out = useEvent("mouseout", ref, wrapHandler(onMouseOut), options);
        var enter = useEvent("mouseenter", ref, wrapHandler(onMouseEnter), options);
        var leave = useEvent("mouseleave", ref, wrapHandler(onMouseLeave), options);
        return mergeUseEventResults(down, move, up, over, out, enter, leave);
    };
    var useTouchEvents = function (_a, ref, options) {
        var onTouchStart = _a.onTouchStart, onTouchMove = _a.onTouchMove, onTouchEnd = _a.onTouchEnd, onTouchCancel = _a.onTouchCancel;
        var down = useEvent("touchstart", ref, wrapHandler(onTouchStart), options);
        var move = useEvent("touchmove", ref, wrapHandler(onTouchMove), options);
        var up = useEvent("touchend", ref, wrapHandler(onTouchEnd), options);
        var cancel = useEvent("touchcancel", ref, wrapHandler(onTouchCancel), options);
        return mergeUseEventResults(down, move, up, cancel);
    };
    var useNativePointerEvents = function (_a, ref, options) {
        var onPointerDown = _a.onPointerDown, onPointerMove = _a.onPointerMove, onPointerUp = _a.onPointerUp, onPointerCancel = _a.onPointerCancel, onPointerOver = _a.onPointerOver, onPointerOut = _a.onPointerOut, onPointerEnter = _a.onPointerEnter, onPointerLeave = _a.onPointerLeave;
        var down = useEvent("pointerdown", ref, wrapHandler(filterPrimaryPointer(onPointerDown)), options);
        var move = useEvent("pointermove", ref, wrapHandler(onPointerMove), options);
        var up = useEvent("pointerup", ref, wrapHandler(onPointerUp), options);
        var cancel = useEvent("pointercancel", ref, wrapHandler(onPointerCancel), options);
        var over = useEvent("pointerover", ref, wrapHandler(onPointerOver), options);
        var out = useEvent("pointerout", ref, wrapHandler(onPointerOut), options);
        var enter = useEvent("pointerenter", ref, wrapHandler(onPointerEnter), options);
        var leave = useEvent("pointerleave", ref, wrapHandler(onPointerLeave), options);
        return mergeUseEventResults(down, move, up, cancel, over, out, enter, leave);
    };
    var supportsPointerEvents = function () {
        return safeWindow.onpointerdown === null &&
            safeWindow.onpointermove === null &&
            safeWindow.onpointerup === null;
    };
    var supportsTouchEvents = function () {
        return safeWindow.ontouchstart === null &&
            safeWindow.ontouchmove === null &&
            safeWindow.ontouchend === null;
    };
    var supportsMouseEvents = function () {
        return safeWindow.onmousedown === null &&
            safeWindow.onmousemove === null &&
            safeWindow.onmouseup === null;
    };
    var usePointerEvents = function (eventHandlers, ref, options) {
        var onPointerDown = eventHandlers.onPointerDown, onPointerMove = eventHandlers.onPointerMove, onPointerUp = eventHandlers.onPointerUp, onPointerCancel = eventHandlers.onPointerCancel, onPointerOver = eventHandlers.onPointerOver, onPointerOut = eventHandlers.onPointerOut, onPointerEnter = eventHandlers.onPointerEnter, onPointerLeave = eventHandlers.onPointerLeave;
        var mouseEvents = {}, touchEvents = {}, pointerEvents = {};
        if (supportsPointerEvents()) {
            pointerEvents = eventHandlers;
        }
        else if (supportsTouchEvents()) {
            touchEvents = {
                onTouchStart: onPointerDown,
                onTouchMove: onPointerMove,
                onTouchEnd: onPointerUp,
                onTouchCancel: onPointerCancel,
            };
        }
        else if (supportsMouseEvents()) {
            mouseEvents = {
                onMouseDown: onPointerDown,
                onMouseMove: onPointerMove,
                onMouseUp: onPointerUp,
                onMouseOver: onPointerOver,
                onMouseOut: onPointerOut,
                onMouseEnter: onPointerEnter,
                onMouseLeave: onPointerLeave,
            };
        }
        var pointer = useNativePointerEvents(pointerEvents, ref, options);
        var touch = useTouchEvents(touchEvents, ref, options);
        var mouse = useMouseEvents(mouseEvents, ref, options);
        return mergeUseEventResults(pointer, touch, mouse);
    };
    /**
     * A Conditional version of usePointerEvents
     * As opposed to usePointerEvents, this function does not require to pass a target or ref.
     * When calling this without a target, the handlers passed to the function are returned
     * @param eventHandlers
     * @param ref
     * @param options
     */
    var useConditionalPointerEvents = function (eventHandlers, ref, options) {
        var pointerEventsRef;
        var pointerEventsHandlers;
        var emptyRef = React.useRef(null);
        // Because hooks can't be conditional, we need to this dance, instead of just an early return
        if (ref) {
            pointerEventsRef = ref;
            pointerEventsHandlers = eventHandlers;
        }
        else {
            pointerEventsRef = emptyRef;
            pointerEventsHandlers = {};
        }
        usePointerEvents(pointerEventsHandlers, pointerEventsRef, options);
        if (ref) {
            return undefined;
        }
        else {
            return eventHandlers;
        }
    };

    /** @public */
    (function (Point) {
        /** @beta */
        Point.subtract = function (a, b) {
            return { x: a.x - b.x, y: a.y - b.y };
        };
        /** @beta */
        Point.relativeTo = function (idOrElem) {
            var elem;
            var getElem = function () {
                // Caching element here could be leaky because of React lifecycle
                if (elem !== undefined)
                    return elem;
                if (typeof idOrElem === "string") {
                    elem = document.getElementById(idOrElem);
                }
                else {
                    elem = idOrElem;
                }
                return elem;
            };
            return function (_a) {
                var x = _a.x, y = _a.y;
                var localElem = getElem();
                if (!localElem)
                    return undefined;
                var rect = localElem.getBoundingClientRect();
                return {
                    x: x - rect.left - window.scrollX,
                    y: y - rect.top - window.scrollY,
                };
            };
        };
    })(exports.Point || (exports.Point = {}));

    /**
     * @internal
     */
    var MotionPluginContext = React.createContext({});
    /**
     * @internal
     * @internalremarks For now I think this should remain a private API for our own use
     * until we can figure out a nicer way of allowing people to add these
     */
    function MotionPlugins(_a) {
        var children = _a.children, props = __rest(_a, ["children"]);
        var pluginContext = React.useContext(MotionPluginContext);
        var value = React.useRef(__assign({}, pluginContext)).current;
        // Mutative to prevent triggering rerenders in all listening
        // components every time this component renders
        for (var key in props) {
            value[key] = props[key];
        }
        return (React.createElement(MotionPluginContext.Provider, { value: value }, children));
    }

    var isViewportScrollBlocked = false;
    safeWindow.addEventListener("touchmove", function (event) {
        if (isViewportScrollBlocked) {
            event.preventDefault();
        }
    }, { passive: false });
    var blockViewportScroll = function () { return (isViewportScrollBlocked = true); };
    var unblockViewportScroll = function () { return (isViewportScrollBlocked = false); };

    function startDevicePoint(session) {
        return session.pointHistory[0];
    }
    function lastDevicePoint(session) {
        return session.pointHistory[session.pointHistory.length - 1];
    }
    function getVelocity$1(session, timeDelta) {
        var pointHistory = session.pointHistory;
        if (pointHistory.length < 2) {
            return { x: 0, y: 0 };
        }
        var i = pointHistory.length - 1;
        var timestampedPoint = null;
        var lastPoint = lastDevicePoint(session);
        while (i >= 0) {
            timestampedPoint = pointHistory[i];
            if (lastPoint.timestamp - timestampedPoint.timestamp >
                timeDelta * 1000) {
                break;
            }
            i--;
        }
        if (!timestampedPoint) {
            return { x: 0, y: 0 };
        }
        var time = (lastPoint.timestamp - timestampedPoint.timestamp) / 1000;
        if (time === 0) {
            return { x: 0, y: 0 };
        }
        var currentVelocity = {
            x: (lastPoint.x - timestampedPoint.x) / time,
            y: (lastPoint.y - timestampedPoint.y) / time,
        };
        if (currentVelocity.x === Infinity) {
            currentVelocity.x = 0;
        }
        if (currentVelocity.y === Infinity) {
            currentVelocity.y = 0;
        }
        return currentVelocity;
    }
    function usePanGesture(_a, ref) {
        var onPan = _a.onPan, onPanStart = _a.onPanStart, onPanEnd = _a.onPanEnd, onPanSessionStart = _a.onPanSessionStart;
        var session = React.useRef(null);
        var pointer = React.useRef(null);
        var lastMoveEvent = React.useRef(null);
        var lastMoveEventInfo = React.useRef(null);
        var transformPagePoint = React.useContext(MotionPluginContext).transformPagePoint;
        var getPanInfo = function (_a) {
            var point = _a.point;
            var currentPoint = session.current;
            return {
                point: point,
                delta: exports.Point.subtract(point, lastDevicePoint(currentPoint)),
                offset: exports.Point.subtract(point, startDevicePoint(currentPoint)),
                velocity: getVelocity$1(currentPoint, 0.1),
            };
        };
        var transformPoint = function (info) {
            return transformPagePoint
                ? { point: transformPagePoint(info.point) }
                : info;
        };
        var updatePoint = React.useCallback(function () {
            if (!session.current ||
                pointer.current === null ||
                lastMoveEventInfo.current === null ||
                lastMoveEvent.current === null) {
                warning(false, "Pointer move without started session");
                stopPointerMove();
                stopPointerUp();
                unblockViewportScroll();
                return;
            }
            var info = getPanInfo(lastMoveEventInfo.current);
            var point = info.point;
            var timestamp = getFrameData().timestamp;
            session.current.pointHistory.push(__assign({}, point, { timestamp: timestamp }));
            pointer.current.x.set(point.x);
            pointer.current.y.set(point.y);
            if (!session.current.startEvent) {
                if (onPanStart) {
                    onPanStart(lastMoveEvent.current, info);
                }
                session.current.startEvent = lastMoveEvent.current;
            }
            if (onPan) {
                onPan(lastMoveEvent.current, info);
            }
        }, [onPan, onPanStart]);
        var onPointerMove = React.useCallback(function (event, info) {
            lastMoveEvent.current = event;
            lastMoveEventInfo.current = transformPoint(info);
            // because Safari doesn't trigger mouseup event when it's happening above <select> tag
            if (event instanceof MouseEvent && event.buttons === 0) {
                onPointerUp(event, info);
                return;
            }
            // Throttle mouse move event to once per frame
            sync.update(updatePoint, true);
        }, [onPan, onPanStart]);
        var onPointerUp = React.useCallback(function (event, info) {
            cancelSync.update(updatePoint);
            stopPointerMove();
            stopPointerUp();
            if (!session.current || pointer.current === null) {
                warning(false, "Pointer end without started session");
                unblockViewportScroll();
                return;
            }
            if (onPanEnd) {
                onPanEnd(event, getPanInfo(transformPoint(info)));
            }
            session.current = null;
        }, [onPanEnd, onPointerMove]);
        var _b = usePointerEvents({ onPointerUp: onPointerUp }, safeWindow), startPointerUp = _b[0], stopPointerUp = _b[1];
        var _c = usePointerEvents({ onPointerMove: onPointerMove }, safeWindow, { capture: true }), startPointerMove = _c[0], stopPointerMove = _c[1];
        var onPointerDown = React.useCallback(function (event, info) {
            var initialInfo = transformPoint(info);
            var point = initialInfo.point;
            pointer.current = {
                x: motionValue(point.x),
                y: motionValue(point.y),
            };
            var timestamp = getFrameData().timestamp;
            session.current = {
                target: event.target,
                pointHistory: [__assign({}, point, { timestamp: timestamp })],
            };
            if (onPanSessionStart) {
                onPanSessionStart(event, getPanInfo(initialInfo));
            }
            startPointerMove();
            startPointerUp();
        }, [onPointerUp, onPointerMove]);
        React.useEffect(function () {
            return function () {
                stopPointerMove();
                stopPointerUp();
            };
        }, []);
        var handlers = { onPointerDown: onPointerDown };
        if (!(onPan || onPanStart || onPanEnd || onPanSessionStart)) {
            handlers = {};
        }
        return useConditionalPointerEvents(handlers, ref);
    }

    /**
     * Recursively traverse up the tree to check whether the provided child node
     * is the parent or a descendant of it.
     *
     * @param parent - Element to find
     * @param child - Element to test against parent
     */
    var isNodeOrChild = function (parent, child) {
        if (!child) {
            return false;
        }
        else if (parent === child) {
            return true;
        }
        else {
            return isNodeOrChild(parent, child.parentElement);
        }
    };

    var order$1 = ["whileHover", "whileTap", "whileDrag"];
    var getGesturePriority = function (gesture) {
        return order$1.indexOf(gesture) + 1;
    };

    function createLock(name) {
        var lock = null;
        return function () {
            var openLock = function () {
                lock = null;
            };
            if (lock === null) {
                lock = name;
                return openLock;
            }
            return false;
        };
    }
    var globalHorizontalLock = createLock("dragHorizontal");
    var globalVerticalLock = createLock("dragVertical");
    function getGlobalLock(drag) {
        var lock = false;
        if (drag === "y") {
            lock = globalVerticalLock();
        }
        else if (drag === "x") {
            lock = globalHorizontalLock();
        }
        else {
            var openHorizontal_1 = globalHorizontalLock();
            var openVertical_1 = globalVerticalLock();
            if (openHorizontal_1 && openVertical_1) {
                lock = function () {
                    openHorizontal_1();
                    openVertical_1();
                };
            }
            else {
                // Release the locks because we don't use them
                if (openHorizontal_1)
                    openHorizontal_1();
                if (openVertical_1)
                    openVertical_1();
            }
        }
        return lock;
    }

    var tapGesturePriority = getGesturePriority("whileTap");
    /**
     * Explanation:
     *
     * Currently there's a problem where props can change while a gesture is active. In
     * this hook, we have event handlers that are created in `useMemo`, which is sync.
     * The event handlers themselves are activated in `useEvent` with `useEffect`, which
     * is async.
     *
     * Most event handlers are being set like this: onTap={() => {}}
     * This creates a new function every render, thus breaking `useMemo`. When any props
     * are set during a gesture, this creates a new event. We then have a `useEffect` in this
     * hook that, when props change, unsets the previous event handlers. I suspect that at
     * some point, the pointer up event is fired in between the previous one
     * being deactivate and the new one being activated.
     *
     * I attempted to rewrite the hook without `useMemo` at all but encountered the same issue.
     * At some point the `pointerUp` event is being unset and not reapplied until after it fires.
     *
     * By setting props to a ref we maintain longer gesture sessions and don't encounter this funny
     * business, but I'd rather a solution that required less code.
     *
     * @param props
     */
    var usePropsRef = function (props) {
        var propsRef = React.useRef(props);
        for (var key in props) {
            propsRef.current[key] = props[key];
        }
        return propsRef.current;
    };
    function useTapGesture(props, ref) {
        var isTapping = React.useRef(false);
        var onTap = props.onTap, onTapStart = props.onTapStart, onTapCancel = props.onTapCancel, whileTap = props.whileTap, controls = props.controls;
        var propsRef = usePropsRef(props);
        if (whileTap && controls) {
            controls.setOverride(whileTap, tapGesturePriority);
        }
        var handlers = React.useMemo(function () {
            if (!onTap && !onTapStart && !onTapCancel && !whileTap) {
                return {
                    onPointerUp: function () { },
                    onPointerDown: function () { },
                };
            }
            var onPointerUp = function (event, _a) {
                var point = _a.point;
                if (!isTapping.current) {
                    return;
                }
                isTapping.current = false;
                if (controls && propsRef.whileTap) {
                    controls.clearOverride(tapGesturePriority);
                }
                // Check the gesture lock - if we get it, it means no drag gesture is active
                // and we can safely fire the tap gesture.
                var openGestureLock = getGlobalLock(true);
                if (!openGestureLock) {
                    return;
                }
                openGestureLock();
                if (!ref ||
                    !ref.current ||
                    !isNodeOrChild(ref.current, event.target)) {
                    if (propsRef.onTapCancel) {
                        propsRef.onTapCancel(event, { point: point });
                    }
                    return;
                }
                if (propsRef.onTap) {
                    propsRef.onTap(event, { point: point });
                }
            };
            var onPointerDown = function (event, _a) {
                var point = _a.point;
                startPointerUp();
                if (!ref || !ref.current)
                    return;
                isTapping.current = true;
                if (propsRef.onTapStart) {
                    propsRef.onTapStart(event, { point: point });
                }
                if (controls && propsRef.whileTap) {
                    controls.startOverride(tapGesturePriority);
                }
            };
            return { onPointerUp: onPointerUp, onPointerDown: onPointerDown };
        }, [
            onTap === undefined,
            onTapStart === undefined,
            onTapCancel === undefined,
        ]);
        var _a = usePointerEvents({ onPointerUp: handlers.onPointerUp }, safeWindow), startPointerUp = _a[0], stopPointerUp = _a[1];
        React.useEffect(function () { return function () {
            stopPointerUp();
        }; }, [handlers.onPointerUp]);
        return useConditionalPointerEvents({ onPointerDown: handlers.onPointerDown }, ref);
    }

    var hoverPriority = getGesturePriority("whileHover");
    // TODO: Optimisation here is find a way to conditionally add these listeners based on
    // whether we're receiving hover or event listeners
    /**
     *
     * @param props
     * @param ref
     * @internal
     */
    function useHoverGesture(_a, ref) {
        var whileHover = _a.whileHover, onHoverStart = _a.onHoverStart, onHoverEnd = _a.onHoverEnd, controls = _a.controls;
        if (whileHover && controls) {
            controls.setOverride(whileHover, hoverPriority);
        }
        var handlers = React.useMemo(function () {
            var onPointerEnter = function (event) {
                if (onHoverStart)
                    onHoverStart(event);
                if (whileHover && controls) {
                    controls.startOverride(hoverPriority);
                }
            };
            var onPointerLeave = function (event) {
                if (onHoverEnd)
                    onHoverEnd(event);
                if (whileHover && controls) {
                    controls.clearOverride(hoverPriority);
                }
            };
            return { onPointerEnter: onPointerEnter, onPointerLeave: onPointerLeave };
        }, [whileHover, onHoverStart, onHoverEnd, controls]);
        return useConditionalPointerEvents(handlers, ref);
    }

    /**
     * Add pan and tap gesture recognition to an element.
     *
     * @param props - Gesture event handlers
     * @param ref - React `ref` containing a DOM `Element`
     * @public
     */
    function useGestures(props, ref) {
        usePanGesture(props, ref);
        useTapGesture(props, ref);
        useHoverGesture(props, ref);
    }

    var gestureProps = [
        "drag",
        "onPan",
        "onPanStart",
        "onPanEnd",
        "onPanSessionStart",
        "onTap",
        "onTapStart",
        "onTapCancel",
        "whileTap",
        "whileHover",
        "onHoverStart",
        "onHoverEnd",
    ];
    var Gestures = {
        shouldRender: function (props) {
            return gestureProps.some(function (key) { return props.hasOwnProperty(key); });
        },
        component: makeHookComponent(function (_a) {
            var innerRef = _a.innerRef, props = __rest(_a, ["innerRef"]);
            useGestures(props, innerRef);
        }),
    };

    var isRefObject = function (ref) {
        return typeof ref === "object" && ref.hasOwnProperty("current");
    };

    /**
     * A hook to use the window resize listener. In future it might be cool to use `ResizeObserver`
     * but it currently needs to be polyfilled and the size trade-off isn't there for this use-case.
     *
     * TODO: Debounce this bad boy
     *
     * @param onResize
     */
    function useResize(element, onResize) {
        React.useEffect(function () {
            if (!element || !isRefObject(element))
                return;
            safeWindow.addEventListener("resize", onResize);
            return function () { return safeWindow.removeEventListener("resize", onResize); };
        }, [element, onResize]);
    }

    var getBoundingBox = function (ref) {
        return ref.current.getBoundingClientRect();
    };
    var getCurrentOffset = function (point) {
        return point ? point.get() : 0;
    };
    /**
     * Takes a parent Element and a draggable Element and returns pixel-based drag constraints.
     *
     * @param constraintsRef
     * @param draggableRef
     */
    var calculateConstraintsFromDom = function (constraintsRef, draggableRef, point) {
        invariant(constraintsRef.current !== null && draggableRef.current !== null, "If `dragConstraints` is set as a React ref, that ref must be passed to another component's `ref` prop.");
        var parentBoundingBox = getBoundingBox(constraintsRef);
        var draggableBoundingBox = getBoundingBox(draggableRef);
        var top = parentBoundingBox.top -
            draggableBoundingBox.top +
            getCurrentOffset(point.y);
        var left = parentBoundingBox.left -
            draggableBoundingBox.left +
            getCurrentOffset(point.x);
        var constraints = {
            top: top,
            left: left,
            right: parentBoundingBox.width - draggableBoundingBox.width + left,
            bottom: parentBoundingBox.height - draggableBoundingBox.height + top,
        };
        return constraints;
    };
    var flattenConstraints = function (constraints) {
        if (!constraints) {
            return [0, 0, 0, 0];
        }
        else if (isRefObject(constraints)) {
            return [constraints.current];
        }
        else {
            var top_1 = constraints.top, left = constraints.left, bottom = constraints.bottom, right = constraints.right;
            return [top_1, left, bottom, right];
        }
    };
    function shouldDrag(direction, drag, currentDirection) {
        return ((drag === true || drag === direction) &&
            (currentDirection === null || currentDirection === direction));
    }
    var getConstraints = function (axis, _a) {
        var top = _a.top, right = _a.right, bottom = _a.bottom, left = _a.left;
        if (axis === "x") {
            return { min: left, max: right };
        }
        else {
            return { min: top, max: bottom };
        }
    };
    function applyConstraints(axis, value, constraints, dragElastic) {
        var constrainedValue = value instanceof MotionValue ? value.get() : value;
        if (!constraints) {
            return constrainedValue;
        }
        var _a = getConstraints(axis, constraints), min = _a.min, max = _a.max;
        if (min !== undefined && constrainedValue < min) {
            constrainedValue = dragElastic
                ? applyOverdrag(min, constrainedValue, dragElastic)
                : Math.max(min, constrainedValue);
        }
        else if (max !== undefined && constrainedValue > max) {
            constrainedValue = dragElastic
                ? applyOverdrag(max, constrainedValue, dragElastic)
                : Math.min(max, constrainedValue);
        }
        if (value instanceof MotionValue) {
            value.set(constrainedValue);
        }
        return constrainedValue;
    }
    var applyOverdrag = function (origin, current, dragElastic) {
        var dragFactor = typeof dragElastic === "number" ? dragElastic : 0.35;
        return mix(origin, current, dragFactor);
    };
    var bothAxis = function (handler) { return [
        handler("x"),
        handler("y"),
    ]; };
    /**
     * A hook that allows an element to be dragged.
     *
     * @internalremarks
     *
     * TODO:
     *  - When drag momentum animations are running and a `ref` constraints is resized,
     *    everything breaks.
     *
     * @param param
     * @param ref
     * @param values
     * @param controls
     *
     * @internal
     */
    function useDraggable(_a, ref, values, controls) {
        var _b = _a.drag, drag = _b === void 0 ? false : _b, _c = _a.dragDirectionLock, dragDirectionLock = _c === void 0 ? false : _c, _d = _a.dragPropagation, dragPropagation = _d === void 0 ? false : _d, _e = _a.dragConstraints, dragConstraints = _e === void 0 ? false : _e, _f = _a.dragElastic, dragElastic = _f === void 0 ? true : _f, _g = _a.dragMomentum, dragMomentum = _g === void 0 ? true : _g, dragTransition = _a.dragTransition, handlers = __rest(_a, ["drag", "dragDirectionLock", "dragPropagation", "dragConstraints", "dragElastic", "dragMomentum", "dragTransition"]);
        var point = React.useRef({}).current;
        var origin = React.useRef({ x: 0, y: 0 }).current;
        // By keeping a reference to the user-defined drag handlers and referring
        // to that from within our `useMemo`-generated pan handlers, we can ensure
        // we're always referring to the latest ones *without* having to reinitialise
        // the handlers. These are almost guaranteed to be different every render
        // but don't necessitate creating new handlers.
        var dragHandlers = React.useRef(handlers);
        dragHandlers.current = handlers;
        // If `dragConstraints` is a React `ref`, we should resolve the constraints once the
        // component has rendered.
        var constraintsNeedResolution = isRefObject(dragConstraints);
        // If `dragConstraints` is a React `ref`, we need to track changes in its
        // size and update the current draggable position relative to that.
        var prevConstraintsBox = React.useRef({
            width: 0,
            height: 0,
            x: 0,
            y: 0,
        }).current;
        var scalePoint = function () {
            if (!isRefObject(dragConstraints))
                return;
            var constraintsBox = getBoundingBox(dragConstraints);
            var draggableBox = getBoundingBox(ref);
            // Scale a point relative to the transformation of a constraints-providing element.
            var scaleAxisPoint = function (axis, dimension) {
                var pointToScale = point[axis];
                if (!pointToScale)
                    return;
                // Stop any current animations as they bug out if you resize during one
                if (pointToScale.isAnimating()) {
                    pointToScale.stop();
                    recordBoxInfo();
                    return;
                }
                // If the previous dimension was `0` (default), set `scale` to `1` to prevent
                // divide by zero errors.
                var scale = prevConstraintsBox[dimension]
                    ? (constraintsBox[dimension] - draggableBox[dimension]) /
                        prevConstraintsBox[dimension]
                    : 1;
                pointToScale.set(prevConstraintsBox[axis] * scale);
            };
            scaleAxisPoint("x", "width");
            scaleAxisPoint("y", "height");
        };
        useResize(dragConstraints, scalePoint);
        // If our drag constraints are a potentially live bounding box, record its previously-calculated
        // dimensions and the current x/y
        var recordBoxInfo = function (constraints) {
            if (constraints) {
                var right = constraints.right, left = constraints.left, bottom = constraints.bottom, top_2 = constraints.top;
                prevConstraintsBox.width = (right || 0) - (left || 0);
                prevConstraintsBox.height = (bottom || 0) - (top_2 || 0);
            }
            if (point.x)
                prevConstraintsBox.x = point.x.get();
            if (point.y)
                prevConstraintsBox.y = point.y.get();
        };
        var applyConstraintsToPoint = function (constraints) {
            return bothAxis(function (axis) {
                var axisPoint = point[axis];
                axisPoint && applyConstraints(axis, axisPoint, constraints, 0);
            });
        };
        // On mount, if our bounding box is a ref, we need to resolve the constraints
        // and immediately apply them to our point.
        React.useEffect(function () {
            if (!constraintsNeedResolution)
                return;
            var constraints = calculateConstraintsFromDom(dragConstraints, ref, point);
            applyConstraintsToPoint(constraints);
            recordBoxInfo(constraints);
        });
        // Create our handlers for the `pan` gesture.
        var panHandlers = React.useMemo(function () {
            if (!drag)
                return {};
            // We'll use this to determine whether to fire the `onDragEnd` callback.
            var hasDragged = false;
            // Don't start dragging until we've detected a direction.
            var currentDirection = null;
            // This is a reference to the global drag gesture lock, ensuring only one component
            // can "capture" the drag of one or both axes. In some odd circumstances, a re-render
            // has caused reference to this lock to be lost, if we see this appear again it might
            // be safer to move this to a hook-root `ref`.
            var openGlobalLock = null;
            // If `dragConstraints` is set to `false` or `Constraints`, set constraints immediately.
            // Otherwise we'll resolve on mount.
            var resolvedDragConstraints = constraintsNeedResolution
                ? false
                : dragConstraints;
            // Get the `MotionValue` for both draggable axes, or create them if they don't already
            // exist on this component.
            bothAxis(function (axis) {
                if (!shouldDrag(axis, drag, currentDirection))
                    return;
                var axisValue = values.get(axis, 0);
                point[axis] = axisValue;
            });
            // Apply constraints immediately, even before render, if our constraints are a plain object.
            if (resolvedDragConstraints && !constraintsNeedResolution) {
                applyConstraintsToPoint(resolvedDragConstraints);
            }
            // Add additional information to the `PanInfo` object before passing it to drag listeners.
            var convertPanToDrag = function (info) { return (__assign({}, info, { point: {
                    x: point.x ? point.x.get() : 0,
                    y: point.y ? point.y.get() : 0,
                } })); };
            // This function will be used to update each axis point every frame.
            var updatePoint = function (axis, offset) {
                var axisPoint = point[axis];
                // If we're not dragging this axis, do an early return.
                if (!shouldDrag(axis, drag, currentDirection) || !axisPoint) {
                    return;
                }
                hasDragged = true;
                var current = origin[axis] + offset[axis];
                current = applyConstraints(axis, current, resolvedDragConstraints, dragElastic);
                axisPoint.set(current);
            };
            var onPointerDown = function () {
                // Initiate viewport scroll blocking on touch start. This is a very aggressive approach
                // which has come out of the difficulty in us being able to do this once a scroll gesture
                // has initiated in mobile browsers. This means if there's a horizontally-scrolling carousel
                // on a page we can let a user scroll the page itself from it. Ideally what we'd do is
                // trigger this once we've got a scroll direction determined. This approach sort-of worked
                // but if the component was dragged quite far in a single frame page scrolling would initiate.
                // Maybe if we turn the direction lock threshold down.
                blockViewportScroll();
                // Stop any animations on both axis values immediately. This allows the user to throw and catch
                // the component.
                bothAxis(function (axis) {
                    var axisPoint = point[axis];
                    axisPoint && axisPoint.stop();
                });
            };
            var onPanStart = function (event, info) {
                hasDragged = false;
                // Resolve the constraints again in case anything has changed in the meantime.
                if (constraintsNeedResolution) {
                    resolvedDragConstraints = calculateConstraintsFromDom(dragConstraints, ref, point);
                    applyConstraintsToPoint(resolvedDragConstraints);
                }
                // Set point origin and stop any existing animations.
                bothAxis(function (axis) {
                    var axisPoint = point[axis];
                    if (!axisPoint)
                        return;
                    origin[axis] = axisPoint.get();
                    axisPoint.stop();
                });
                // Attempt to grab the global drag gesture lock.
                if (!dragPropagation) {
                    openGlobalLock = getGlobalLock(drag);
                    if (!openGlobalLock) {
                        return;
                    }
                }
                currentDirection = null;
                // Alert listeners that dragging has started.
                var onDragStart = dragHandlers.current.onDragStart;
                onDragStart && onDragStart(event, convertPanToDrag(info));
            };
            var onPan = function (event, info) {
                // If we didn't successfully receive the gesture lock, early return.
                if (!dragPropagation && !openGlobalLock) {
                    return;
                }
                var offset = info.offset;
                // Attempt to detect drag direction if directionLock is true
                if (dragDirectionLock && currentDirection === null) {
                    currentDirection = getCurrentDirection(offset);
                    // If we've successfully set a direction, notify listener
                    if (currentDirection !== null) {
                        var onDirectionLock = dragHandlers.current.onDirectionLock;
                        onDirectionLock && onDirectionLock(currentDirection);
                    }
                    return;
                }
                updatePoint("x", offset);
                updatePoint("y", offset);
                var onDrag = dragHandlers.current.onDrag;
                onDrag && onDrag(event, convertPanToDrag(info));
            };
            var onPanEnd = function (event, info) {
                unblockViewportScroll();
                var velocity = info.velocity;
                if (!dragPropagation && openGlobalLock) {
                    openGlobalLock();
                }
                else if (!openGlobalLock) {
                    return;
                }
                if (!hasDragged)
                    return;
                // If we have `dragMomentum` defined, initiate momentum animations for both axis.
                if (dragMomentum) {
                    var momentumAnimations = bothAxis(function (axis) {
                        var _a;
                        if (!shouldDrag(axis, drag, currentDirection)) {
                            return;
                        }
                        var transition = resolvedDragConstraints
                            ? getConstraints(axis, resolvedDragConstraints)
                            : {};
                        return controls.start((_a = {},
                            _a[axis] = 0,
                            // TODO: It might be possible to allow `type` animations to be set as
                            // Popmotion animations as well as strings. Then people could define their own
                            // and it'd open another route for us to code-split.
                            _a.transition = __assign({ type: "inertia", velocity: velocity[axis], bounceStiffness: 200, bounceDamping: 40, timeConstant: 750, restDelta: 1 }, dragTransition, transition),
                            _a));
                    });
                    // Run all animations and then resolve the new drag constraints.
                    Promise.all(momentumAnimations).then(function () {
                        recordBoxInfo(resolvedDragConstraints);
                        scalePoint();
                        var onDragTransitionEnd = dragHandlers.current.onDragTransitionEnd;
                        onDragTransitionEnd && onDragTransitionEnd();
                    });
                }
                else {
                    recordBoxInfo(resolvedDragConstraints);
                }
                var onDragEnd = dragHandlers.current.onDragEnd;
                onDragEnd && onDragEnd(event, convertPanToDrag(info));
            };
            return {
                onPanStart: onPanStart,
                onPan: onPan,
                onPanEnd: onPanEnd,
                onPointerDown: onPointerDown,
            };
        }, [
            drag,
            dragDirectionLock,
            dragPropagation,
            dragElastic,
            dragMomentum
        ].concat(flattenConstraints(dragConstraints), [
            dragTransition,
        ]));
        usePanGesture(panHandlers, ref);
        usePointerEvents({ onPointerDown: panHandlers.onPointerDown }, ref);
    }
    /**
     * Based on an x/y offset determine the current drag direction. If both axis' offsets are lower
     * than the provided threshold, return `null`.
     *
     * @param offset - The x/y offset from origin.
     * @param lockThreshold - (Optional) - the minimum absolute offset before we can determine a drag direction.
     */
    function getCurrentDirection(offset, lockThreshold) {
        if (lockThreshold === void 0) { lockThreshold = 10; }
        var direction = null;
        if (Math.abs(offset.y) > lockThreshold) {
            direction = "y";
        }
        else if (Math.abs(offset.x) > lockThreshold) {
            direction = "x";
        }
        return direction;
    }

    var Drag = {
        shouldRender: function (props) { return !!props.drag; },
        component: makeHookComponent(function (_a) {
            var innerRef = _a.innerRef, values = _a.values, controls = _a.controls, props = __rest(_a, ["innerRef", "values", "controls"]);
            return useDraggable(props, innerRef, values, controls);
        }),
    };

    function isCSSVariable(value) {
        return typeof value === "string" && value.startsWith("var(--");
    }
    /**
     * Parse Framer's special CSS variable format into a CSS token and a fallback.
     *
     * ```
     * `var(--foo, #fff)` => [`--foo`, '#fff']
     * ```
     *
     * @param current
     */
    var cssVariableRegex = /var\((--[a-zA-Z0-9-_]+),? ?([a-zA-Z0-9 ()%#.,-]+)?\)/;
    function parseCSSVariable(current) {
        var match = cssVariableRegex.exec(current);
        if (!match)
            return [,];
        var token = match[1], fallback = match[2];
        return [token, fallback];
    }
    var maxDepth = 4;
    function getVariableValue(current, element, depth) {
        if (depth === void 0) { depth = 1; }
        invariant(depth <= maxDepth, "Max CSS variable fallback depth detected in property \"" + current + "\". This may indicate a circular fallback dependency.");
        var _a = parseCSSVariable(current), token = _a[0], fallback = _a[1];
        // No CSS variable detected
        if (!token)
            return;
        // Attempt to read this CSS variable off the element
        var resolved = window.getComputedStyle(element).getPropertyValue(token);
        if (resolved) {
            return resolved;
        }
        else if (isCSSVariable(fallback)) {
            // The fallback might itself be a CSS variable, in which case we attempt to resolve it too.
            return getVariableValue(fallback, element, depth + 1);
        }
        else {
            return fallback;
        }
    }
    /**
     * Resolve CSS variables from
     *
     * @internal
     */
    function resolveCSSVariables(values, ref, _a, transitionEnd) {
        var target = __rest(_a, []);
        var element = ref.current;
        if (!(element instanceof HTMLElement))
            return { target: target, transitionEnd: transitionEnd };
        // If `transitionEnd` isn't `undefined`, clone it. We could clone `target` and `transitionEnd`
        // only if they change but I think this reads clearer and this isn't a performance-critical path.
        if (transitionEnd) {
            transitionEnd = __assign({}, transitionEnd);
        }
        // Go through existing `MotionValue`s and ensure any existing CSS variables are resolved
        values.forEach(function (value) {
            var current = value.get();
            if (!isCSSVariable(current))
                return;
            var resolved = getVariableValue(current, element);
            if (resolved)
                value.set(resolved);
        });
        // Cycle through every target property and resolve CSS variables. Currently
        // we only read single-var properties like `var(--foo)`, not `calc(var(--foo) + 20px)`
        for (var key in target) {
            var current = target[key];
            if (!isCSSVariable(current))
                continue;
            var resolved = getVariableValue(current, element);
            if (!resolved)
                continue;
            // Clone target if it hasn't already been
            target[key] = resolved;
            // If the user hasn't already set this key on `transitionEnd`, set it to the unresolved
            // CSS variable. This will ensure that after the animation the component will reflect
            // changes in the value of the CSS variable.
            if (transitionEnd && transitionEnd[key] === undefined) {
                transitionEnd[key] = current;
            }
        }
        return { target: target, transitionEnd: transitionEnd };
    }

    var positionalKeys = new Set([
        "width",
        "height",
        "top",
        "left",
        "right",
        "bottom",
        "x",
        "y",
    ]);
    var isPositionalKey = function (key) { return positionalKeys.has(key); };
    var hasPositionalKey = function (target) {
        return Object.keys(target).some(isPositionalKey);
    };
    var setAndResetVelocity = function (value, to) {
        // Looks odd but setting it twice doesn't render, it'll just
        // set both prev and current to the latest value
        value.set(to, false);
        value.set(to);
    };
    var isNumOrPxType = function (v) {
        return v === number || v === px;
    };
    var BoundingBoxDimension;
    (function (BoundingBoxDimension) {
        BoundingBoxDimension["width"] = "width";
        BoundingBoxDimension["height"] = "height";
        BoundingBoxDimension["left"] = "left";
        BoundingBoxDimension["right"] = "right";
        BoundingBoxDimension["top"] = "top";
        BoundingBoxDimension["bottom"] = "bottom";
    })(BoundingBoxDimension || (BoundingBoxDimension = {}));
    var getPosFromMatrix = function (matrix, pos) {
        return parseFloat(matrix.split(", ")[pos]);
    };
    var getTranslateFromMatrix = function (pos2, pos3) { return function (_bbox, _a) {
        var transform = _a.transform;
        if (transform === "none" || !transform)
            return 0;
        var matrix3d = transform.match(/^matrix3d\((.+)\)$/);
        if (matrix3d) {
            return getPosFromMatrix(matrix3d[1], pos3);
        }
        else {
            var matrix = transform.match(/^matrix\((.+)\)$/);
            return getPosFromMatrix(matrix[1], pos2);
        }
    }; };
    var positionalValues = {
        // Dimensions
        width: function (_a) {
            var width = _a.width;
            return width;
        },
        height: function (_a) {
            var height = _a.height;
            return height;
        },
        top: function (_bbox, _a) {
            var top = _a.top;
            return parseFloat(top);
        },
        left: function (_bbox, _a) {
            var left = _a.left;
            return parseFloat(left);
        },
        bottom: function (_a, _b) {
            var height = _a.height;
            var top = _b.top;
            return parseFloat(top) + height;
        },
        right: function (_a, _b) {
            var width = _a.width;
            var left = _b.left;
            return parseFloat(left) + width;
        },
        // Transform
        x: getTranslateFromMatrix(4, 13),
        y: getTranslateFromMatrix(5, 14),
    };
    var convertChangedValueTypes = function (target, values, ref, changedKeys) {
        var element = ref.current;
        var elementStyler = index(element);
        var originBbox = element.getBoundingClientRect();
        var elementComputedStyle = getComputedStyle(element);
        var top = elementComputedStyle.top, left = elementComputedStyle.left, bottom = elementComputedStyle.bottom, right = elementComputedStyle.right, transform = elementComputedStyle.transform;
        var originComputedStyle = { top: top, left: left, bottom: bottom, right: right, transform: transform };
        // Apply the latest values (as set in checkAndConvertChangedValueTypes)
        elementStyler.render();
        var targetBbox = element.getBoundingClientRect();
        changedKeys.forEach(function (key) {
            // Restore styles to their **calculated computed style**, not their actual
            // originally set style. This allows us to animate between equivalent pixel units.
            var value = values.get(key);
            setAndResetVelocity(value, positionalValues[key](originBbox, originComputedStyle));
            target[key] = positionalValues[key](targetBbox, elementComputedStyle);
        });
        // Reapply original values
        elementStyler.render();
        return target;
    };
    var checkAndConvertChangedValueTypes = function (values, ref, target, transitionEnd) {
        if (transitionEnd === void 0) { transitionEnd = {}; }
        target = __assign({}, target);
        transitionEnd = __assign({}, transitionEnd);
        var targetPositionalKeys = Object.keys(target).filter(isPositionalKey);
        var changedValueTypeKeys = targetPositionalKeys.reduce(function (acc, key) {
            var value = values.get(key);
            if (!value)
                return acc;
            var from = value.get();
            var to = target[key];
            var fromType = getDimensionValueType(from);
            var toType;
            // TODO: The current implementation of this basically throws an error
            // if you try and do value conversion via keyframes. There's probably
            // a way of doing this but the performance implications would need greater scrutiny,
            // as it'd be doing multiple resize-remeasure operations.
            if (isKeyframesTarget(to)) {
                var numKeyframes = to.length;
                for (var i = to[0] === null ? 1 : 0; i < numKeyframes; i++) {
                    if (!toType) {
                        toType = getDimensionValueType(to[i]);
                        invariant(toType === fromType ||
                            (isNumOrPxType(fromType) &&
                                isNumOrPxType(toType)), "Keyframes must be of the same dimension as the current value");
                    }
                    else {
                        invariant(getDimensionValueType(to[i]) === toType, "All keyframes must be of the same type");
                    }
                }
            }
            else {
                toType = getDimensionValueType(to);
            }
            if (fromType !== toType) {
                // If they're both just number or px, convert them both to numbers rather than
                // relying on resize/remeasure to convert (which is wasteful in this situation)
                if (isNumOrPxType(fromType) && isNumOrPxType(toType)) {
                    var current = value.get();
                    if (typeof current === "string") {
                        value.set(parseFloat(current));
                    }
                    if (typeof to === "string") {
                        target[key] = parseFloat(to);
                    }
                    else if (Array.isArray(to) && toType === px) {
                        target[key] = to.map(parseFloat);
                    }
                }
                else {
                    acc.push(key);
                    transitionEnd[key] =
                        transitionEnd[key] !== undefined
                            ? transitionEnd[key]
                            : target[key];
                    setAndResetVelocity(value, to);
                }
            }
            return acc;
        }, []);
        return changedValueTypeKeys.length
            ? {
                target: convertChangedValueTypes(target, values, ref, changedValueTypeKeys),
                transitionEnd: transitionEnd,
            }
            : { target: target, transitionEnd: transitionEnd };
    };
    /**
     * Convert value types for x/y/width/height/top/left/bottom/right
     *
     * Allows animation between `'auto'` -> `'100%'` or `0` -> `'calc(50% - 10vw)'`
     *
     * @param values
     * @param ref
     * @param target
     * @param transitionEnd
     * @internal
     */
    function unitConversion(values, ref, target, transitionEnd) {
        return hasPositionalKey(target)
            ? checkAndConvertChangedValueTypes(values, ref, target, transitionEnd)
            : { target: target, transitionEnd: transitionEnd };
    }

    var parseDomVariant = function (values, ref) {
        return function (target, transitionEnd) {
            var resolved = resolveCSSVariables(values, ref, target, transitionEnd);
            target = resolved.target;
            transitionEnd = resolved.transitionEnd;
            return unitConversion(values, ref, target, transitionEnd);
        };
    };

    /**
     * Maintain a list of event handlers. Emotion's `is-prop-valid` doesn't whitelist every event
     * handler, it just pattern-matches `onX`. So we can manually strip out ours.
     */
    var eventHandlers = new Set([
        "onAnimationComplete",
        "onUpdate",
        "onDragStart",
        "onDrag",
        "onDragEnd",
        "onDirectionLock",
        "onDragTransitionEnd"
    ].concat(gestureProps));
    /**
     * Returns an object of only valid props. We remove invalid DOM props according to
     * Emotion's `isPropValid`, and any of our own event handlers. This was necessary when
     * we passed all a component's props through to dynamic variant resolvers, but since
     * moving that logic to custom it's possible for us to ditch this and save some filesize/performance
     * by simply destructuring out the props that we use.
     */
    var validProps = function (props) {
        var valid = {};
        for (var key in props) {
            if (index$2(key) && !eventHandlers.has(key)) {
                valid[key] = props[key];
            }
        }
        return valid;
    };
    /**
     * Create a configuration for `motion` components that provides DOM-specific functionality.
     *
     * @internal
     */
    function createDomMotionConfig(Component) {
        var isDOM = typeof Component === "string";
        var isSVG = isDOM && svgElements.indexOf(Component) !== -1;
        /**
         * Create a component that renders the DOM element. This step of indirection
         * could probably be removed at this point, and `createElement` could be moved
         * to the `activeComponents.push`.
         */
        var RenderComponent = function (_a) {
            var innerRef = _a.innerRef, style = _a.style, values = _a.values, isStatic = _a.isStatic, componentProps = _a.componentProps;
            var forwardProps = isDOM ? validProps(componentProps) : componentProps;
            return React.createElement(Component, __assign({}, forwardProps, { ref: innerRef, style: isSVG ? style : buildStyleAttr(values, style, isStatic) }));
        };
        return {
            /**
             * This hook gets used by the `motion` component
             *
             * Each functionality component gets provided the `ref`, animation controls and the `MotionValuesMap`
             * generated for that component, as well as all the `props` passed to it by the user.
             *
             * The pattern used to determine whether to load and use each piece of functionality is
             * consistent (should render? Then push component) and could be used to extend functionality.
             *
             * By exposing a mutable piece of memory via an API like `extendMotionComponent` we could
             * allow users to add `FunctionalComponentDefinition`s. This would allow us to offer file size
             * reductions by shipping an entry point that doesn't load gesture and drag functionality, and
             * also offer a way for users to develop plugins/other functionality.
             *
             * For user-defined functionality we'd need to allow
             *  1) User-defined prop typing (extending `P`)
             *  2) User-defined "clean props" that removes their plugin's props before being passed to the DOM.
             */
            useFunctionalityComponents: function (props, values, controls, ref, style, isStatic) {
                var activeComponents = [];
                if (!isStatic && Drag.shouldRender(props)) {
                    activeComponents.push(React.createElement(Drag.component, __assign({}, props, { values: values, controls: controls, innerRef: ref, key: "drag" })));
                }
                if (!isStatic && Gestures.shouldRender(props)) {
                    activeComponents.push(React.createElement(Gestures.component, __assign({}, props, { values: values, controls: controls, innerRef: ref, key: "gestures" })));
                }
                activeComponents.push(React.createElement(RenderComponent, { componentProps: props, values: values, controls: controls, innerRef: ref, style: style, isStatic: isStatic, key: "renderComponent" }));
                return activeComponents;
            },
            getValueControlsConfig: function (ref, values) {
                return {
                    values: values,
                    readValueFromSource: function (key) {
                        return index(ref.current).get(key);
                    },
                    // TODO: This is a good second source of plugins. This function contains the CSS variable
                    // and unit conversion support. These functions share a common signature. We could make another
                    // API for adding these and extend to support stuff like FLIP
                    makeTargetAnimatable: parseDomVariant(values, ref),
                };
            },
        };
    }

    var htmlMotionComponents = htmlElements.reduce(function (acc, Component) {
        acc[Component] = createMotionComponent(createDomMotionConfig(Component));
        return acc;
    }, {});
    var svgMotionComponents = svgElements.reduce(function (acc, Component) {
        acc[Component] = createMotionComponent(createDomMotionConfig(Component));
        return acc;
    }, {});
    /**
     * HTML & SVG components, optimised for use with gestures and animation. These can be used as
     * drop-in replacements for any HTML & SVG component, all CSS & SVG properties are supported.
     *
     * @internalremarks
     *
     * I'd like to make it possible for these to be loaded "on demand" - to reduce bundle size by only
     * including HTML/SVG stylers, animation and/or gesture support when necessary.
     *
     * ```jsx
     * <motion.div animate={{ x: 100 }} />
     *
     * <motion.p animate={{ height: 200 }} />
     *
     * <svg><motion.circle r={10} animate={{ r: 20 }} /></svg>
     * ```
     *
     * @public
     */
    var motion = __assign({ custom: function (Component) {
            return createMotionComponent(createDomMotionConfig(Component));
        } }, htmlMotionComponents, svgMotionComponents);

    /**
     * Creates a `MotionValue` to track the state and velocity of a value.
     *
     * Usually, these are created automatically. For advanced use-cases, like use with `useTransform`, you can create `MotionValue`s externally and pass them into the animated component via the `style` prop.
     *
     * ```jsx
     * export function MyComponent() {
     *   const scale = useMotionValue(1)
     *
     *   return <Frame style={{ scale: scale }} />
     * }
     * ```
     *
     * @param initial - The initial state.
     *
     * @public
     */
    function useMotionValue(initial) {
        return useConstant(function () { return motionValue(initial); });
    }

    /**
     * If the provided value is a MotionValue, this returns the actual value, otherwise just the value itself
     *
     * TODO: Remove and move to library
     *
     * @internal
     */
    function unwrapMotionValue(value) {
        var unwrappedValue = value instanceof MotionValue ? value.get() : value;
        return isCustomValue(unwrappedValue)
            ? unwrappedValue.toValue()
            : unwrappedValue;
    }

    function useCustomValue(parent, getTransformer, comparitor) {
        var value = React.useRef(null);
        return React.useMemo(function () {
            if (value.current)
                value.current.destroy();
            value.current = parent.addChild({ transformer: getTransformer() });
            return value.current;
        }, comparitor);
    }

    var isCustomValueType = function (v) {
        return typeof v === "object" && v.mix;
    };
    var getMixer$1 = function (v) { return (isCustomValueType(v) ? v.mix : undefined); };
    function transform() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var useImmediate = !Array.isArray(args[0]);
        var argOffset = useImmediate ? 0 : -1;
        var inputValue = args[0 + argOffset];
        var inputRange = args[1 + argOffset];
        var outputRange = args[2 + argOffset];
        var options = args[3 + argOffset];
        var interpolator = interpolate$$1(inputRange, outputRange, __assign({ mixer: getMixer$1(outputRange[0]) }, options));
        return useImmediate ? interpolator(inputValue) : interpolator;
    }

    var isTransformer = function (v) {
        return typeof v === "function";
    };
    var noop$1 = function () { return function (v) { return v; }; };
    function useTransform(value, customTransform, to, options) {
        var comparitor = [value];
        var transformer = noop$1;
        if (isTransformer(customTransform)) {
            transformer = function () { return customTransform; };
        }
        else if (Array.isArray(to)) {
            var from_1 = customTransform;
            transformer = function () { return transform(from_1, to, options); };
            comparitor = [value, from_1.join(","), to.join(",")];
        }
        return useCustomValue(value, transformer, comparitor);
    }

    var scrollX = motionValue(0);
    var scrollY = motionValue(0);
    var scrollXProgress = motionValue(0);
    var scrollYProgress = motionValue(0);
    var setProgress = function (offset, maxOffset, value) {
        value.set(!maxOffset || !offset ? 0 : offset / maxOffset);
    };
    var hasEventListener = false;
    var addScrollListener = function () {
        hasEventListener = true;
        if (typeof window === "undefined")
            return;
        var updateScrollValues = function () {
            var xOffset = window.pageXOffset;
            var yOffset = window.pageYOffset;
            // Set absolute positions
            scrollX.set(xOffset);
            scrollY.set(yOffset);
            // Set 0-1 progress
            setProgress(xOffset, document.body.clientWidth - window.innerWidth, scrollXProgress);
            setProgress(yOffset, document.body.clientHeight - window.innerHeight, scrollYProgress);
        };
        updateScrollValues();
        window.addEventListener("resize", updateScrollValues);
        window.addEventListener("scroll", updateScrollValues, { passive: true });
    };
    var viewportMotionValues = {
        scrollX: scrollX,
        scrollY: scrollY,
        scrollXProgress: scrollXProgress,
        scrollYProgress: scrollYProgress,
    };
    /**
     * Provides a `MotionValue` that updates when the viewport scrolls.
     * Returns the following four values.
     *
     * - `scrollX` — Horizontal scroll distance in pixels.
     * - `scrollY` — Vertical scroll distance in pixels.
     * - `scrollXProgress` — Horizontal scroll distance between `0` and `1`.
     * - `scrollYProgress` — Vertical scroll distance between `0` and `1`.
     *
     *
     * ```jsx
     * import * as React from "react"
     * import {
     *   Frame,
     *   useViewportScroll,
     *   useTransform
     * } from "framer"
     *
     * export function MyComponent() {
     *   const { scrollYProgress } = useViewportScroll()
     *   return <Frame style={{ scaleX: scrollYProgress }} />
     * }
     * ```
     *
     * @public
     */
    function useViewportScroll() {
        if (!hasEventListener) {
            addScrollListener();
        }
        return viewportMotionValues;
    }

    /**
     * The `useAnimation` hook returns `AnimationControls`, which can be used to manually start, stop and sequence animations on one or more Frames.
     *
     * The returned `AnimationControls` should be passed as the `animate` property of the `Frame`. It can be passed to any number of Frames. These Frames can be animated with the `start` method. It accepts the same properties as the `animate` property.
     *
     * ```jsx
     *
     * import * as React from "react"
     * import { Frame, useAnimation } from "framer"
     *
     * export function MyComponent(props) {
     *    const controls = useAnimation()
     *
     *    controls.start({
     *        x: 100,
     *        transition: { duration: 0.5 },
     *    })
     *
     *    return <Frame animate={controls} />
     * }
     *
     * ```
     *
     * @returns Animation controller with `start` and `stop` methods
     *
     * @public
     */
    function useAnimation() {
        var animationControls$$1 = useConstant(function () { return new AnimationControls(); });
        React.useEffect(function () {
            animationControls$$1.mount();
            return function () { return animationControls$$1.unmount(); };
        }, []);
        return animationControls$$1;
    }

    /**
     * Makes an animated version of `useState`.
     *
     * @remarks
     *
     * When the returned state setter is called, values will be animated to their new target.
     *
     * This allows the animation of arbitrary React components.
     *
     * **Note:** When animating DOM components, it's always preferable to use the `animate` prop, as Framer
     * will bypass React's rendering cycle with one optimised for 60fps motion. This Hook is specifically
     * for animating props on arbitrary React components, or for animating text content.
     *
     * ```jsx
     * const [state, setState] = useAnimatedState({ percentage: 0 })
     *
     * return (
     *   <Graph
     *     percentage={state.percentage}
     *     onTap={() => setState({ percentage: 50 })}
     *   />
     * )
     * ```
     *
     * @internalremarks
     *
     * TODO:
     * - Make hook accept a typed version of Target that accepts any value (not just DOM values)
     * - Allow hook to accept single values. ie useAnimatedState(0)
     * - Allow providing MotionValues via initialState.
     *
     * @beta
     */
    function useAnimatedState(initialState) {
        var _a = React.useState(initialState), animationState = _a[0], onUpdate = _a[1];
        var config = useConstant(function () { return ({ onUpdate: onUpdate }); });
        var values = useMotionValues(config);
        var controls = useValueAnimationControls({
            values: values,
            readValueFromSource: function (key) { return animationState[key]; },
        }, {}, false);
        var startAnimation = useConstant(function () { return function (animationDefinition) {
            return controls.start(animationDefinition);
        }; });
        React.useEffect(function () {
            values.mount();
            return function () { return values.unmount(); };
        });
        return [animationState, startAnimation];
    }

    /**
     * Cycles through a series of visual properties. Can be used to toggle between or cycle through animations. It works similar to `useState` in React. It is provided an initial array of possible states, and returns an array of two arguments.
     *
     *
     * ```jsx
     * import * as React from "react"
     * import { Frame, useCycle } from "framer"
     *
     * export function MyComponent() {
     *   const [x, cycleX] = useCycle(0, 50, 100)
     *
     *   return (
     *     <Frame
     *       animate={{ x: x }}
     *       onTap={() => cycleX()}
     *      />
     *    )
     * }
     * ```
     *
     * @param items - items to cycle through
     * @returns [currentState, cycleState]
     *
     * @public
     */
    function useCycle() {
        var items = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            items[_i] = arguments[_i];
        }
        // TODO: After Framer X beta, remove this warning
        warning(items.length > 1, "useCycle syntax has changed. `useCycle([0, 1, 2])` becomes `useCycle(0, 1, 2)`");
        var index = React.useRef(0);
        var _a = React.useState(items[index.current]), item = _a[0], setItem = _a[1];
        return [
            item,
            function (next) {
                index.current =
                    typeof next !== "number"
                        ? wrap$1(0, items.length, index.current + 1)
                        : next;
                setItem(items[index.current]);
            },
        ];
    }

    exports.motion = motion;
    exports.useExternalRef = useExternalRef;
    exports.createMotionComponent = createMotionComponent;
    exports.useMotionValue = useMotionValue;
    exports.MotionValue = MotionValue;
    exports.motionValue = motionValue;
    exports.unwrapMotionValue = unwrapMotionValue;
    exports.useTransform = useTransform;
    exports.useViewportScroll = useViewportScroll;
    exports.AnimationControls = AnimationControls;
    exports.animationControls = animationControls;
    exports.useAnimation = useAnimation;
    exports.useAnimatedState = useAnimatedState;
    exports.useTapGesture = useTapGesture;
    exports.usePanGesture = usePanGesture;
    exports.useGestures = useGestures;
    exports.useCycle = useCycle;
    exports.transform = transform;
    exports.MotionPlugins = MotionPlugins;
    exports.MotionPluginContext = MotionPluginContext;
    exports.MotionContext = MotionContext;
    exports.safeWindow = safeWindow;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
